/**
 * 简单组件包装器
 * 将页面元素包装为组件，方便按键操作
 * @dependent:/swatapps/ape/evm.js
 * @version:v1.0
 * @author:wuqq@evmtv.com
 * @createddate:2016-03-14
 *
 * 2016-05-05 v1.1
 * 1.增加动态生成页面元素，可以enabled=false设置元素隐藏或者样式
 */

(function(window){
	function CompDecorator(options){
		//类只能被实例化
		if (this === evm.ext.comp || this === window) {
			return new arguments.callee(options);
		}

		//默认对象的配置
		var defaults = {
			//页面元素的数据包装对象
			//对象结构：[{id:id1,enabled:true},{id:id2,enabled,false}]
			//id:表示页面元素的id属性，enabled表示元素是否可以被选择
			data:[],
			//元素排列方向,V-垂直排列，H-水平排列
			direction:'H',
			//当前焦点元素在元素组中的索引值
			activeIndex:0,
			//页面元素是动态生成的
			isDynamic:false,
			//用户自定义方法，可以响应对象的操作方法
			//@param {string} method 对象操作方法名称
			//@param {object} args 传递的参数对象
			userEvent:function(){},
			//用于表示组件是否可被选中状态
			enabled:false
		};

		evm.extend(defaults, options);
		evm.extend(this, defaults);
		//继承基类
		evm.ext.comp.BaseComponent.call(this, defaults);
	}
	evm.extend(CompDecorator.prototype,{
		_trigger:function(method,obj){
			if(evm.isFunction(this.userEvent)){
				var args = obj || {};
				if(args.activeIndex == null){
					args.activeIndex = this.activeIndex;
				}
				if(!args.activeData){
					args.activeData = this.getActiveData();
				}
				if(!args.activeElm){
					args.activeElm = this.getActiveElm();
				}
				return this.userEvent(method,args);
			}
		},
		_findValidIndex:function(startIndex,keyAlias){
			var validIndex,
					i = startIndex,
					len = this.data.length;
			if((this.direction === 'V' && keyAlias === 'DOWN') || this.direction === 'H' && keyAlias === 'RIGHT'){
				for(; i < len; i++){
					if(this.data[i].enabled){
						return i;
					}
				}
			}
			else if(this.direction === 'V' && keyAlias === 'UP' || this.direction === 'H' && keyAlias === 'LEFT'){
				for(; i >= 0; i--){
					if(this.data[i].enabled){
						return i;
					}
				}
			}

			return validIndex;
		},
		_move:function(keyAlias,myEvent){
			var ai = this.activeIndex;
			var max = this.data.length;

			if(this.direction === 'H'){
				if(keyAlias === 'LEFT'){
					ai--;
				}
				else if(keyAlias === 'RIGHT'){
					ai++;
				}
			}
			else if(this.direction === 'V'){
				if(keyAlias === 'UP'){
					ai--;
				}
				else if(keyAlias === 'DOWN'){
					ai++;
				}
			}

			ai = this._findValidIndex(ai,keyAlias);

			if(ai == null){
				this.onWidgetBlur(keyAlias);
			}
			else if(ai !== this.activeIndex){
				this.onBlur();
				this.activeIndex = ai;
				this.onFocus();
			}

			return;
		},
		action:function(keyAlias,myEvent){
			switch(keyAlias){
				case 'UP':
				case 'DOWN':
				case 'RIGHT':
				case 'LEFT':
						this._move(keyAlias,myEvent);
					break;
				case 'SELECT':
						this.onSelect();
					break;
			}
		},
		init:function(){
			this._trigger('onPreInit');
			for(var i = 0;i < this.data.length;i++){
				if(this.data[i].enabled){
					this.enabled = true;
					break;
				}
			}
			this._trigger('onInitComplete');
		},
		onFocus:function(){
			var activeElm = this.getActiveElm();
			if(activeElm){
				evm.addClass(activeElm,'focus');

				this._trigger('onFocus',{
					activeElm:activeElm
				});
			}
		},
		onBlur:function(){
			var activeElm = this.getActiveElm();
			if(activeElm){
				evm.removeClass(activeElm,'focus');

				this._trigger('onBlur',{
					activeElm:activeElm
				});
			}
		},
		onSelect:function(){
			var activeElm = this.getActiveElm();
			if(activeElm){
				this._trigger('onSelect',{
					activeElm:activeElm
				});
			}
		},
		onWidgetBlur:function(keyAlias){
			if(this._trigger('onWidgetBlur',{alias:keyAlias})){
				this.onBlur();
			}
		},
		getActiveData:function(){
			var activeData = this.data[this.activeIndex] || null;
			return activeData;
		},
		getActiveElm:function(){
			var activeData = this.getActiveData();
			var activeElm;
			if(activeData && activeData.el && !this.isDynamic){
				return activeData.el;
			}
			else if(activeData && activeData.id){
				activeElm = evm.$(activeData.id);
				if(activeElm){
					activeData.el = activeElm;
					return activeElm;
				}
			}
			return null;
		},
		/**
		 * 设置元素不能选中
		 * @param id {string}or{Array}
		 */
		setDisabled:function(ids){
			if(evm.isString(ids)){
				ids = [ids];
			}
			if(!(ids instanceof Array)){
				throw new Error('setDisabled:传入参数必须为字符串或者数组!');
			}
			var i = 0,
				len = ids.length,
				j = 0,
				jLen = this.data.length;
			for(; i < len; i++){
				for(; j < jLen; j++){
					if(this.data[j].id === ids[i]){
						this.data[j].enabled = false;
						evm.addClass(evm.$(this.data[j].id),'disabled');
					}
					else{
						evm.removeClass(evm.$(this.data[j].id),'disabled');
					}
				}
			}
		},
		/**
		 * 设置元素隐藏
		 * @param id {string}or{Array}
		 */
		hideElm:function(ids){
			if(evm.isString(ids)){
				ids = [ids];
			}
			if(!(ids instanceof Array)){
				throw new Error('setDisabled:传入参数必须为字符串或者数组!');
			}
			var i = 0,
					len = ids.length,
					j = 0,
					jLen = this.data.length;
			for(; i < len; i++){
				for(; j < jLen; j++){
					if(this.data[j].id === ids[i]){
						var elm = evm.$(ids[i]);
						if(elm){
							evm.css(elm,{
								display:'none'
							});
						}
					}
				}
			}
		},
		/**
		 * 设置元素显示
		 * @param id {string}or{Array}
		 */
		showElm:function(ids){
			if(evm.isString(ids)){
				ids = [ids];
			}
			if(!(ids instanceof Array)){
				throw new Error('setDisabled:传入参数必须为字符串或者数组!');
			}
			var i = 0,
					len = ids.length,
					j = 0,
					jLen = this.data.length;
			for(; i < len; i++){
				for(; j < jLen; j++){
					if(this.data[j].id === ids[i]){
						var elm = evm.$(ids[i]);
						if(elm){
							evm.css(elm,{
								display:'block'
							});
						}
					}
				}
			}
		},
		onWidgetFocus:function(){
			//遍历从activeIndex之后查找enabled=true项
			var hasFound = false;
			if(this.data.length){
				var index = this.activeIndex;
				while(!hasFound){
					if(this.data[index].enabled){
						this.activeIndex = index;
						hasFound = true;
						break;
					}

					index++;
					index = index%this.data.length;

					if(this.activeIndex === index){
						//查找一圈没有找到
						break;
					}
				}
			}
			if(hasFound){
				this._trigger('onWidgetFocus');
				this.onFocus();
			}
			else{
				this.enabled = false;
			}
		}
	});

	//外部调用
	evm.ext.comp.CompDecorator = CompDecorator;
})(this);