/**
 * 水平列表组件
 * 用于生成水平列表
 * 依赖：/swatapps/ape/evm.js /swatapps/ape/util/marquee/main.js
 * 作者：wuqq@evmtv.com
 * 版本：v0.0.1
 * todo：
 * 1.增加获取数据的本地缓存
 *
 * 2016-04-12 v0.1.1
 * 1.配置文件新增marqueeInst属性，用于焦点时候超长文字自动滚动展示
 * 2016-05-17 v0.2.1
 * 1.新增获取容器DOM对象的方法getBoxElm
 * 2016-05-17 v0.3.2
 * 1.动态生成DOM，默认父级元素次序为.container,body
 */
(function (window) {
	//定义水平列表类
	function HList(options) {
		//类只能被实例化
		if (this === evm.ext.comp || this === window) {
			return new arguments.callee(options);
		}

		//默认配置参数
		var defaults = {
			/*基类配置参数*/
			prefix: 'Ape',
			compName: 'HList',
			parentComp: null,
			/*类本身配置参数*/
			//获取数据方式 once/一次 paging/分页
			//默认为once
			fetchDataType: 'once',
			//配置数据接口地址
			fetchDataUrl: '',
			data:null,
			//当前页中的焦点位置
			activeIndex: 0,
			//容器元素属性
			boxId: '',
			boxLeft: 0,
			boxTop: 0,
			//列表项属性
			itemWidth: 0,
			itemHeight: 0,
			itemGapH: 0,
			//文字字数限制
			textLimit: 100,
			//文字超出部分替代
			//默认为空
			moreReplacement: '',
			//配置滚动对象后自动开启文字滚动
			marqueeInst:null,
			//分页属性
			pageIndex: 1,
			pageSize: 8,
			pageTotalNum: 1,
			switchAutoPaging: true,
			//默认关闭焦点切换收尾循环
			switchLoop:false,
			//列表作为菜单
			//用于标识列表的菜单项
			isMenu: false,
			menuIndex: 0,
			menuPageIndex: 1,
			//需要按确认键才更新菜单项对应的内容
			selectForChange: false,
			//列表数据处理
			model: [],
			//没有数据给出默认提示
			emptyMsg: '',
			emptyImage: '',
			//定义配置方法
			/**
			 * userEvent类似自定义的事件触发时候执行
			 * 可以配置执行自己的操作
			 * @param {string} method 自定义事件名称 onInitComplete,onWidgetBlur,onWidgetFocus,onFocus,onBlur,onSelect
			 *        ,onPreFetchData,fetchDataSuccess,onMenuBlur,onMenuFocus，onFlipover
			 * @param {object} args 返回一个对象
			 */
			userEvent: function () {
			},

			//数据结构可能不同，需要处理获取到的数据
			//默认给出一种栏目数据结构
			//注意：需要计算总页码数
			dataHandler: function (data) {
				var node = data && data.node;
				var ret = null;
				if (node) {
					ret = data[node];
				}
				if (ret && ret instanceof Array) {
					//初始化分页
					if (this.fetchDataType === 'once') {
						this.pageTotalNum = Math.ceil(ret.length / this.pageSize);
					}
					else if (this.fetchDataType === 'paging') {
						//总页码需要根据返回的总项数计算

					}
				}
				return ret;
			},
			//用于表示组件是否可被选中状态
			enabled: false
		};

		evm.extend(defaults, options);
		evm.extend(this, defaults);
		//继承基类
		evm.ext.comp.BaseComponent.call(this, defaults);

		/*私有属性*/
		//容器元素
		this._boxElm = null;
		//接口返回的原始数据
		this._oriData = null;
		//按键的别名
		this._keyAlias = 'N/A';
	}

	evm.extend(HList.prototype, {
		/**
		 * @param method
		 * @param {object} obj
		 * @returns {*}
		 * @private
		 */
		_trigger: function (method, obj) {
			var args = {
				activeIndex: this.activeIndex,
				pageIndex: this.pageIndex
			}
			evm.extend(args, obj||{});

			if (!args.activeData) {
				var activeData = this.getDataByPage()[this.activeIndex];
				args.activeData = activeData;
			}
			if (!args.activeElm) {
				var activeElm = evm.$(this.comp_name + '-item-' + this.activeIndex);
				args.activeElm = activeElm;
			}
			if(!args.alias){
				args.alias = this._keyAlias;
			}

			if (evm.isFunction(this['userEvent'])) {
				return this['userEvent'].call(this, method, args);
			}
		},
		/**
		 * 获取单页显示数据
		 * @private
		 */
		getDataByPage: function () {
			var dataInPage = [],
				start,
				end;
			if (this.data) {
				if (this.fetchDataType === 'once') {
					start = (this.pageIndex - 1) * this.pageSize;
					end = this.pageIndex * this.pageSize;
				}
				else if (this.fetchDataType === 'paging') {
					start = 0;
					end = this.pageSize;
				}
				else {
					throw new Error('配置属性fetchDataType的值不正确!');
				}
				dataInPage = this.data.slice(start, end);
			}
			return dataInPage;
		},
		_fetchData: function (callback) {
			this._trigger('onPreFetchData');
			var self = this;
			if(this.data && this.data.length && this.fetchDataType === 'once' || this.data && (!this._oriData || this._oriData.length === 0) && this.fetchDataType === 'paging'){
				if(!this._oriData){
					this.setOriData(this.data);
				}
				this.dataHandler(this.data);
				callback.call(this);
				return;
			}
			if(this.fetchDataUrl){
				evm.ajax(this.fetchDataUrl, function (data) {
					if (evm.isString(data)) {
						data = eval('(' + data + ')');
					}
					self.setOriData(data);
					self.data = self.dataHandler(data);

					callback.call(self);
				});
			}
		},
		_moveH: function (keyAlias, myEvent) {
			var activeIndex = this.activeIndex,
				pageInData = this.getDataByPage(),
				limit = pageInData.length - 1;
			if (keyAlias === 'LEFT') {
				activeIndex--;
			}
			else if (keyAlias === 'RIGHT') {
				activeIndex++;
			}

			if (activeIndex < 0) {
				if (this.switchAutoPaging) {
					this.prev();
				}
			}
			else if (activeIndex >= 0 && activeIndex <= limit) {
				this.onBlur();
				this.activeIndex = activeIndex;
				this.onFocus();
				if(!this.selectForChange){
					this.onChange();
				}
			}
			else {
				if (this.switchAutoPaging) {
					this.next();
				}
			}
		},
		onFocus: function () {
			var activeElm = evm.$(this.comp_name + '-item-' + this.activeIndex);
			var activeData = this.getActiveData();
			if (activeElm) {
				evm.addClass(activeElm, 'focus');

				if(this.isMenu && this.selectForChange){
					this.onMenuFocus();
				}

				if(this.marqueeInst){
					var textWrapElm = evm.getElementsByClassName('inner',activeElm,'div')[0];
					if (textWrapElm && textWrapElm.innerText && textWrapElm.innerText.length > this.textLimit) {
						this.marqueeInst.startScroll(textWrapElm, textWrapElm.innerText.substr(0, this.textLimit) + this.moreReplacement, textWrapElm.innerText);
					}
				}

				this._trigger('onFocus', {
					activeData: activeData,
					activeElm: activeElm
				});
			}
		},
		onBlur: function (method) {
			var activeElm = evm.$(this.comp_name + '-item-' + this.activeIndex);
			var activeData = this.getActiveData();
			if (activeElm) {
				evm.removeClass(activeElm, 'focus');
				if(this.marqueeInst){
					this.marqueeInst.stopScroll();
				}

				this._trigger('onBlur', {
					activeData: activeData,
					activeElm: activeElm
				});
			}
		},
		onChange:function(){
			if(this.selectForChange){
				this.onMenuBlur();

				this.menuIndex = this.activeIndex;
				this.menuPageIndex = this.pageIndex;

				this.onMenuFocus();
			}
			else{
				this.onMenuBlur();

				this.menuIndex = this.activeIndex;
				this.menuPageIndex = this.pageIndex;
			}
			this._trigger('onChange');
		},
		//组件按键事件入口
		action: function (keyAlias, myEvent) {
			this._keyAlias = keyAlias;
			switch (keyAlias) {
				case 'LEFT':
				case 'RIGHT':
					this._moveH(keyAlias, myEvent);
					break;
				case 'UP':
					this.onWidgetBlur('UP');
					break;
				case 'DOWN':
					this.onWidgetBlur('DOWN');
					break;
				case 'SELECT':
					this.onSelect();
					break;
			}
		},
		/**
		 * 组件初始化
		 */
		init: function () {
			this._trigger('onPreInit');
			var self = this;

			//处理列表数据
			this._fetchData(function () {
				if (!(self.data instanceof Array)) {
					throw new Error('传入列表数据应该为数组!');
				}

				//容器元素
				if (!self.boxId) {
					throw new Error('配置参数boxId不能为空!');
				}
				self._boxElm = evm.$(self.boxId);

				var boxStyle = 'position:absolute;'
					+ 'left:0px;'
					+ 'top:0px';
				if (!self._boxElm) {
					var parentNode = evm.$('container') || document.body;
					self._boxElm = evm.createElem('div', {id: self.boxId, style: boxStyle},parentNode);
				}
				else {
					//清空容器
					evm.html(self._boxElm, '');
					evm.css(self._boxElm, boxStyle);
				}
				//数据准备完成，加载页面视图之前触发
				self._trigger('onInitPreBuildView');

				self._buildView();

				var dataInPage = self.getDataByPage();
				if(dataInPage.length){
					self.enabled = true;
				}
				self._trigger('onInitComplete');
			});
		},
		/**
		 * 构建视图
		 */
		_buildView: function () {
			var dataInPage = this.getDataByPage(),
				model = this.model,
				i = 0,
				iLen = dataInPage.length,
				j = 0,
				jLen = model.length,
				html = '',
				prop;
			for (; i < iLen; i++) {
				var itemStyle = 'position:absolute;'
					+ 'width:' + this.itemWidth + 'px;'
					+ 'height:' + this.itemHeight + 'px;'
					+ 'line-height:' + this.itemHeight + 'px;'
					+ 'left:' + (this.boxLeft + ((this.itemWidth + this.itemGapH) * i)) + 'px;'
					+ 'top:' + this.boxTop + 'px;';
				var itemClassName = this.class_name + '-item';
				html += '<div id="' + this.comp_name + '-item-' + i + '" class="' + itemClassName + '" style="' + itemStyle + '">';
				j = 0;
				if (jLen > 0) {
					for (; j < jLen; j++) {
						if (model[j].text) {
							prop = dataInPage[i][model[j].text];
						}
						else {
							throw new Error('需要配置model参数!');
						}
						//如果是文字
						if (model[j].text === 'text' && prop.length > this.textLimit) {
							prop = prop.substr(0, this.textLimit) + this.moreReplacement;
						}
						if (model[j].callback && evm.isFunction(model[j].callback)) {
							prop = model[j].callback.call(this, prop, i, dataInPage, model[j].text);
						}
						html += prop;
					}
				}
				else {
					throw new Error('需要配置model参数!');
				}
				html += '</div>';
			}
			evm.html(this._boxElm, html);

			//页码等分页相关视图
			this._trigger('onFlipover', {
				pageIndex: this.pageIndex,
				pageTotalNum: this.pageTotalNum
			});
		},
		prev: function () {
			var self = this;
			var currPageIndex = this.pageIndex;
			currPageIndex--;
			if (currPageIndex >= 1) {
				this.onBlur();
				this.pageIndex = currPageIndex;
				this._fetchData(function () {
					var dataInPage = this.getDataByPage();
					if (dataInPage) {
						this._buildView();
						if (this.switchAutoPaging) {
							this.activeIndex = dataInPage.length - 1;
							this.onFocus();
							if(!this.selectForChange){
								this.onChange();
							}
						}
					}
				});
			}
			else {
				this.onWidgetBlur('LEFT');
			}
		},
		next: function () {
			var self = this;
			var currPageIndex = this.pageIndex;
			currPageIndex++;
			if (currPageIndex <= this.pageTotalNum) {
				this.onBlur();
				this.pageIndex = currPageIndex;
				this._fetchData(function () {
					var dataInPage = this.getDataByPage();
					if (dataInPage) {
						this._buildView();
						if (this.switchAutoPaging) {
							this.activeIndex = 0;
							this.onFocus();
							if(!this.selectForChange){
								this.onChange();
							}
						}
					}
				});
			}
			else {
				this.onWidgetBlur('RIGHT');
			}
		},
		//获取焦点对应的数据
		getActiveData: function () {
			var dataInPage = this.getDataByPage();
			var ret = null;
			if (dataInPage) {
				ret = dataInPage[this.activeIndex];
			}
			return ret;
		},
		/**
		 * 恢复对象的初始化状态
		 */
		destroy: function () {
			//初始化一些属性
			this._oriData = null;
			this.data = null;

			this.activeIndex = 0;
			this.pageIndex = 1;
			this.menuIndex = 0;
			this.menuPageIndex = 1;

			this.enabled = false;

			evm.html(this._boxElm,'');
		},
		hasListData: function () {
			if (this.data instanceof Array) {
				return !!this.data.length;
			}
			return false;
		},
		setFetchDataUrl: function (url) {
			this.fetchDataUrl = url;
		},
		/**
		 * 去除项目的菜单状态
		 */
		onMenuBlur:function(){
			if(this.menuPageIndex !== this.pageIndex){
				//记录的菜单页码如果不在当前页，直接退出
				return;
			}
			var menuElm = evm.$(this.comp_name + '-item-' + this.menuIndex);
			if (menuElm) {
				evm.removeClass(menuElm, 'menu');
				this._trigger('onMenuBlur', {
					menuPageIndex: this.menuPageIndex,
					menuIndex: this.menuIndex,
					menuElm: menuElm
				});
			}
		},
		/**
		 * 设置项目菜单状态
		 */
		onMenuFocus:function(){
			if(this.menuPageIndex !== this.pageIndex){
				//记录的菜单页码如果不在当前页，直接退出
				return;
			}
			//菜单新位置
			var menuElm = evm.$(this.comp_name + '-item-' + this.menuIndex);
			if(menuElm){
				evm.addClass(menuElm, 'menu');
				this._trigger('onMenuFocus', {
					menuPageIndex: this.menuPageIndex,
					menuIndex: this.menuIndex,
					menuElm: menuElm
				});
			}
		},
		/**
		 * 按确定键操作
		 */
		onSelect: function () {
			if(this.selectForChange){
				this.onChange();
				this.onFocus();
			}
			this._trigger('onSelect');
		},
		setOriData: function (data) {
			this._oriData = data;
		},
		getOriData: function () {
			return this._oriData;
		},
		getData: function () {
			return this.data;
		},
		/**
		 * 组件获得焦点时候执行操作
		 */
		onWidgetFocus:function(){
			this._trigger('onWidgetFocus');
			this.onFocus();
		},
		/**
		 * 组件失去焦点时候执行操作
		 * @param dir
		 */
		onWidgetBlur:function(dir){
			if (this._trigger('onWidgetBlur', {
						alias: dir
					})) {
				this.onBlur();
				if (this.isMenu) {
					this.onMenuFocus();
				}
			}
			else{
				if(this.switchLoop){
					if(dir === 'LEFT'){
						this.onBlur();
						this.pageIndex = this.pageTotalNum;
						this.activeIndex = this.getDataByPage().length - 1;
						if(this.pageTotalNum > 1){
							this._buildView();
						}
						if (!this.selectForChange) {
							this.onChange();
						}
						this.onFocus();
					}
					else if(dir === 'RIGHT'){
						this.onBlur();
						this.activeIndex = 0;
						this.pageIndex = 1;
						if(this.pageTotalNum > 1){
							this._buildView();
						}
						if (!this.selectForChange) {
							this.onChange();
						}
						this.onFocus();
					}
				}
			}
		},
		/**
		 * 获取容器元素
		 */
		getBoxElm:function(){
			return this._boxElm;
		}
	});

	//外部调用
	evm.ext.comp.HList = HList;
})(this);