if (!evm) {
    throw new Error("未引用evm.js");
}
(function (window) {
    /**
     * 磁贴块组件 创建于2016-08-28
     *
     * @version 2.1.3
     * @author wuqq@evmtv.com
     *
     * @requires /swatapps/ape/evm.js
     * @requires /swatapps/ape/util/common.js
     *
     * @alias evm.ext.comp.Tiles
     * @param {Object} options 配置对象，具体成员可以参考默认配置对象{@link Tiles.defaults}
     * @constructor
     *
     * v1.0.1 2016-11-11
     * 处理视图缓存问题，默认缓存10个视图，可以配置
     * 处理多次按键导致连续翻页问题
     * 处理多次按键导致下一页焦点位置改变的问题
     * 按方向键增加了100ms延时
     * 删除视图结点时候，将包含的图片结点src属性赋空值，防止src指向的图片资源被占用着
     *
     * v2.0.0 2016-11-13
     *
     * v2.0.1 2016-12-7
     * 1.翻页时候清空原来节点内容，需要在model属性更据value为空写清空节点内容的逻辑
     * 2.init中加载数据为空或者加载数据失败的自定义事件onFetchDataError
     *
     * v2.1.0 2017-6-19
     *
     * v2.1.2 2017-11-09
     * 1.修复内部焦点循环Bug
     *
     * v2.1.3 2017-11-30
     * 1.移除对日志组件evm.util.common.logger的依赖
     * 2.修改destroy方法逻辑，增加DOM节点恢复为初始状态
     */
    /**
     * 日志打印方法
     * @param [type]
     * @param inst 插件类实例
     * @param message
     */
    function logIt(type, inst, message) {
        if (arguments.length === 2) {
            message = instance;
            instance = type;
            type = 'debug';
        }
        if (arguments.length >= 2 && inst) {
            try {
                evm.util.common.logger[type]('Tiles(id=' + inst.boxId + ')' + message);
            }
            catch (e) {
                if (console) {
                    console.warn('打印日志报错:' + e.message);
                    console.warn('可能原因:未引入日志打印文件，如果/swatapps/ape/util/common-1.3.1.js，之后版本，需要单独引入/swatapps/ape/util/logger.js');
                }
            }
        }
    }

    function Tiles(options) {
        //类只能被实例化
        if (this === evm.ext.comp || this === window) {
            return new arguments.callee(options);
        }
        var config = evm.util.common.extend(true, {}, Tiles.defaults, options || {});
        evm.util.common.extend(this, config);
        //继承基类
        evm.ext.comp.BaseComponent.call(this, config);
        this._config = config;
        //私有属性
        //接口获取的原始数据对象
        this._oriData = null;
        //容器元素
        this._boxElm = null;
        //每页数据
        this._dataInPage = null;
        //记录上一次操作记录，用于互逆操作返回到刚才位置
        this._lastDir;
        this._lastActivePos;
        //按键的别名
        this._keyAlias = 'N/A';
        this._trackPageIndex = this.pageIndex;
        this._xhr = null;

        this._viewIndex = 0;
        //当前视图元素
        this._currentViewElm = null;

        //试图改变
        this._viewChanged = false;
    }

    Tiles.prototype = {
        /**
         * 构建或更新视图
         * @private
         */
        _buildView: function () {
            var self = this;
            logIt('debug', this, '<ext.comp.Tiles _buildView()> start');

            function fill(viewWrapper) {
                logIt('debug', self, '<ext.comp.Tiles _buildView()>当前页码:' + this.pageIndex);
                logIt('debug', self, '<ext.comp.Tiles _buildView()>执行fill函数 开始');
                var viewIndex = this.getViewIndex(),
                    view = this.getView(viewIndex),
                    model = this.model,
                    viewCounter = 0,
                    viewLen = view.length,
                    modelCounter = 0,
                    modelLen = model.length,
                    viewElm = this.getViewElmByView(view);
                //如果视图DOM不存在就构建新的视图
                if (!viewElm) {
                    viewElm = evm.createElem('div', {
                        id: this.comp_name + '-view-' + this._viewIndex,
                        className: this.comp_name + '-view',
                        style: 'position:absolute;left:0;top:0;'
                    });
                    viewWrapper.appendChild(viewElm);

                    try {
                        var serializeStr = this._serializeView(view);
                        viewElm.setAttribute('data-view', serializeStr);

                        var dataView = viewElm.getAttribute("data-view");
                    }
                    catch (e) {
                        logIt('error', self, '<ext.comp.Tiles _buildView()> viewElm可能不支持setAttribute、getAttribute,报错: ' + e.message);
                    }
                    if (!dataView) {
                        logIt('warn', self, '<ext.comp.Tiles _buildView()> viewElm.getAttribute("data-view")获取不了序列化view的属性');
                    }

                    for (; viewCounter < viewLen; viewCounter++) {
                        var tileStyle = 'position:absolute;'
                            + 'z-index:' + (viewLen - viewCounter) + ';'
                            + 'left:' + view[viewCounter].left + 'px;'
                            + 'top:' + view[viewCounter].top + 'px;'
                            + 'width:' + view[viewCounter].width + 'px;'
                            + 'height:' + view[viewCounter].height + 'px;';
                        var tileElm = evm.createElem('div', {
                            id: this.comp_name + '-view-' + this._viewIndex + '-tile-' + viewCounter,
                            className: this.class_name + '-tile',
                            style: tileStyle
                        });
                        viewElm.appendChild(tileElm);
                        //增加每个磁贴块内部节点
                        if (modelLen) {
                            var html = '';
                            for (modelCounter = 0; modelCounter < modelLen; modelCounter++) {
                                //检测model对象合法
                                var modelItem = this.model[modelCounter];
                                if (!evm.isObject(modelItem)) {
                                    logIt('warn', self, '<ext.comp.Tiles _buildView()>model属性index为' + modelCounter + '的成员不是对象');
                                    continue;
                                }
                                var text = modelItem.text;
                                var createDomFn = modelItem.createDom;
                                var fillDataFn = modelItem.fillData;
                                if (!evm.isString(text)) {
                                    logIt('error', self, '<ext.comp.Tiles _buildView()>model属性需要配置text属性');
                                }
                                if (!evm.isFunction(createDomFn)) {
                                    logIt('error', self, '<ext.comp.Tiles _buildView()>model属性需要配置createDom函数用于生成正确的DOM结构');
                                }
                                if (!evm.isFunction(fillDataFn)) {
                                    logIt('error', self, '<ext.comp.Tiles _buildView()>model属性需要配置fillDataFn函数用于更新节点数据');
                                }
                                var domHtml = createDomFn();
                                html += domHtml;
                            }
                            evm.html(tileElm, html);
                        }
                        else {
                            logIt('error', self, '<ext.comp.Tiles _buildView()>对象实例化传入配置对象需要配置model属性');
                        }
                    }

                    this._viewIndex++;
                }

                if (this._currentViewElm !== viewElm) {
                    this.hideView();
                }

                this._currentViewElm = viewElm;

                this.updateDom(this._dataInPage);
                //显示视图
                this.showView();

                logIt('debug', self, '<ext.comp.Tiles _buildView()> end');
            }

            logIt('debug', this, '<ext.comp.Tiles _buildView()> this._dataInPage = ' + evm.util.common.json2str(this._dataInPage));
            if (this._dataInPage && this._dataInPage.length) {
                //建立视图的包装DOM节点
                var wrapperId = this.comp_name + '-view-wrapper';
                var viewWrapper = evm.$(wrapperId);
                if (!viewWrapper) {
                    viewWrapper = evm.createElem('div', {
                        id: wrapperId,
                        className: this.class_name + '-view-wrapper',
                        style: 'position:absolute;left:0;top:0;'
                    }, this._boxElm);
                }
                fill.call(this, viewWrapper);
            }
            else {
                logIt('warn', this, '<ext.comp.Tiles _buildView()>this._dataInPage应该为数组，typeof this._dataInPage = ' + (typeof this._dataInPage));
            }
        },
        updateDom: function (dataInPage) {
            if (!this._currentViewElm) {
                return;
            }

            var viewIndex = this.getViewIndex(),
                view = this.getView(viewIndex),
                model = this.model,
                viewCounter = 0,
                viewLen = view.length,
                modelCounter = 0,
                modelLen = model.length;
            //新增或者更新节点数据
            logIt('debug', this, '<ext.comp.Tiles _buildView()>新增或者更新节点数据');
            for (viewCounter = 0; viewCounter < viewLen; viewCounter++) {
                for (modelCounter = 0; modelCounter < modelLen; modelCounter++) {
                    var name = this.model[modelCounter].text;
                    var value = '';
                    if (dataInPage) {
                        value = dataInPage[viewCounter] && dataInPage[viewCounter][name];
                        if (value == null) {
                            value = '';
                        }
                        //处理文字超长
                        //todo:中文标点符号宽度可能造成文本长度显示不美观
                        if (name === this.marqueeTextProp && value.length > this.textLimit) {
                            value = value.substr(0, this.textLimit) + this.moreReplacement;
                        }
                    }
                    var tileId = this._currentViewElm.id + '-tile-' + viewCounter;
                    var tileElem = evm.$(tileId);
                    var args = {
                        tileElem: tileElem,
                        value: value,
                        indexInPage: viewCounter,
                        dataInPage: dataInPage
                    };
                    if (dataInPage && dataInPage[viewCounter]) {
                        evm.util.common.show(tileElem);
                        this.model[modelCounter].fillData.call(this, args);
                    }
                    else if (tileElem) {
                        evm.util.common.hide(tileElem);
                        // this.model[modelCounter].fillData.call(this, args);
                    }
                }
            }
        },
        //处理内部焦点循环
        _dealLoop: function (dir, tile) {
            if (tile) {
                if (dir === 'UP') {
                    tile.top = 1440;
                }
                else if (dir === 'DOWN') {
                    tile.top = -720;
                }
                else if (dir === 'LEFT') {
                    tile.left = 2560;
                }
                else if (dir === 'RIGHT') {
                    tile.left = -1280;
                }
                this._move(dir, null, tile);
                this._lastDir = 'N/A';
            }
        },
        /**
         * 获取接口数据
         * @param callback
         * @private
         */
        _fetchData: function (callback) {
            logIt('debug', this, '<ext.comp.Tiles._fetchData()> start.');
            var self = this;
            this._trigger('onPreFetchData');
            var self = this;
            if (this.fetchDataType === 'once' && this.data && this._oriData) {//一次加载数据并且已经设置了数据和原始数据
                self._trigger('onFetchDataComplete');
                callback.call(this);
                return;
            }
            else if (this.fetchDataType === 'once' && this.data && !this._oriData//一次加载数据，设置了数据，未设置原始数据
                || this.fetchDataType === 'paging' && this.data && !this._oriData) {//分页加载数据，设置了数据，未设置原始数据
                self._trigger('onFetchDataComplete');
                this.setOriData(this.data);
                this.data = this.dataHandler(this.data);
                callback.call(this);
                return;
            }
            if (this.fetchDataUrl) {
                logIt('debug', this, '<ext.comp.Tiles._fetchData()> response url:' + this.fetchDataUrl);
                if (this._xhr) {
                    this._xhr.onabort();
                    this._xhr = null;
                }
                this._xhr = evm.ajax(this.fetchDataUrl, function (data) {
                    logIt('debug', self, '<ext.comp.Tiles._fetchData()> response data:' + data);
                    self._trigger('onFetchDataComplete');
                    try {
                        if (evm.isString(data)) {
                            data = eval('(' + data + ')');
                        }
                    }
                    catch (e) {
                        //如果解析失败，默认为空数组
                        data = [];
                        logIt('error', self, '<ext.comp.Tiles _fetchData()>解析json字符串失败，this.fetchDataUrl = ' + self.fetchDataUrl);
                    }
                    self.setOriData(data);
                    self.data = self.dataHandler(data);
                    callback.call(self);
                    self._xhr = null;
                }, function () {
                });
            }
            else {
                callback.call(this);
                return;
            }
        },
        /**
         * 返回true表示找到更好的合适块
         * @param currentPos 当前块的索引值
         * @param availablePos 当前最优块索引值
         * @returns {*}
         * @private
         */
        _compare: function (currentPos, availablePos, keyAlias, virtualView) {
            //焦点块边界坐标
            var view = this.getView(this.pageIndex - 1),
                activeView = virtualView || view[this.activeIndex],
                al = activeView.left,
                ar = activeView.left + activeView.width,
                at = activeView.top,
                ab = activeView.top + activeView.height;
            //当前块边界坐标
            var currentView = view[currentPos],
                cl = currentView.left,
                cr = currentView.left + currentView.width,
                ct = currentView.top,
                cb = currentView.top + currentView.height;
            if (availablePos === -1) {
                //初始找到可以选中的项目
                if (currentView.disabled) {
                    return false;
                }
                else {
                    return true;
                }
            }
            //比较跳过不可选中的项目
            if (currentView.disabled) {
                return false;
            }
            //最优边界坐标
            var mostView = view[availablePos],
                ml = mostView.left,
                mr = mostView.left + mostView.width,
                mt = mostView.top,
                mb = mostView.top + mostView.height;
            //条件二：按照和焦点块交叉面积多为优先
            if (keyAlias === 'LEFT' || keyAlias === 'RIGHT') {
                var dx = al - (view[currentPos].left + view[currentPos].width);
                var dx1 = al - (view[availablePos].left + view[availablePos].width);
                if (keyAlias === 'RIGHT') {
                    dx = view[currentPos].left - ar;
                    dx1 = view[availablePos].left - ar;
                }
                var dy,
                    dy1;
                //判断是否有垂直交叉
                //当前块和最优块都在焦点块上下边界之外，没有垂直交叉
                if ((ct > ab || cb < at) && (mt > ab || mb < at)) {
                    if (ct > ab) {
                        dy = ct - ab;
                    }
                    else {
                        dy = at - cb;
                    }
                    if (mt > ab) {
                        dy1 = mt - ab;
                    }
                    else {
                        dy1 = at - mb;
                    }
                    //取水平方向距离近的
                    if (dx < dx1) {
                        return true;
                    }
                }
                //最优块没有垂直交叉，当前块有垂直交叉
                else if ((mt > ab || mb < at) && (ct >= at && ct <= ab || cb >= at && cb <= ab || ct <= at && cb >= ab)) {
                    return true;
                }
                //当前块和最优块都和焦点块有边界延长重叠
                //重叠长度大者优先
                else if ((ct >= at && ct <= ab || cb >= at && cb <= ab || ct <= at && cb >= ab) && (mt >= at && mt <= ab || mb >= at && mb <= ab || mt <= at && mb >= ab)) {
                    var dl//当前块和焦点块重叠长度
                        , dl1;//最优块和焦点块重叠长度
                    if (ct <= at && cb >= ab) {
                        dl = ab - at;
                    }
                    else if (ct <= at && cb <= ab) {
                        dl = cb - at;
                    }
                    else if (ct >= at && cb <= ab) {
                        dl = cb - ct;
                    }
                    else if (ct >= at && cb >= ab) {
                        dl = ab - ct;
                    }
                    if (mt <= at && mb >= ab) {
                        dl1 = ab - at;
                    }
                    else if (mt <= at && mb <= ab) {
                        dl1 = mb - at;
                    }
                    else if (mt >= at && mb <= ab) {
                        dl1 = mb - mt;
                    }
                    else if (mt >= at && mb >= ab) {
                        dl1 = ab - mt;
                    }
                    if (dl != null && dl1 != null) {
                        if (dl > dl1) {
                            //当前块必须不能超过最优块的右侧
                            if (dx + view[currentPos].width <= dx1) {
                                return true;
                            }
                        }
                        else if (dl === dl1) {
                            if (dx < dx1) {
                                return true;
                            }
                        }
                        else {
                            //最优块距离当前块远端，选择当前块
                            if (dx + view[currentPos].width <= dx1) {
                                return true;
                            }
                        }
                    }
                }
            }
            else if (keyAlias === 'UP' || keyAlias === 'DOWN') {
                var dy = at - (view[currentPos].top + view[currentPos].height);
                var dy1 = at - (view[availablePos].top + view[availablePos].height);
                if (keyAlias === 'DOWN') {
                    dy = view[currentPos].top - ab;
                    dy1 = view[availablePos].top - ab;
                }
                var dx,
                    dx1;
                if (dy < dy1) {
                    return true;
                }
                else if (dy === dy1) {
                    //判断是否有边界延长重叠
                    if ((cr < al || cl > ar) && (mr < al || ml > ar)) {
                        //当前块和最优块都在焦点块左右边界之外
                        if (al > cr) {
                            dx = al - cr;
                        }
                        else {
                            dx = cl - ar;
                        }
                        if (al > mr) {
                            dx1 = al - mr;
                        }
                        else {
                            dx1 = ml - ar;
                        }
                        if (dx < dx1) {
                            return true;
                        }
                    }
                    else if ((mr < al || ml > ar) && (cl >= al && cl <= ar || cr <= ar && cr >= al || cr >= ar && cl <= al)) {
                        //最优块左右边界之外,当前块和焦点块有边界延长重叠
                        return true;
                    }
                    else if ((cl >= al && cl <= ar || cr <= ar && cr >= al || cl <= al && cr >= ar) && (ml >= al && ml <= ar || mr <= ar && mr >= al || ml <= al && mr >= ar)) {
                        //当前块和最优块都和焦点块有边界延长重叠
                        //重叠长度大者优先
                        var dl,
                            dl1;
                        if (cl >= al && cr <= ar) {
                            dl = cr - cl;
                        }
                        else if (cl <= al && cr >= ar) {
                            dl = ar - al;
                        }
                        else if (cl <= al && cr <= ar) {
                            dl = cr - al;
                        }
                        else if (cl >= al && cr >= ar) {
                            dl = ar - cl;
                        }
                        if (ml >= al && mr <= ar) {
                            dl1 = mr - ml;
                        }
                        else if (ml <= al && mr >= ar) {
                            dl1 = ar - al;
                        }
                        else if (ml <= al && mr <= ar) {
                            dl1 = mr - al;
                        }
                        else if (ml >= al && mr >= ar) {
                            dl1 = ar - ml;
                        }
                        if (dl != null && dl1 != null) {
                            if (dl >= dl1) {
                                return true;
                            }
                        }
                    }
                }
            }
        },
        /**
         * 查找最优块
         * @param keyAlias
         * @param virtualView 可选，用于处理内部焦点循环
         * @returns {number}
         * @private
         */
        _findAvailablePos: function (keyAlias, virtualView) {
            var i = 0,
                j,
                view = this.getView(this.pageIndex - 1),
                x = view[this.activeIndex] && view[this.activeIndex].left,
                y = view[this.activeIndex] && view[this.activeIndex].top,
                dx = 0,
                dy = 0,
                available = null,
                availablePos = -1,
                posibbleIndexs = [];
            if (virtualView) {
                x = virtualView.left;
                y = virtualView.top;
            }
            //焦点块边界查找可能的位置索引，Push进filterIndexs
            for (; i < this._dataInPage.length; i++) {
                if (!virtualView && i === this.activeIndex) {
                    continue;
                }
                var item = this._dataInPage[i];
                dx = view[i].left - x;
                dy = view[i].top - y;
                if (keyAlias === 'LEFT') {
                    dx += view[i].width;
                    if (dx <= 0) {
                        posibbleIndexs.push(i);
                    }
                }
                else if (keyAlias === 'RIGHT') {
                    if (virtualView) {
                        dx -= virtualView.width;
                    }
                    else {
                        dx -= view[this.activeIndex].width;
                    }
                    if (dx >= 0) {
                        posibbleIndexs.push(i);
                    }
                }
                else if (keyAlias === 'UP') {
                    dy += view[i].height;
                    if (dy <= 0) {
                        posibbleIndexs.push(i);
                    }
                }
                else if (keyAlias === 'DOWN') {
                    if (virtualView) {
                        dx -= virtualView.height;
                    }
                    else {
                        dy -= view[this.activeIndex].height;
                    }
                    if (dy >= 0) {
                        posibbleIndexs.push(i);
                    }
                }
            }
            for (i = 0, j = posibbleIndexs.length; i < j; i++) {
                var index = posibbleIndexs[i];
                var item = this._dataInPage[index];
                if (this._compare(index, availablePos, keyAlias, virtualView)) {
                    available = item;
                    availablePos = index;
                }
            }
            return availablePos;
        },
        /**
         * 获取单页显示数据
         * @private
         * @return 未找到数据返回null
         */
        _getDataByPage: function () {
            var data = null,
                start = 0,
                end = 0,
                i = 0;
            logIt('debug', this, '<ext.comp.Tiles _getDataByPage()> this.data = ' + evm.util.common.json2str(this.data));
            //this.data instanceof Array
            //华数盒子不支持
            if (!this.data /*|| !(this.data instanceof Array)*/ || !this.data.length) {
                return data;
            }
            if (this.fetchDataType === 'once') {
                for (; i < this.pageIndex; i++) {
                    var view = this.getView(i);
                    if (i < this.pageIndex - 1) {
                        start += view.length;
                    }
                    end += view.length;
                }
            }
            else if (this.fetchDataType === 'paging') {
                var view = this.getView(this.pageIndex - 1);
                end = view.length;
            }
            logIt('debug', this, evm.util.common.template('<ext.comp.Tiles _getDataByPage()> start = {start},end = {end}', {
                start: start,
                end: end
            }));
            if (start !== end) {
                var arr = [];
                for (var i = start; i < end; i++) {
                    if (this.data[i])
                        arr.push(this.data[i]);
                }
                data = arr;
                //华数盒子不支持
                //data = this.data.slice(start, end);
            }
            return data;
        },
        /**
         * 焦点移动方法
         * @param keyAlias
         * @param myEvent
         * @param virtualView，可选，用于处理内部焦点循环
         * @private
         */
        _move: function (keyAlias, myEvent, virtualView) {
            logIt('debug', this, '<ext.comp.Tiles> _move() 开始移动');
            var availablePos;
            if (this._lastDir === 'LEFT' && keyAlias === 'RIGHT' || this._lastDir === 'RIGHT' && keyAlias === 'LEFT') {
                availablePos = this._lastActivePos;
            }
            else if (this._lastDir === 'UP' && keyAlias === 'DOWN' || this._lastDir === 'DOWN' && keyAlias === 'UP') {
                availablePos = this._lastActivePos;
            }
            else {
                availablePos = this._findAvailablePos(keyAlias, virtualView);
            }
            logIt('debug', this, '<ext.comp.Tiles> _move() availablePos=' + availablePos);
            if (virtualView && (availablePos === -1 || availablePos === this.activeIndex) && !this._viewChanged) {
                //没有找到循环的焦点位置
                return;
            }

            if (availablePos !== -1 && (availablePos !== this.activeIndex || this._viewChanged)) {
                this._viewChanged = false;

                this._lastDir = keyAlias;
                this._lastActivePos = this.activeIndex;
                this.onBlur();
                this.activeIndex = availablePos;
                this.onFocus();
                if (!this.selectForChange) {
                    this.onChange();
                }
            }
            else {
                if (this.switchAutoPaging) {
                    if (this.autoPagingDir === 'H') {
                        if (keyAlias === 'LEFT' && this.pageIndex > 1) {
                            this.prev(keyAlias);
                        }
                        else if (keyAlias === 'RIGHT' && this.pageIndex < this.pageTotalNum) {
                            this.next(keyAlias);
                        }
                        else {
                            this.onWidgetBlur(keyAlias);
                        }
                    }
                    else if (this.autoPagingDir === 'V') {
                        if (keyAlias === 'UP' && this.pageIndex > 1) {
                            this.prev(keyAlias);
                        }
                        else if (keyAlias === 'DOWN' && this.pageIndex < this.pageTotalNum) {
                            this.next(keyAlias);
                        }
                        else {
                            this.onWidgetBlur(keyAlias);
                        }
                    }
                    else {
                        this.onWidgetBlur(keyAlias);
                    }
                }
                else {
                    this.onWidgetBlur(keyAlias);
                }
            }
        },
        /**
         * @param method
         * @param {object} obj
         * @returns {*}
         * @private
         */
        _trigger: function (method, obj) {
            var args = {
                activeIndex: this.activeIndex,
                pageIndex: this.pageIndex
            }
            evm.extend(args, obj);
            if (!args.activeData) {
                var activeData = this.getActiveData();
                args.activeData = activeData;
            }
            if (!args.activeElm) {
                var activeElm = this.getActiveElm();
                args.activeElm = activeElm;
            }
            if (!args.alias) {
                args.alias = this._keyAlias;
            }
            if (evm.isFunction(this['userEvent'])) {
                return this['userEvent'].call(this, method, args);
            }
        },
        /**
         * 组件按键事件入口
         * @param keyAlias
         * @param myEvent
         * @returns {number}
         */
        action: function (keyAlias, myEvent) {
            var self = this;
            this._keyAlias = keyAlias;
            var res = 0;
            switch (keyAlias) {
                case 'UP':
                case 'RIGHT':
                case 'DOWN':
                case 'LEFT':
                    if (this._trackPageIndex - this.pageIndex == 0) {
                        res = this._move(keyAlias, myEvent);
                    }
                    break;
                case 'SELECT':
                    res = this.onSelect();
                    break;
                default:
                    var args = {
                        alias: keyAlias,
                        event: myEvent
                    };
                    res = this._trigger('onActionDefault', args);
                    break;
            }
            return res;
        },
        /**
         * 组件对象部分恢复为初始状态
         */
        destroy: function () {
            //清空DOM子节点
            //先把图片关联移除掉
            //todo 待测试
            var imgs = this._boxElm.getElementsByTagName('img');
            for (var i = 0; i < imgs.length; i++) {
                imgs[i].src = '';
            }
            this._boxElm.innerHTML = '';

            //清空数据
            this.data = null;
            this._oriData = null;

            evm.util.common.extend(this, this._config);

            this._dataInPage = null;
            this._lastDir = null;
            this._lastActivePos = -1;
            this._keyAlias = 'N/A';
            this._trackPageIndex = this.pageIndex;
            this._xhr = null;
            this._viewIndex = 0;
            this._currentViewElm = null;
            this._viewChanged = false;
            this._trigger('onDestroy');
        },
        /**
         * 获取焦点对应的数据对象
         * @returns {Object|null}
         */
        getActiveData: function () {
            return this._dataInPage && this._dataInPage[this.activeIndex];
        },
        /**
         * 获取焦点对应的DOM对象
         */
        getActiveElm: function () {
            if (!this._currentViewElm) {
                return;
            }
            var viewCounter = this.activeIndex;
            var tileId = this._currentViewElm.id + '-tile-' + viewCounter;
            var activeElm = evm.$(tileId);
            return activeElm;
        },
        /**
         * 获取容器元素对象
         * @returns {null|*}
         */
        getBoxElm: function () {
            return this._boxElm;
        },
        /**
         * 获取接口返回的原始数据对象
         * @returns {null|*}
         */
        getOriData: function () {
            return this._oriData;
        },
        /**
         * 获取单页数据
         * @returns {*|null}
         */
        getDataInPage: function () {
            return this._dataInPage;
        },
        /**
         * 序列化视图
         * @private
         */
        _serializeView: function (view) {
            return evm.util.common.json2str(view);
        },
        /**
         * 获取视图元素
         */
        getViewElmByView: function (view) {
            var viewElmList = evm.getElementsByClassName(this.comp_name + '-view', this._boxElm, 'div');

            var serializeView = this._serializeView(view);

            for (var i = 0; i < viewElmList.length; i++) {
                var dataView;
                try {
                    dataView = viewElmList[i].getAttribute("data-view");
                }
                catch (e) {
                    logIt('error', this, '<ext.comp.Tiles getViewElmByView()> viewElmList[i]可能不支持getAttribute,报错: ' + e.message);
                }
                if (serializeView === dataView) {
                    return viewElmList[i];
                }
            }
            return null;
        },
        /**
         * 获取view数组成员
         * @param {number} index
         * @returns {*}
         */
        getView: function (index) {
            if (!this.view || !(this.view instanceof Array) || this.view.length < 1) {
                throw new Error('未正确配置view');
            }
            var view = this.view[index];
            if (!view) {
                view = this.view[this.view.length - 1];
            }
            if (view.length === 0) {
                var error = "ext.comp.Tiles.getView(" + index + ")获取失败";
                logIt('error', this, error);
                throw new Error(error);
            }
            return view;
        },
        getViewIndex: function () {
            var view = this.getView(this.pageIndex - 1);
            for (var i = 0; i < this.view.length; i++) {
                if (this.view[i] === view) {
                    return i;
                }
            }
            return this.view.length - 1;
        },
        /**
         * 隐藏页码对应的磁贴块视图
         */
        hideView: function () {
            if (this._currentViewElm) {
                evm.css(this._currentViewElm, {
                    display: 'none'
                });
            }
            //清空原来的内容
            this.updateDom();
        },
        /**
         * 组件初始化
         */
        init: function () {
            logIt('error', this, '<ext.comp.Tiles.init()> ' + this.boxId + '组件初始化开始');
            var self = this;
            this._trigger('onPreInit');
            this._trackPageIndex = this.pageIndex;
            this._fetchData(function () {
                if (!self.data || !self.data.length) {
                    self._trigger('onFetchDataError', {
                        oriData: self.getOriData()
                    });
                    self._trigger('onInitComplete');
                    self._trigger('onFlipover', {
                        pageIndex: 0,
                        pageTotalNum: 0
                    });
                    return;
                }
                //准备容器元素
                var containerElm = evm.$('container') || document.body;
                if (!evm.isString(self.boxId)) {
                    self.boxId = self.comp_name;
                }
                self._boxElm = evm.$(self.boxId);
                var boxStyle = 'position:absolute;'
                    + 'left:' + self.boxLeft + 'px;'
                    + 'top:' + self.boxTop + 'px;'
                    + 'display:' + (self.isVisible === true ? 'block' : 'none') + ';';
                if (!self._boxElm) {
                    self._boxElm = evm.createElem('div', {
                        id: self.boxId,
                        className: self.class_name,
                        style: boxStyle
                    }, containerElm);
                }
                else {
                    evm.css(self._boxElm, boxStyle);
                }
                var pageData = self._dataInPage = self._getDataByPage();
                //数据准备完成，加载页面视图之前触发
                self._trigger('onInitPreBuildView');
                self._buildView();
                if (pageData) {
                    self.enabled = true;
                }
                self._trigger('onFlipover', {
                    pageTotalNum: self.pageTotalNum
                });
                if (self.isActive) {
                    self.onWidgetFocus();
                }
                else if (self.isMenu) {
                    self.onMenuFocus();
                }
                self._trigger('onInitComplete');
                logIt('debug', this, '<ext.comp.Tiles.init()> ' + this.boxId + '组件初始化结束');
            });
        },
        /**
         * 向后翻页
         */
        next: function (keyAlias) {
            logIt('debug', this, evm.util.common.template('next({keyAlias}) start', {
                keyAlias: keyAlias
            }));

            var self = this;
            if (this.pageIndex >= this.pageTotalNum) {
                return;
            }

            var tile = evm.util.common.extend(true, {}, this.getView(this.pageIndex - 1)[this.activeIndex]);

            this.onBlur();
            this.pageIndex++;
            this._lastDir = '';
            this._fetchData(function () {
                var dataInPage = self._dataInPage = self._getDataByPage();
                if (dataInPage) {
                    self._buildView();
                    self.showView();

                    logIt('debug', self, evm.util.common.template('next({keyAlias}) this.switchAutoPaging = {switchAutoPaging}', {
                        keyAlias: keyAlias,
                        switchAutoPaging: self.switchAutoPaging
                    }));
                    if (!keyAlias) {
                        self.activeIndex = 0;
                        if (self.switchAutoPaging) {
                            self.onFocus();
                        }
                    }
                    else {
                        if (self.switchAutoPaging) {
                            self._dealLoop(keyAlias, tile);
                        }
                    }
                    self._trigger('onFlipover', {
                        pageTotalNum: self.pageTotalNum
                    });
                    if (!self.selectForChange) {
                        self.onChange();
                    }
                    self._trigger('onNextComplete');
                }
                self._trackPageIndex = self.pageIndex;
            });
        },
        /**
         * 加载目标页试图
         * @param targetPageIndex
         * @param {Function} callback
         * @private
         */
        _go: function (targetPageIndex, callback) {
            var self = this;

            this._viewChanged = true;

            this.onBlur();
            this.pageIndex = targetPageIndex;
            this._lastDir = '';
            this._fetchData(function () {
                var dataInPage = self._dataInPage = self._getDataByPage();
                if (dataInPage) {
                    self._buildView();
                    self.showView();

                    self._trigger('onFlipover', {
                        pageTotalNum: self.pageTotalNum
                    });
                    if (!self.selectForChange) {
                        self.onChange();
                    }

                    callback.call(self);
                }
                self._trackPageIndex = self.pageIndex;
            });
        },
        /**
         * 磁贴块项失去焦点
         */
        onBlur: function () {
            logIt('debug', this, '<ext.comp.Tiles> onBlur() 开始');
            var activeElm = this.getActiveElm();
            logIt('debug', this, '<ext.comp.Tiles> onBlur() 获得焦点元素');
            if (activeElm) {
                evm.removeClass(activeElm, 'focus');
                try {
                    if (this.marqueeInst) {
                        var textWrapElm = evm.getElementsByClassName(this.marqueeInst.wrapClassName, activeElm, 'div')[0];
                        var hasDataMarquee = false;
                        if (textWrapElm) {
                            hasDataMarquee = !!evm.getElementsByClassName('evm-marquee-box', textWrapElm, 'div')[0];
                        }
                        if (hasDataMarquee) {
                            this.marqueeInst.stopScroll();
                        }
                    }
                }
                catch (e) {
                    logIt('debug', this, '<ext.comp.Tiles> onBlur() 停止元素文字滚动报错:' + e.message);
                }

                this._trigger('onBlur');
            }
        },
        /**
         * 磁贴块焦点切换
         */
        onChange: function () {
            if (this.selectForChange) {
                this.onMenuBlur();
                this.menuIndex = this.activeIndex;
                this.menuPageIndex = this.pageIndex;
                this.onMenuFocus();
            }
            else {
                this.onMenuBlur();
                this.menuIndex = this.activeIndex;
                this.menuPageIndex = this.pageIndex;
            }
            this._trigger('onChange');
        },
        /**
         * 磁贴块项获取焦点
         */
        onFocus: function () {
            var self = this;
            var activeElm = this.getActiveElm();
            if (activeElm) {
                evm.addClass(activeElm, 'focus');
                if (this.marqueeInst) {

                    //设置文字超长的项获取焦点时滚动显示
                    var activeData = this.getActiveData();
                    var textWrapElm = evm.getElementsByClassName(this.marqueeInst.wrapClassName, activeElm, 'div')[0];
                    var hasDataMarquee = false;
                    if (textWrapElm) {
                        hasDataMarquee = !!evm.getElementsByClassName(this.prefix + '-box', textWrapElm, 'div')[0];
                    }
                    if (hasDataMarquee || (activeData[this.marqueeTextProp] && activeData[this.marqueeTextProp].length > this.textLimit)) {
                        this.marqueeInst.startScroll(textWrapElm, activeData[this.marqueeTextProp].substr(0, this.textLimit) + this.moreReplacement, activeData[this.marqueeTextProp]);
                    }
                }
                this._trigger('onFocus');
            }
        },
        /**
         * 磁贴块菜单项失去焦点
         */
        onMenuBlur: function () {
            if (this.menuPageIndex === this.pageIndex) {
                var viewIndex = this.getViewIndex();
                var viewCounter = this.menuIndex;
                var tileId = this.comp_name + '-view-' + viewIndex + '-tile-' + viewCounter;
                var menuElm = evm.$(tileId);
                if (menuElm) {
                    evm.removeClass(menuElm, 'menu');
                    this._trigger('onMenuBlur', {
                        menuPageIndex: this.menuPageIndex,
                        menuIndex: this.menuIndex,
                        menuElm: menuElm
                    });
                }
            }
        },
        /**
         * 磁贴块菜单项获取焦点
         * 假设menuIndex和menuPageIndex更新完成
         */
        onMenuFocus: function () {
            if (this.menuPageIndex === this.pageIndex) {
                var viewIndex = this.getViewIndex();
                var viewCounter = this.menuIndex;
                var tileId = this.comp_name + '-view-' + viewIndex + '-tile-' + viewCounter;
                var menuElm = evm.$(tileId);
                if (menuElm) {
                    evm.addClass(menuElm, 'menu');
                    this._trigger('onMenuFocus', {
                        menuPageIndex: this.menuPageIndex,
                        menuIndex: this.menuIndex,
                        menuElm: menuElm
                    });
                }
            }
        },
        /**
         * 按确认键
         */
        onSelect: function () {
            if (this.selectForChange) {
                this.onChange();
                this.onFocus();
            }
            this._trigger('onSelect');
        },
        /**
         * 组件获取焦点
         * @returns {boolean}
         */
        onWidgetFocus: function () {
            this._trigger('onWidgetFocus');
            //初始焦点位置设置
            var view = this.getView(this.pageIndex - 1);
            var availablePos = this.activeIndex;
            if (view[this.activeIndex] && view[this.activeIndex].disabled) {
                //如果初始焦点位置项目设置为不可选中，重新设置初始焦点位置
                availablePos = this._findAvailablePos('RIGHT');
                if (availablePos === -1) {
                    availablePos = this._findAvailablePos('LEFT');
                }
                if (availablePos === -1) {
                    availablePos = this._findAvailablePos('DOWN');
                }
                if (availablePos === -1) {
                    availablePos = this._findAvailablePos('UP');
                }
                if (availablePos !== -1 && availablePos !== this.activeIndex) {
                    this.activeIndex = availablePos;
                }
                else {
                    return false;
                }
            }
            this.isActive = true;
            this._lastDir = 'N/A';
            //列表项获取焦点
            this.onFocus();
        },
        /**
         * 组件失去焦点
         * @param keyAlias
         */
        onWidgetBlur: function (keyAlias) {
            if (this._trigger('onWidgetBlur', {
                alias: keyAlias
            })) {
                this.isActive = false;
                this.onBlur();
                if (this.isMenu) {
                    this.onMenuFocus();
                }
            }
            else {
                if (this.autoPagingDir === 'V' && (keyAlias === 'UP' || keyAlias === 'DOWN') ||
                    this.autoPagingDir === 'H' && (keyAlias === 'LEFT' || keyAlias === 'RIGHT')) {
                    if (this.switchLoop) {
                        var tile = evm.util.common.extend(true, {}, this.getView(this.pageIndex - 1)[this.activeIndex]);

                        if (this.pageTotalNum > 1) {
                            //加载目标试图
                            var targetPageIndex = this.pageIndex;
                            if (this.autoPagingDir === 'V') {
                                targetPageIndex = keyAlias === 'UP' ? targetPageIndex - 1 : targetPageIndex + 1;
                            }
                            else {
                                targetPageIndex = keyAlias === 'LEFT' ? targetPageIndex - 1 : targetPageIndex + 1;
                            }
                            if (targetPageIndex < 1) {
                                targetPageIndex = this.pageTotalNum;
                            }
                            else if (targetPageIndex >= this.pageTotalNum) {
                                targetPageIndex = 1;
                            }

                            this._go(targetPageIndex, function () {
                                if (this.switchAutoPaging) {
                                    this._dealLoop(keyAlias, tile);
                                }
                            });
                        }
                        else {
                            if (this.switchAutoPaging) {
                                this._dealLoop(keyAlias, tile);
                            }
                        }
                    }
                }

            }
        },
        /**
         * 页码和页码中的位置转换为数据中的位置
         * @param activeIndex 焦点在单页中的位置
         * @param pageIndex 焦点所在的页码
         * @returns {*}
         */
        page2PosInData: function (activeIndex, pageIndex) {
            var posInData = 0;
            var index = 1;
            while (index < pageIndex) {
                var view = this.getView(index - 1);
                posInData += view.length;
                index++;
            }
            if (posInData >= 0) {
                return posInData + activeIndex;
            }
            else {
                return -1;
            }
        },
        /**
         * 在数据中的位置转换为页码和当前页中的位置
         * @param pos 焦点对应的数据对象在数据中的位置
         * @returns {{}}
         */
        posInData2Page: function (pos) {
            var obj = {};
            var pageIndex = 1;
            while (pos >= 0) {
                obj.activeIndex = pos;
                obj.pageIndex = pageIndex;
                var view = this.getView(pageIndex - 1);
                pos -= view.length;
                pageIndex++;
            }
            return obj;
        },
        /**
         * 向前翻页
         */
        prev: function (keyAlias) {
            this._viewChanged = true;

            var self = this;
            if (this.pageIndex <= 1) {
                return;
            }

            var tile = evm.util.common.extend(true, {}, this.getView(this.pageIndex - 1)[this.activeIndex]);
            this.onBlur();
            //先清空节点内容
            this.pageIndex--;
            this._lastDir = '';
            this._fetchData(function () {
                var dataInPage = self._dataInPage = self._getDataByPage();
                if (dataInPage) {
                    self._buildView();
                    self.showView();
                    if (!keyAlias) {
                        self.activeIndex = dataInPage.length - 1;
                        if (self.switchAutoPaging) {
                            self.onFocus();
                        }
                    }
                    else {
                        if (self.switchAutoPaging) {
                            self._dealLoop(keyAlias, tile);
                        }
                    }

                    self._trigger('onFlipover', {
                        pageTotalNum: self.pageTotalNum
                    });
                    if (!self.selectForChange) {
                        self.onChange();
                    }
                    self._trigger('onPrevComplete');
                }
                self._trackPageIndex = self.pageIndex;
            });
        },
        /**
         * 设置焦点位置
         * @param {number} val
         */
        setActiveIndex: function (val) {
            if (this._dataInPage && this._dataInPage[val]) {
                this.activeIndex = val;
            }
            else {
                this.activeIndex = 0;
            }
        },
        /**
         * 设置数据接口地址
         * @param {string} url
         */
        setFetchDataUrl: function (url) {
            this.fetchDataUrl = url;
        },
        /**
         * 保存接口返回的原始数据对象
         * @param data
         */
        setOriData: function (data) {
            this._oriData = data;
        },
        /**
         * 根据页码显示磁贴块视图
         */
        showView: function () {
            if (this._currentViewElm && this._currentViewElm.style.display != 'block') {
                evm.css(this._currentViewElm, {
                    display: 'block'
                });
            }
        },
        /**
         * 显示组件容器DOM对象
         */
        show: function () {
            this.isVisible = true;
            evm.util.common.show(this._boxElm);
            this._trigger('show');
        },
        /**
         * 隐藏组件容器DOM对象
         */
        hide: function () {
            this.isVisible = false;
            evm.util.common.hide(this._boxElm);
            this._trigger('hide');
        }
    };
    Tiles.prototype.constructor = Tiles;
    /**
     * 默认配置对象
     */
    Tiles.defaults = {
        /*基类配置参数*/
        prefix: 'Ape',
        compName: 'NonRegularTiles',
        parentComp: null,
        /*类本身配置参数*/
        //磁贴块数据
        data: null,
        /**
         * 获取数据方式，once或者paging
         * once表示请求的一次加载数据接口
         * paging表示请求分页加载数据接口
         */
        fetchDataType: 'once',
        //配置获取数据接口地址
        fetchDataUrl: '',
        //用于标识作为菜单项
        isMenu: false,
        //菜单项在单页中的位置
        menuIndex: 0,
        //菜单项所在的页数，起始页为1
        menuPageIndex: 1,
        //数据结构可能不同，需要处理数据
        //默认给出一种栏目数据结构
        //注意：需要计算总页码数
        dataHandler: function (data) {
            var node = data && data.node;
            var ret = null;
            if (node) {
                ret = data[node];
            }
            if (!ret) {
                return null;
            }
            //处理ret，去掉空的对象
            for (var i = 0; i < ret.length; i++) {
                if (evm.util.common.isEmptyObject(ret[i])) {
                    ret.splice(i, 1);
                    i = i - 1;
                }
            }
            //计算总页数
            if (ret && ret instanceof Array) {
                //初始化分页
                if (this.fetchDataType === 'once') {
                    var pageIndex = 0;
                    var i = 0;//计数器，累加每页显示的数量
                    while (i < ret.length) {
                        var view = this.getView(pageIndex);
                        pageIndex++;
                        i += view.length;
                    }
                    this.pageTotalNum = pageIndex;
                }
                else if (this.fetchDataType === 'paging') {
                    //总页码需要根据返回的总项数计算
                }
            }
            return ret;
        },
        //配置磁贴块每页显示数量，位置
        //用二维数组view配置,二维数组每一项都是配置单页展示
        //默认配置view[0]
        /**
         * @example
         * view[0] = [
         {
         left:0,
         top:0,
         width:0,
         height:0
         },
         ...
         ]
         */
        view: [],
        //当前页中的焦点位置
        activeIndex: 0,
        //当前页的页数，起始页为1
        pageIndex: 1,
        //总页数
        pageTotalNum: 1,
        //按确定更新内容
        selectForChange: false,
        //自动翻页标识
        switchAutoPaging: true,
        //自动翻页，水平H或垂直V
        autoPagingDir: 'H',
        //容器元素属性
        boxId: '',
        boxLeft: 0,
        boxTop: 0,
        //磁贴块项的Html生成配置
        //@example
        /**
         [{
			text: 'text',
			callback: function (val) {
				return '<div class="inner">' + val + '</div>';
			}
		}]
         */
        model: [],
        //定义配置方法
        /**
         * userEvent类似自定义的事件触发时候执行
         * 可以配置执行自己的操作
         * @param {string} method 自定义事件名称 onInitComplete,onWidgetBlur,onWidgetFocus,onFocus,onBlur,onSelect
         *        ,onPreFetchData,fetchDataSuccess,onMenuBlur,onMenuFocus
         * @param {object} args 返回一个对象
         */
        userEvent: function (method, args) {
        },
        //显示文字的最大字数
        textLimit: 100,
        //超出最大限制的文字的替换字符
        //默认替换字符为空
        moreReplacement: '',
        //设置滚动对象，用于焦点文字超出时滚动显示
        marqueeInst: null,
        //滚动文字对应的数据对象中的属性名称
        //如果属性名称不是'text'，必须手工配置
        marqueeTextProp: 'text',
        //获取焦点项的文字
        getActiveItemText: function () {
            var activeData = this.getActiveData();
            return activeData.text;
        },
        //没有数据给出默认提示
        //todo:该功能没有实现
        emptyMsg: '',
        emptyImage: '',
        //标识组件是否可用
        enabled: false,
        //扩展属性
        ext: {},
        //组件处于焦点状态，默认非焦点状态
        isActive: false,
        //组件容器DOM是否可见，默认为可见
        isVisible: true,
        //失去焦点时候，在没有其他组件获取焦点
        //是否在组件内部改变捕获焦点
        switchLoop: false
    };
    evm.ext.comp.Tiles = Tiles;
})(this);