/**
 * Win8磁贴块组件
 * @author wuqq@evmtv.com
 * 广告图片磁贴，不可选中
 * 视频位磁贴
 * 图片延时加载
 */
evm.ext.comp.Tiles = function (options) {
	//默认配置属性
	var defaults = {
		parentNode: null,
		originData:null,
		data: null,
		rows: 2,
		cols: 4,
		baseLeft: 0,
		baseTop: 0,
		cellWidth: 100,
		cellHeight: 100,
		gapH: 10,
		gapV: 10,
		model: [],
		//当组件失去焦点时候触发
		onBlur:function(){},
		//当组件获取焦点时候出发
		onFocus:function(){},
		onEvent:function(){}
	};
	evm.extend(defaults, options);
	evm.ext.comp.BaseComponent.call(this, defaults);
	defaults.boxID = this.comp_name;
	evm.extend(this, defaults);
	//共有属性
	this.isBuilt = false; // 是否已生成节点
	this.box_elem = null;
	//记录上一个操作
	this.backDir;
	this.backActiveTileIndex;
	//磁贴焦点索引位置
	this.activeTileIndex = 0;
	//私有属性
	this._cells = null;
	this._tiles = null;
};
evm.extend(evm.ext.comp.Tiles.prototype, {
	_findFreeCell: function (tileData) {
		var startRow = 0;
		var startCol = 0;
		//行优先顺序扫描单元格
		for (var iRow = startRow; iRow < this.rows; iRow++) {
			for (var iCol = startCol; iCol < this.cols; iCol++) {
				var hasFound = true;
				for (var y = 0; (y < tileData.sizeY) && hasFound; y++) {
					for (var x = 0; (x < tileData.sizeX) && hasFound; x++) {
						if (this._cells[iRow + x][iCol + y] && this._cells[iRow + x][iCol + y].hasFilled) {
							hasFound = false;
						}
					}
				}
				if (hasFound) {
					return {
						startRow: iRow,
						startCol: iCol
					}
				}
			}
		}
		return null;
	},
	_findTargetTile:function(row,col){
		var targetTile = null;
		var findCell = this._cells[row] && this._cells[row][col];
		if(findCell){
			targetTile = findCell.relatedTile;
		}
		return targetTile;
	},
	/**
	 * 初始化单元格
	 */
	_initCells: function () {
		this._cells = [];
		for (var iRow = 0; iRow < this.rows; iRow++) {
			this._cells[iRow] = [];
			for (var iCol = 0; iCol < this.cols; iCol++) {
				this._cells[iRow][iCol] = {
					hasFilled: false,
					relatedTile: null
				};
			}
		}
	},
	_initTiles: function () {
		this._tiles = [];
		for (var i = 0; i < this.data.length; i++) {
			var item = this.data[i];
			var size = item.size || '1x1';
			var sizeArray = size.split('x');
			var tileSizeX = 1;
			var tileSizeY = 1;
			if (sizeArray.length === 2) {
				tileSizeX = parseInt(sizeArray[0]);
				tileSizeY = parseInt(sizeArray[1]);
			}
			if (isNaN(tileSizeX) || tileSizeX < 0 || tileSizeX > this.cols) {
				tileSizeX = 1;
			}
			if (isNaN(tileSizeY) || tileSizeY < 0 || tileSizeY > this.rows) {
				tileSizeY = 1;
			}
			var tileData = {
				activeIndex:-1,
				guid:-1,
				sizeX: tileSizeX,
				sizeY: tileSizeY,
				startRow:-1,
				startCol:-1,
				left: 0,
				top: 0,
				width: 0,
				height: 0,
				canSelect: false,
				data: item
			};
			var posInCell = this._findFreeCell(tileData);
			if (posInCell) {
				tileData.activeIndex = i;
				tileData.guid = i;
				tileData.left = this.baseLeft + posInCell.startCol * (this.cellWidth + this.gapH);
				tileData.top = this.baseTop + posInCell.startRow * (this.cellHeight + this.gapV);
				tileData.width = (this.cellWidth + this.gapH) * tileData.sizeX - this.gapH;
				tileData.height = (this.cellHeight + this.gapV) * tileData.sizeY - this.gapV;
				tileData.startRow = posInCell.startRow;
				tileData.startCol = posInCell.startCol;
				//单元格关联磁贴数据对象并设置已填充属性
				for (var y = 0; y < tileData.sizeY; y++) {
					for (var x = 0; x < tileData.sizeX; x++) {
						this._cells[posInCell.startRow + y][posInCell.startCol + x].hasFilled = true;
						this._cells[posInCell.startRow + y][posInCell.startCol + x].relatedTile = tileData;
					}
				}
				this._tiles.push(tileData);
			}
		}
	},
	_trigger:function(type,args){
		if(evm.isFunction(this[type])){
			this[type].apply(this,args);
		}
	},
	action: function (alias) {
		var activeTile = this._tiles[this.activeTileIndex];

		switch(alias){
			case 'UP':
			case 'RIGHT':
			case 'DOWN':
			case 'LEFT':
				this.move(alias);
				break;
			case 'SELECT':
				var evt = {
					type:'select',
					alias:alias,
					tile:activeTile
				};
				this._trigger('onEvent',[evt]);
				break;
			case undefined:
				var evt = {
					type:'focus',
					alias:alias,
					tile:activeTile
				};
				this._trigger('onEvent',[evt]);
				break;
		}
	},
	buildView: function () {
		this.box_elem = evm.$(this.boxID);
		if (!this.box_elem) {
			this.box_elem = evm.createElem('div', {
				id: this.boxID,
				className: this.class_name
			}, this.parentNode || document.body);
		}
		var tpl = '';
		for (var i = 0; i < this._tiles.length; i++) {
			var tile = this._tiles[i];
			tpl += '<div id="'+this.comp_name+'-'+ tile.guid +'" class="mod-comp-tile" style="position:absolute;left:' + tile.left + 'px;top:' + tile.top + 'px;width:' + tile.width + 'px;height:' + tile.height + 'px;line-height:' + tile.height + 'px;">';
			for (var j = 0; j < this.model.length; j++) {
				var val = '';
				if (this.model[j].text) {
					val = tile['data'][this.model[j].text];
				}
				if (evm.isFunction(this.model[j].callback)) {
					val = this.model[j].callback.call(this, val, i, tile.data, this.model[j].text);
				}
				tpl += val;
			}
			tpl += '</div>';
		}
		evm.html(this.box_elem, tpl);
		this.isBuilt = true;
	},
	//最右边界磁贴的索引值
	findRightBound:function(row,col){
		var findCell = this._cells[row] && this._cells[row][col];
		if(findCell && findCell.relatedTile){
			return findCell.relatedTile.activeIndex;
		}

		while(row > 0){
			row--;
			findCell = this._cells[row] && this._cells[row][col];
			if(findCell && findCell.relatedTile){
				return findCell.relatedTile.activeIndex;
			}
		}
		while(col > 0){
			col--;
			findCell = this._cells[row] && this._cells[row][col];
			if(findCell && findCell.relatedTile){
				return findCell.relatedTile.activeIndex;
			}
		}
	},
	getActiveTile:function(index){
		return this._tiles[index];
	},
	init: function () {
		if (!this.data || !this.data.length) {
			throw new Error(this.comp_name + '.init:磁贴块组件没有数据!');
		}
		this._initCells();
		this._initTiles();
		this.buildView();
	},
	move:function(alias){
		var activeTile = this._tiles[this.activeTileIndex];

		var targetTile = null;
		//当前焦点磁贴左上角单元格位置
		var startRow = activeTile.startRow;
		var startCol = activeTile.startCol;

		if(alias == 'LEFT' && this.backDir == 'RIGHT' || alias == 'RIGHT' && this.backDir == 'LEFT' || alias == 'UP' && this.backDir == 'DOWN' || alias == 'DOWN' && this.backDir == 'UP'){
			targetTile = this._tiles[this.backActiveTileIndex];
		}
		else{
			switch(alias){
				case 'UP':
					startRow -= 1;
					break;
				case 'RIGHT':
					startCol += activeTile.sizeX;
					break;
				case 'DOWN':
					startRow += 1;
					break;
				case 'LEFT':
					startCol -= 1;
					break;
			}
			targetTile = this._findTargetTile(startRow,startCol);
		}



		if(targetTile){
			this.backDir = alias;
			this.backActiveTileIndex = this.activeTileIndex;

			var evt = {
				type:'blur',
				alias:alias,
				tile:activeTile
			};
			this._trigger('onEvent',[evt]);

			this.activeTileIndex = targetTile.activeIndex;
			evt = {
				type:'focus',
				alias:alias,
				tile:targetTile
			};
			this._trigger('onEvent',[evt]);
		}
		else{
			this.backDir = null;
			this.backActiveTileIndex = null;

			evt = {
				type:'blur',
				alias:alias,
				tile:activeTile
			}
			this._trigger('onBlur',[evt]);
		}


	},
	setData: function (data) {
		this.data = data;
	}
});