/**
 * 磁贴块组件_v1
 * @dependence:/swatapps/ape/evm.js
 * @author:wuqq@evmtv.com
 * @version:v1.2
 * @created:2016-03-17
 *
 * 2016-05-04 v1.1
 * 1.增加对图片等广告磁贴位置不可选中状态，通过view属性中配置磁贴块的disabled属性为true
 *
 * 2016-05-05 v1.2
 *1.增加两个方法，方便所有数据中的位置与页码和当前页码中的位置之间的转换，即posInData <-> {activeIndex:n,pageIndex:m}
 * todo:
 * 2.自动分页焦点位置最优选择
 */

(function (window) {
	function Tiles(options) {
		//类只能被实例化
		if (this === evm.ext.comp || this === window) {
			return new arguments.callee(options);
		}

		//默认配置参数
		var defaults = {
			/*基类配置参数*/
			prefix: 'Ape',
			compName: 'NonRegularTiles',
			parentComp: null,
			/*类本身配置参数*/
			//磁贴块数据
			data: null,
			fetchDataType: 'once',
			//配置获取数据接口
			fetchDataUrl: '',
			//数据结构可能不同，需要处理获取到的数据
			//默认给出一种栏目数据结构
			//注意：需要计算总页码数
			dataHandler: function (data) {
				var node = data && data.node;
				var ret = null;
				if (node) {
					ret = data[node];
				}
				if (ret && ret instanceof Array) {
					//初始化分页
					if (this.fetchDataType === 'once') {
						var pageIndex = 0;
						var i = 0;//计数器，累加每页显示的数量
						while (i < ret.length) {
							var view = this.getView(pageIndex);
							pageIndex++;
							i += view.length;
						}

						this.pageTotalNum = pageIndex;
					}
					else if (this.fetchDataType === 'paging') {
						//总页码需要根据返回的总项数计算

					}
				}
				return ret;
			},
			//配置磁贴块每页显示数量，位置
			//用二维数组view配置,二维数组每一项都是配置单页展示
			//默认配置view[0]
			/*
			 view[0] = [
			 {
			 left:0,
			 top:0,
			 width:0,
			 height:0
			 },
			 ...
			 ]
			 */
			view: [],
			//当前页中的焦点位置
			activeIndex: 0,
			/*分页属性*/
			pageIndex: 1,
			pageTotalNum: 1,
			//自动翻页标识
			switchAutoPaging: true,
			//自动翻页方向H，水平或者垂直V
			autoPagingDir: 'H',
			//容器元素属性
			boxId: '',
			boxLeft: 0,
			boxTop: 0,
			//列表数据处理
			model: [],
			//定义配置方法
			userEvent: function (method, args) {
			},
			//展示文字最大字数限制
			textLimit: 100,
			//超出最大限制的文字替换字符
			moreReplacement: '',
			//设置滚动对象，用于显示焦点文字超出时滚动显示
			marqueeInst: null,
			//没有数据给出默认提示
			emptyMsg: '',
			emptyImage: '',
			//标识组件是否被选中状态
			enabled: false
		};
		evm.extend(defaults, options || {});
		evm.extend(this, defaults);
		//继承基类
		evm.ext.comp.BaseComponent.call(this, defaults);
		//私有属性
		//接口获取的原始数据对象
		this._oriData = null;
		//容器元素
		this._boxElm = null;
		//每页数据
		this._dataInPage = null;
		//记录上一次操作记录，用于互逆操作返回到刚才位置
		this._lastDir;
		this._lastActivePos;
		//按键的别名
		this._keyAlias = 'N/A';
	}

	evm.extend(Tiles.prototype, {
		/**
		 * @param method
		 * @param {object} obj
		 * @returns {*}
		 * @private
		 */
		_trigger: function (method, obj) {
			var args = {
				activeIndex: this.activeIndex,
				pageIndex: this.pageIndex
			}
			evm.extend(args, obj);

			if (!args.activeData) {
				var activeData = this.getActiveData();
				args.activeData = activeData;
			}
			if (!args.activeElm) {
				var activeElm = this.getActiveElm();
				args.activeElm = activeElm;
			}
			if (!args.alias) {
				args.alias = this._keyAlias;
			}

			if (evm.isFunction(this['userEvent'])) {
				return this['userEvent'].call(this, method, args);
			}
		},
		//获取view数组成员
		getView: function (index) {
			if (!this.view || !(this.view instanceof Array) || this.view.length < 1) {
				throw new Error('未正确配置view');
			}
			var view = this.view[index];
			if (!view) {
				view = this.view[this.view.length - 1];
			}

			return view;
		},
		/**
		 * 获取单页显示数据
		 * @private
		 * @return 未找到数据返回null
		 */
		_getDataByPage: function () {
			var data = null,
				start = 0,
				end = 0,
				i = 0;

			if (!this.data || !(this.data instanceof Array) || !this.data.length) {
				return data;
			}
			if (this.fetchDataType === 'once') {
				for (; i < this.pageIndex; i++) {
					var view = this.getView(i);

					if (i < this.pageIndex - 1) {
						start += view.length;
					}
					end += view.length;
				}
			}
			else if (this.fetchDataType === 'paging') {
				var view = this.getView(this.pageIndex - 1);

				end = view.length;
			}

			if (start !== end) {
				data = this.data.slice(start, end);
			}
			return data;
		},
		/**
		 * 构建展示视图
		 * @param {Array} dataInPage 必须
		 * @private
		 */
		_buildView: function () {
			var self = this;

			function fill(elm) {
				var view = self.getView(self.pageIndex - 1),
					model = self.model,
					i = 0,
					iLen = self._dataInPage.length,
					j = 0,
					jLen = model.length,
					html = '',
					prop;
				for (; i < iLen; i++) {
					var itemStyle = 'position:absolute;'
						+ 'left:' + view[i].left + 'px;'
						+ 'top:' + view[i].top + 'px;'
						+ 'width:' + view[i].width + 'px;'
						+ 'height:' + view[i].height + 'px;';
					if (view[i].picBlur) {
						itemStyle += 'background:transparent url(' + view[i].picBlur + ') no-repeat left top';
					}

					var itemClassName = self.class_name + '-item';
					html += '<div id="' + self.comp_name + '-p' + self.pageIndex + '-item-' + i + '" class="' + itemClassName + '" style="' + itemStyle + '">';
					j = 0;
					if (jLen > 0) {
						for (; j < jLen; j++) {
							if (model[j].text) {
								prop = self._dataInPage[i][model[j].text];
							}
							else {
								throw new Error('需要配置model参数!');
							}
							//如果是文字
							if (model[j].text === 'text' && prop.length > this.textLimit) {
								prop = prop.substr(0, this.textLimit) + this.moreReplacement;
							}
							if (model[j].callback && evm.isFunction(model[j].callback)) {
								prop = model[j].callback.call(self, prop, i, self._dataInPage, model[j].text);
							}
							html += prop;
						}
					}
					else {
						throw new Error('需要配置model参数!');
					}
					html += '</div>';
				}
				evm.html(elm, html);
			}

			if (this._dataInPage && this._dataInPage instanceof Array) {
				var viewElm = evm.$(this.comp_name + '-view-p' + this.pageIndex);
				if (!viewElm) {
					viewElm = evm.createElem('div', {
						id: this.comp_name + '-view-p' + this.pageIndex,
						className: this.class_name + '-view',
						style: 'position:absolute;left:0;top:0;'
					}, this._boxElm);
					fill(viewElm);
				}
				else {
					if (this.fetchDataType === 'once') {
						this.showView(this.pageIndex);
					}
					else if (this.fetchDataType === 'paging') {
						evm.html(viewElm, '');
						fill(viewElm);
					}
				}
			}
		},
		_fetchData: function (callback) {
			var self = this;
			this._trigger('onPreFetchData');
			var self = this;
			if(this.fetchDataType === 'once' && this.data && this._oriData){//一次加载数据并且已经设置了数据和原始数据
				self._trigger('onFetchDataComplete');
				callback.call(this);
				return;
			}
			else if(this.fetchDataType === 'once' && this.data && !this._oriData//一次加载数据，设置了数据，未设置原始数据
					|| this.fetchDataType === 'paging' && this.data && !this._oriData){//分页加载数据，设置了数据，未设置原始数据
				self._trigger('onFetchDataComplete');
				this.setOriData(this.data);
				this.data = this.dataHandler(this.data);
				callback.call(this);
				return;
			}
			if (this.fetchDataUrl) {
				evm.ajax(this.fetchDataUrl, function (data) {
					self._trigger('onFetchDataComplete');
					if (evm.isString(data)) {
						data = eval('(' + data + ')');
					}
					self.setOriData(data);
					self.data = self.dataHandler(data);

					callback.call(self);
				});
			}
		},
		_findAvailablePos: function (keyAlias) {
			var self = this;

			/**
			 * 返回true表示找到更好的合适块
			 * @param currentPos
			 * @param availablePos
			 * @returns {*}
			 */
			function compare(currentPos, availablePos) {
				//焦点块边界坐标
				var activeView = view[self.activeIndex],
					al = activeView.left,
					ar = activeView.left + activeView.width,
					at = activeView.top,
					ab = activeView.top + activeView.height;
				//当前块边界坐标
				var currentView = view[currentPos],
					cl = currentView.left,
					cr = currentView.left + currentView.width,
					ct = currentView.top,
					cb = currentView.top + currentView.height;
				if (availablePos === -1) {
					//初始找到可以选中的项目
					if(currentView.disabled){
						return false;
					}
					else{
						return true;
					}
				}
				//比较跳过不可选中的项目
				if(currentView.disabled){
					return false;
				}
				//最优边界坐标
				var mostView = view[availablePos],
					ml = mostView.left,
					mr = mostView.left + mostView.width,
					mt = mostView.top,
					mb = mostView.top + mostView.height;


				//按照和焦点块交叉面积多为优先
				if (keyAlias === 'LEFT' || keyAlias === 'RIGHT') {
					var dx = al - (view[currentPos].left + view[currentPos].width);
					var dx1 = al - (view[availablePos].left + view[availablePos].width);
					if (keyAlias === 'RIGHT') {
						dx = view[currentPos].left - ar;
						dx1 = view[availablePos].left - ar;
					}
					var dy,
						dy1;
					if (dx < dx1) {
						return true;
					}
					else if (dx === dx1) {
						//判断是否有垂直交叉
						if ((ct > ab || cb < at) && (mt > ab || mb < at)) {
							//当前块和最优块都在焦点块上下边界之外，没有垂直交叉
							if (ct > ab) {
								dy = ct - ab;
							}
							else {
								dy = at - cb;
							}
							if (mt > ab) {
								dy1 = mt - ab;
							}
							else {
								dy1 = at - mb;
							}
							if (dy < dy1) {
								return true;
							}
						}
						else if ((mt > ab || mb < at) && (ct >= at && ct <= ab || cb >= at && cb <= ab || ct <= at && cb >= ab)) {
							//最优块没有垂直交叉，当前块有垂直交叉
							return true;
						}
						else if ((ct >= at && ct <= ab || cb >= at && cb <= ab || ct <= at && cb >= ab) && (mt >= at && mt <= ab || mb >= at && mb <= ab || mt <= at && mb >= ab)) {
							//当前块和最优块都和焦点块有边界延长重叠
							//重叠长度大者优先
							var dl//当前块和焦点块重叠长度
								, dl1;//最优块和焦点块重叠长度

							if (ct <= at && cb >= ab) {
								dl = ab - at;
							}
							else if (ct <= at && cb <= ab) {
								dl = cb - at;
							}
							else if (ct >= at && cb <= ab) {
								dl = cb - ct;
							}
							else if (ct >= at && cb >= ab) {
								dl = ab - ct;
							}

							if (mt <= at && mb >= ab) {
								dl1 = ab - at;
							}
							else if (mt <= at && mb <= ab) {
								dl1 = mb - at;
							}
							else if (mt >= at && mb <= ab) {
								dl1 = mb - mt;
							}
							else if (mt >= at && mb >= ab) {
								dl1 = ab - mt;
							}

							if (dl != null && dl1 != null) {
								if (dl > dl1) {
									return true;
								}

							}
						}
					}
				}
				else if (keyAlias === 'UP' || keyAlias === 'DOWN') {
					var dy = at - (view[currentPos].top + view[currentPos].height);
					var dy1 = at - (view[availablePos].top + view[availablePos].height);
					if (keyAlias === 'DOWN') {
						dy = view[currentPos].top - ab;
						dy1 = view[availablePos].top - ab;
					}
					var dx,
						dx1;
					if (dy < dy1) {
						return true;
					}
					else if (dy === dy1) {
						//判断是否有边界延长重叠
						if ((cr < al || cl > ar) && (mr < al || ml > ar)) {
							//当前块和最优块都在焦点块左右边界之外
							if (al > cr) {
								dx = al - cr;
							}
							else {
								dx = cl - ar;
							}
							if (al > mr) {
								dx1 = al - mr;
							}
							else {
								dx1 = ml - ar;
							}
							if (dx < dx1) {
								return true;
							}
						}
						else if ((mr < al || ml > ar) && (cl >= al && cl <= ar || cr <= ar && cr >= al || cr >= ar && cl <= al)) {
							//最优块左右边界之外,当前块和焦点块有边界延长重叠
							return true;
						}
						else if ((cl >= al && cl <= ar || cr <= ar && cr >= al || cl <= al && cr >= ar) && (ml >= al && ml <= ar || mr <= ar && mr >= al || ml <= al && mr >= ar)) {
							//当前块和最优块都和焦点块有边界延长重叠
							//重叠长度大者优先
							var dl,
								dl1;

							if (cl >= al && cr <= ar) {
								dl = cr - cl;
							}
							else if (cl <= al && cr >= ar) {
								dl = ar - al;
							}
							else if (cl <= al && cr <= ar) {
								dl = cr - al;
							}
							else if (cl >= al && cr >= ar) {
								dl = ar - cl;
							}

							if (ml >= al && mr <= ar) {
								dl1 = mr - ml;
							}
							else if (ml <= al && mr >= ar) {
								dl1 = ar - al;
							}
							else if (ml <= al && mr <= ar) {
								dl1 = mr - al;
							}
							else if (ml >= al && mr >= ar) {
								dl1 = ar - ml;
							}

							if (dl != null && dl1 != null) {
								if (dl > dl1) {
									return true;
								}
							}
						}
					}
				}

				/*
				 * 比较逻辑-old
				 //比较当前块和找到的最优块中心点与当前焦点块中心点距离，取最小者
				 //当前焦点块中心点坐标
				 var cx = x + view[self.activeIndex].width / 2;
				 var cy = y + view[self.activeIndex].height / 2;
				 //当前块中心点坐标
				 var cx1 = view[currentPos].left + view[currentPos].width / 2;
				 var cy1 = view[currentPos].top + view[currentPos].height / 2;
				 //最优块中心点坐标
				 var cx2 = view[availablePos].left + view[availablePos].width / 2;
				 var cy2 = view[availablePos].top + view[availablePos].height / 2;

				 if (keyAlias === 'LEFT' || keyAlias === 'RIGHT') {
				 if (Math.abs(cy1 - cy) < Math.abs(cy2 - cy) && Math.abs(cx1 - cx) <= Math.abs(cx2 - cx)) {
				 return true;
				 }
				 else if (Math.abs(cy1 - cy) === Math.abs(cy2 - cy) && Math.abs(cx1 - cx) <= Math.abs(cx2 - cx)) {
				 return true;
				 }
				 }
				 else if (keyAlias === 'UP' || keyAlias === 'DOWN') {
				 if (Math.abs(cx1 - cx) < Math.abs(cx2 - cx) && Math.abs(cy1 - cy) <= Math.abs(cy2 - cy)) {
				 return true;
				 }
				 else if (Math.abs(cx1 - cx) <= Math.abs(cx2 - cx) && Math.abs(cy1 - cy) <= Math.abs(cy2 - cy)) {
				 return true;
				 }
				 }
				 return false;
				 */
			}

			var i = 0,
				view = this.getView(this.pageIndex - 1),
				x = view[this.activeIndex].left,
				y = view[this.activeIndex].top,
				dx = 0,
				dy = 0,
				available = null,
				availablePos = -1;
			for (; i < this._dataInPage.length; i++) {
				if (i === this.activeIndex) {
					continue;
				}
				var item = this._dataInPage[i];

				dx = view[i].left - x;
				dy = view[i].top - y;

				if (keyAlias === 'LEFT') {
					dx += view[i].width;
					if (dx <= 0) {
						if (compare(i, availablePos)) {
							available = item;
							availablePos = i;
						}
					}
				}
				else if (keyAlias === 'RIGHT') {
					dx -= view[this.activeIndex].width;
					if (dx >= 0) {
						if (compare(i, availablePos)) {
							available = item;
							availablePos = i;
						}
					}
				}
				else if (keyAlias === 'UP') {
					dy += view[i].height;
					if (dy <= 0) {
						if (compare(i, availablePos)) {
							available = item;
							availablePos = i;
						}
					}
				}
				else if (keyAlias === 'DOWN') {
					dy -= view[this.activeIndex].height;
					if (dy >= 0) {
						if (compare(i, availablePos)) {
							available = item;
							availablePos = i;
						}
					}
				}
			}
			return availablePos;
		},
		_move: function (keyAlias, myEvent) {
			var availablePos;
			if (this._lastDir === 'LEFT' && keyAlias === 'RIGHT' || this._lastDir === 'RIGHT' && keyAlias === 'LEFT') {
				availablePos = this._lastActivePos;
			}
			else if (this._lastDir === 'UP' && keyAlias === 'DOWN' || this._lastDir === 'DOWN' && keyAlias === 'UP') {
				availablePos = this._lastActivePos;
			}
			else {
				availablePos = this._findAvailablePos(keyAlias);
			}

			if (availablePos !== -1 && availablePos !== this.activeIndex) {
				this._lastDir = keyAlias;
				this._lastActivePos = this.activeIndex;

				this.onBlur();
				this.activeIndex = availablePos;
				this.onFocus();
			}
			else {
				if (this.switchAutoPaging) {
					if (this.autoPagingDir === 'H') {
						if (keyAlias === 'LEFT' && this.pageIndex > 1) {
							this.prev();
						}
						else if (keyAlias === 'RIGHT' && this.pageIndex < this.pageTotalNum) {
							this.next();
						}
						else {
							this.onWidgetBlur(keyAlias);
						}
					}
					else if (this.autoPagingDir === 'V') {
						if (keyAlias === 'UP' && this.pageIndex > 1) {
							this.prev();
						}
						else if (keyAlias === 'DOWN' && this.pageIndex < this.pageTotalNum) {
							this.next();
						}
						else {
							this.onWidgetBlur(keyAlias);
						}
					}
					else {
						this.onWidgetBlur(keyAlias);
					}
				}
				else {
					this.onWidgetBlur(keyAlias);
				}
			}
		},
		action: function (keyAlias, myEvent) {
			this._keyAlias = keyAlias;
			switch (keyAlias) {
				case 'UP':
				case 'RIGHT':
				case 'DOWN':
				case 'LEFT':
					this._move(keyAlias, myEvent);
					break;
				case 'SELECT':
					this.onSelect();
					break;
			}
		},
		init: function () {
			this._trigger('onPreInit');
			var self = this;
			this._fetchData(function () {
				//准备容器元素
				var parentElm = evm.$('container') || document.body;
				if (!evm.isString(self.boxId)) {
					self.boxId = self.comp_name;
				}
				self._boxElm = evm.$(self.boxId);
				var boxStyle = 'position:absolute;'
					+ 'left:' + self.boxLeft + 'px;'
					+ 'top:' + self.boxTop + 'px;';
				if (!self._boxElm) {
					self._boxElm = evm.createElem('div', {
						id: self.boxId,
						className: self.class_name,
						style: boxStyle
					});
				}
				else {
					evm.css(self._boxElm, boxStyle);
				}

				var pageData = self._dataInPage = self._getDataByPage();

				if (self.emptyMsg || self.emptyImage) {
					if (!pageData) {
						pageData = [
							{
								text: self.emptyMsg,
								picBlur: self.emptyImage
							}
						];
					}
				}
				//数据准备完成，加载页面视图之前触发
				self._trigger('onInitPreBuildView');

				self._buildView();

				if (pageData) {
					self.enabled = true;
				}

				self._trigger('onFlipover', {
					pageTotalNum: self.pageTotalNum
				});
				self._trigger('onInitComplete');
			});
		},
		setOriData: function (data) {
			this._oriData = data;
		},
		getOriData: function () {
			return this._oriData;
		},
		getActiveData: function () {
			return this._dataInPage && this._dataInPage[this.activeIndex];
		},
		prev: function () {
			var self = this;

			if (this.pageIndex <= 1) {
				return;
			}

			this.onBlur();
			this.hideView(this.pageIndex);
			this.pageIndex--;
			this._lastDir = '';
			this._fetchData(function () {
				var dataInPage = self._dataInPage = self._getDataByPage();
				if (dataInPage) {
					self._buildView();

					self.activeIndex = dataInPage.length - 1;
					self.showView(self.pageIndex);
					if (self.switchAutoPaging) {
						self.onFocus();
					}

					self._trigger('onFlipover', {
						pageTotalNum: self.pageTotalNum
					});
				}
			});
		},
		next: function () {
			var self = this;

			if (this.pageIndex >= this.pageTotalNum) {
				return;
			}
			this.onBlur();
			this.hideView(this.pageIndex);
			this.pageIndex++;
			this._lastDir = '';
			this._fetchData(function () {
				var dataInPage = self._dataInPage = self._getDataByPage();
				if (dataInPage) {
					self._buildView();
					self.activeIndex = 0;
					self.showView(self.pageIndex);
					if (self.switchAutoPaging) {
						self.onFocus();
					}

					self._trigger('onFlipover', {
						pageTotalNum: self.pageTotalNum
					});
				}
			});
		},
		showView: function (pageIndex) {
			var id = this.comp_name + '-view-p' + pageIndex;
			var viewBoxElm = evm.$(id);
			if (viewBoxElm) {
				evm.css(viewBoxElm, {
					display: 'block'
				});
			}

		},
		hideView: function (pageIndex) {
			var id = this.comp_name + '-view-p' + pageIndex;
			var viewBoxElm = evm.$(id);
			if (viewBoxElm) {
				evm.css(viewBoxElm, {
					display: 'none'
				});
			}

		},
		destroy: function () {
			this.enabled = false;
			this.activeIndex = 0;
			this.pageIndex = 1;
			this._lastDir = null;
			this._lastActivePos = -1;

			//清空DOM
			evm.html(this._boxElm, '');

			//清空数据
			this.data = null;
			this._oriData = null;

			this._trigger('onDestroy');
		},
		getActiveElm: function () {
			var activeElm = evm.$(this.comp_name + '-p' + this.pageIndex + '-item-' + this.activeIndex);
			return activeElm;
		},
		onFocus: function () {
			var activeElm = this.getActiveElm();
			if (activeElm) {
				evm.addClass(activeElm, 'focus');
				if (this.marqueeInst) {
					//设置文字超长的项获取焦点时滚动显示
					var activeData = this.getActiveData();
					var textWrapElm = evm.getElementsByClassName(this.marqueeInst.wrapClassName, activeElm, 'div')[0];
					if (textWrapElm && textWrapElm.innerText && textWrapElm.innerText.length > this.textLimit) {
						this.marqueeInst.startScroll(textWrapElm, textWrapElm.innerText.substr(0, this.textLimit) + this.moreReplacement, textWrapElm.innerText);
					}
				}
				this._trigger('onFocus');
			}
		},
		onBlur: function () {
			var activeElm = this.getActiveElm();
			if (activeElm) {
				evm.removeClass(activeElm, 'focus');
				if (this.marqueeInst) {
					this.marqueeInst.stopScroll();
				}
				this._trigger('onBlur');
			}
		},
		onSelect: function () {
			this._trigger('onSelect');
		},
		onWidgetFocus: function () {
			this._trigger('onWidgetFocus');
			//初始焦点位置设置
			var view = this.getView(this.pageIndex - 1);
			var availablePos = this.activeIndex;
			if(view[this.activeIndex].disabled){
				//如果初始焦点位置项目设置为不可选中，重新设置初始焦点位置
				availablePos = this._findAvailablePos('RIGHT');
				if(availablePos === -1){
					availablePos = this._findAvailablePos('LEFT');
				}
				if(availablePos === -1){
					availablePos = this._findAvailablePos('DOWN');
				}
				if(availablePos === -1){
					availablePos = this._findAvailablePos('UP');
				}

				if(availablePos !== -1 && availablePos !== this.activeIndex){
					this.activeIndex = availablePos;
				}
				else{
					return false;
				}
			}

			//列表项获取焦点
			this.onFocus();
		},
		onWidgetBlur: function (keyAlias) {
			if (this._trigger('onWidgetBlur', {
					alias: keyAlias
				})) {
				this.onBlur();
			}
		},
		setFetchDataUrl: function (url) {
			this.fetchDataUrl = url;
		},
		//在数据中的位置转换为页码和当前页中的位置
		posInData2Page:function(pos){
			var obj = {};
			var pageIndex = 1;
			while(pos >= 0){
				obj.activeIndex = pos;
				obj.pageIndex = pageIndex;

				var view = this.getView(pageIndex - 1);
				pos -= view.length;
				pageIndex++;
			}
			return obj;
		},
		//页码和页码中的位置转换为数据中的位置
		page2PosInData:function(activeIndex,pageIndex){
			var posInData = 0;
			var index = 1;
			while(index < pageIndex){
				var view = this.getView(index - 1);
				posInData += view.length;

				index++;
			}
			if(posInData >= 0)
			{
				return posInData + activeIndex;
			}
			else{
				return -1;
			}
		}
	});

	evm.ext.comp.Tiles = Tiles;
})(this);