(function (window, undefined) {
    /**
     * 固定焦点的垂直列表组件
     * @requires /swatapps/ape/evm.js
     * @requires /swatapps/ape/util/marquee/main.js
     * @requires /swatapps/ape/util/common.js
     *
     * @author wuqq@evmtv.com
     * @version 1.0.0
     *
     * @alias evm.ext.comp.VListFixFocus
     * @param {object} options 初始化类时候传入的配置对象，具体成员可以参考默认配置对象{@link VListFixFocus.defaults}
     * @constructor
     */
    function VListFixFocus(options) {
        evm.util.common.logger.debug('调试开始');
        //类只能被实例化
        if (this === evm.ext.comp || this === window) {
            return new arguments.callee(options);
        }
        var config = evm.util.common.extend(true, {}, VListFixFocus.defaults, options || {});
        evm.util.common.extend(this, config);
        //继承基类
        evm.ext.comp.BaseComponent.call(this, config);
        /*私有属性*/
        //保存初始配置
        this._config = config;
        //容器元素
        this._boxElm = null;
        //接口返回的原始数据
        this._oriData = null;
        //按键别名
        this._keyAlias = 'N/A';
        //固定焦点以上数据项数量
        this._upSize = 0;
        //固定焦点以下数据项数量
        this._downSize = 0;
    }

    _debounceTimeId = null;

    VListFixFocus.prototype = {
        /**
         * @private
         * 构建视图
         */
        _buildView: function () {
            var data = this.data,
                orderedData = [],
                model = this.model,
                i = 0,
                iLen = data.length,
                j = 0,
                jLen = model.length,
                html = '',
                prop,
                upSizeInVisual = this.fixFocusPosInVisual,
                downSizeInVisual = this.visualLength - this.fixFocusPosInVisual - 1,
                upSize = 0,
                downSize = 0;
            //对数据排序生成DOM节点
            //可视区域优先填满
            //可视区域数据填充不够，以尽量填充满为原则
            //固定焦点上下尽量对称分布
            if (iLen >= this.visualLength) {
                var startIndexInData = this.fixFocusPosInData - upSizeInVisual;
                if (startIndexInData < 0) {
                    startIndexInData = (startIndexInData + iLen) % iLen;
                }
                orderedData = orderedData.concat(this.data.slice(startIndexInData));
                orderedData = orderedData.concat(this.data.slice(0, iLen - orderedData.length));
                upSize = upSizeInVisual;
                downSize = downSizeInVisual;
            } else {
                var upData = data.slice(0, this.fixFocusPosInData),
                    downData = data.slice(this.fixFocusPosInData + 1);
                orderedData = data.slice(this.fixFocusPosInData, this.fixFocusPosInData + 1);
                while (upData.length > 0 || downData.length > 0) {
                    if (upData.length > 0) {
                        if (upSize < upSizeInVisual) {
                            orderedData.unshift(upData.pop());
                            upSize++;
                        }
                    } else if (downData.length > 0) {
                        if (upSize < upSizeInVisual) {
                            orderedData.unshift(downData.pop());
                            upSize++;
                        }
                    }
                    if (downData.length > 0) {
                        if (downSize < downSizeInVisual) {
                            orderedData.push(downData.shift());
                            downSize++;
                        }
                    } else if (upData.length > 0) {
                        if (downSize < downSizeInVisual) {
                            orderedData.push(upData.shift());
                            downSize++;
                        }
                    }
                }
            }
            this._upSize = upSize;
            this._downSize = downSize;
            //orderedData的初始位置相对于可视区间初始位置的偏移量
            var offset = this.fixFocusPosInVisual - this._upSize;
            for (; i < iLen; i++) {
                var itemStyle = 'position:absolute;'
                    + 'width:' + this.itemWidth + 'px;'
                    + 'height:' + this.itemHeight + 'px;'
                    + 'line-height:' + this.itemHeight + 'px;';
                if (offset + i < this.view.length) {
                    itemStyle += 'top:' + this.view[offset + i].top + 'px;';
                    itemStyle += 'left:' + this.view[offset + i].left + 'px;';
                } else {
                    itemStyle += 'left:0;';
                    itemStyle += 'top:' + (-this.itemHeight) + 'px;';
                }
                var itemClassName = this.class_name + '-item';
                var id = this.comp_name + '-item-' + i;
                //数据项保存对应的元素id属性
                orderedData[i].elemId = id;
                html += '<div id="' + id + '" class="' + itemClassName + '" style="' + itemStyle + '">';
                j = 0;
                if (jLen > 0) {
                    for (; j < jLen; j++) {
                        if (model[j].text) {
                            prop = orderedData[i][model[j].text];
                        } else {
                            throw new Error('需要配置model参数!');
                        }
                        //如果是文字
                        if ((model[j].text === this.marqueeTextProp) && prop.length > this.textLimit) {
                            prop = prop.substr(0, this.textLimit) + this.moreReplacement;
                        }
                        if (model[j].callback && evm.isFunction(model[j].callback)) {
                            prop = model[j].callback.call(this, prop, i, orderedData, model[j].text);
                        }
                        html += prop;
                    }
                } else {
                    throw new Error('需要配置model参数!');
                }
                html += '</div>';
            }
            evm.html(this._boxElm, html);
        },
        /**
         * @param method
         * @param {object} obj
         * @returns {*}
         * @private
         */
        _trigger: function (method, obj) {
            var args = {};
            evm.extend(args, obj);
            if (!args.activeData) {
                var activeData = this.getActiveData();
                args.activeData = activeData;
            }
            if (!args.activeElm) {
                var activeElm = this.getActiveElm();
                args.activeElm = activeElm;
            }
            if (!args.alias) {
                args.alias = this._keyAlias;
            }
            if (evm.isFunction(this['userEvent'])) {
                return this['userEvent'].call(this, method, args);
            }
        },
        _fetchData: function (callback) {
            this._trigger('onPreFetchData');
            var self = this;
            if (!evm.util.common.isEmptyValue(this.data)) {
                if (this.fetchDataType === 'once' && evm.util.common.isEmptyValue(this.getOriData())
                ) {
                    self._trigger('onFetchDataComplete');
                    this.setOriData(this.data);
                    this.data = this.dataHandler(this.data);
                    callback.call(this);
                    return;
                } else if (this.fetchDataType === 'once') {
                    self._trigger('onFetchDataComplete');
                    callback.call(this);
                    return;
                }
            }
            if (this.fetchDataUrl) {
                evm.ajax(this.fetchDataUrl, function (data) {
                    self._trigger('onFetchDataComplete');
                    data = evm.util.common.str2json(data);
                    self.setOriData(data);
                    self.data = self.dataHandler(data);
                    callback.call(self);
                });
            }
        },
        /**
         * 修改元素的css属性
         * @param el
         * @param obj
         * @private
         */
        _changeProperty: function (el, obj) {
            var deepClone = evm.util.common.extend(true, {}, obj),
                prop;
            for (prop in deepClone) {
                switch (prop) {
                    case 'left':
                    case 'top':
                    case 'fontSize':
                        deepClone[prop] = deepClone[prop] + 'px';
                        break;
                }
            }
            if (!evm.util.common.isEmptyObject(deepClone)) {
                evm.css(el, deepClone);
            }
        },
        /**
         * 按键上下移动的方法
         * @param {string} keyAlias - 移动方向字符串DOWN或UP
         * @param {string} myEvent - 按键监听的event对象
         */
        moveV: function (keyAlias, myEvent) {
            var self = this,
                i,
                elem,
                elemId,
                viewPos,
                indexInView,
                indexInData;
            if (keyAlias !== 'DOWN' && keyAlias !== 'UP') {
                return;
            }
            this.onMenuBlur();
            this.onBlur();
            var len = this._upSize + 1 + this._downSize,
                dLen = this.data.length;
            //第一项在可视区域的位置
            var startIndexInVisual = this.fixFocusPosInVisual - this._upSize;
            var startIndexInData = this.fixFocusPosInData - this._upSize;
            if (keyAlias === 'DOWN') {
                for (i = 0; i < len; i++) {
                    //i对应到view位置
                    indexInView = i + startIndexInVisual;
                    indexInData = i + startIndexInData;
                    var afterLastElem;
                    if (i === len) {
                    } else {
                        elemId = this.data[(indexInData + dLen) % dLen].elemId;
                        elem = evm.$(elemId);
                        this.openCSS3Anim && (elem.style.webkitTransitionDuration = '0s');
                        this._changeProperty(elem, this.view[indexInView]);
                    }
                    if (i === 0) {
                        this.openCSS3Anim && (elem.style.webkitTransitionDuration = this.duration);
                        var preIndex = indexInView - 1;
                        viewPos = this.view[preIndex];
                        if (viewPos) {
                            this._changeProperty(elem, viewPos);
                        } else {
                            this._changeProperty(elem, this.prePos);
                        }
                    } else if (i === len) {
                    } else {
                        this.openCSS3Anim && (elem.style.webkitTransitionDuration = this.duration);
                        this._changeProperty(elem, this.view[indexInView - 1]);
                    }
                }
                if (this.openCSS3Anim) {
                    setTimeout(function () {
                        if (i === len) {
                            var afterLastElemId = self.data[(startIndexInData + len) % dLen].elemId;
                            afterLastElem = evm.$(afterLastElemId);
                            afterLastElem.style.webkitTransitionDuration = '0s'
                            var viewPos = self.view[startIndexInVisual + len];
                            if (viewPos) {
                                self._changeProperty(afterLastElem, viewPos);
                            } else {
                                self._changeProperty(afterLastElem, self.afterPos);
                            }
                        }
                    }, 100);
                    setTimeout(function () {
                        afterLastElem.style.webkitTransitionDuration = self.duration
                        var lastIndex = startIndexInVisual + len - 1;
                        self._changeProperty(afterLastElem, self.view[lastIndex]);
                    }, 150);
                } else {
                    if (i === len) {
                        var afterLastElemId = self.data[(startIndexInData + len) % dLen].elemId;
                        afterLastElem = evm.$(afterLastElemId);
                        var viewPos = self.view[startIndexInVisual + len];
                        if (viewPos) {
                            self._changeProperty(afterLastElem, viewPos);
                        } else {
                            self._changeProperty(afterLastElem, self.afterPos);
                        }
                    }

                    var lastIndex = startIndexInVisual + len - 1;
                    self._changeProperty(afterLastElem, self.view[lastIndex]);
                }
            } else {
                for (i = 0; i < len; i++) {
                    var preElm;
                    indexInView = i + startIndexInVisual;//i对应到view位置
                    indexInData = startIndexInData + i;
                    if (i === len) {
                    } else {
                        elemId = this.data[(indexInData + dLen) % dLen].elemId;
                        elem = evm.$(elemId);
                        this.openCSS3Anim && (elem.style.webkitTransitionDuration = '0s');
                        this._changeProperty(elem, this.view[indexInView]);
                    }
                    if (i === len) {
                    } else if (i === len - 1) {
                        this.openCSS3Anim && (elem.style.webkitTransitionDuration = this.duration);
                        viewPos = this.view[startIndexInVisual + len];
                        if (viewPos) {
                            this._changeProperty(elem, viewPos);
                        } else {
                            this._changeProperty(elem, this.afterPos);
                        }
                    } else {
                        this.openCSS3Anim && (elem.style.webkitTransitionDuration = this.duration);
                        this._changeProperty(elem, this.view[indexInView + 1]);
                    }
                }
                if (this.openCSS3Anim) {
                    setTimeout(function () {
                        //准备从顶部向下切入的位置
                        var preIndex = (startIndexInData - 1 + dLen) % dLen;
                        var preElmId = self.data[preIndex].elemId;
                        preElm = evm.$(preElmId);
                        preElm.style.webkitTransitionDuration = '0s';
                        var viewPos = self.view[startIndexInVisual - 1];
                        if (viewPos) {
                            self._changeProperty(preElm, viewPos);
                        } else {
                            self._changeProperty(preElm, self.prePos);
                        }
                    }, 100);
                    setTimeout(function () {
                        preElm.style.webkitTransitionDuration = self.duration;
                        self._changeProperty(preElm, self.view[startIndexInVisual]);
                    }, 150);
                } else {
                    //准备从顶部向下切入的位置
                    var preIndex = (startIndexInData - 1 + dLen) % dLen;
                    var preElmId = self.data[preIndex].elemId;
                    preElm = evm.$(preElmId);
                    var viewPos = self.view[startIndexInVisual - 1];
                    if (viewPos) {
                        self._changeProperty(preElm, viewPos);
                    } else {
                        self._changeProperty(preElm, self.prePos);
                    }

                    self._changeProperty(preElm, self.view[startIndexInVisual]);
                }
            }
            //更新固定焦点在数据中的位置
            if (keyAlias === 'DOWN') {
                this.fixFocusPosInData = (this.fixFocusPosInData + 1) % dLen;
            } else {
                this.fixFocusPosInData = (this.fixFocusPosInData + dLen - 1) % dLen;
            }
            this.onMenuFocus();
            this.onFocus();
            this.onChange();
        },
        /**
         * 焦点位置切换
         */
        onChange: function () {
            this._trigger('onChange');
        },
        /**
         * 列表项获取焦点
         */
        onFocus: function () {
            if (this.isMenu && this.selectForChange) {
                //设置列表项的菜单高亮
                this.onMenuFocus();
            }
            var activeElm = this.getActiveElm();
            var activeData = this.getActiveData();
            if (activeElm) {
                evm.addClass(activeElm, 'focus');
                if (this.marqueeInst) {
                    //设置文字超长的项获取焦点时滚动显示
                    var textWrapElm = evm.getElementsByClassName(this.marqueeInst.wrapClassName, activeElm, 'div')[0];
                    var hasDataMarquee = false;
                    if (textWrapElm) {
                        hasDataMarquee = !!textWrapElm.getAttribute('data-marquee');
                    }
                    if (hasDataMarquee || (activeData[this.marqueeTextProp] && activeData[this.marqueeTextProp].length > this.textLimit)) {
                        this.marqueeInst.startScroll(textWrapElm, activeData[this.marqueeTextProp].substr(0, this.textLimit) + this.moreReplacement, activeData[this.marqueeTextProp]);
                    }
                }
                this._trigger('onFocus', {
                    activeData: activeData,
                    activeElm: activeElm
                });
            }
        },
        /**
         * 列表项失去焦点
         */
        onBlur: function () {
            var activeElm = this.getActiveElm();
            var activeData = this.getActiveData();
            if (activeElm) {
                evm.removeClass(activeElm, 'focus');
                if (this.marqueeInst) {
                    this.marqueeInst.stopScroll();
                }
                this._trigger('onBlur', {
                    activeData: activeData,
                    activeElm: activeElm
                });
            }
        },
        /**
         * 组件按键事件入口
         *
         * @private
         * @param keyAlias
         * @param myEvent
         * @returns {number}
         */
        action: function (keyAlias, myEvent) {
            this._keyAlias = keyAlias;
            var res = 0;
            var self = this;
            switch (keyAlias) {
                case 'UP':
                case 'DOWN':
                    if (_debounceTimeId) {
                        clearTimeout(_debounceTimeId);
                    }
                    _debounceTimeId = setTimeout(function () {
                        self.moveV(keyAlias, myEvent);
                    }, this.keyDefenceDuration);
                    break;
                case 'LEFT':
                    res = this.onWidgetBlur('LEFT');
                    break;
                case 'RIGHT':
                    res = this.onWidgetBlur('RIGHT');
                    break;
                case 'SELECT':
                    res = this.onSelect();
                    break;
                default:
                    var args = {
                        alias: keyAlias,
                        event: myEvent
                    };
                    res = this._trigger('default', args);
                    break;
            }
            return res;
        },
        /**
         * 组件初始化
         */
        init: function () {
            this._trigger('onPreInit');
            var self = this;
            if (!self.view || !self.view.length) {
                throw new Error('固定焦点的垂直列表组件的view需要配置!');
            }
            //处理列表数据
            this._fetchData(function () {
                if (!(self.data instanceof Array)) {
                    throw new Error('传入列表数据应该为数组!');
                }
                //容器元素
                if (!self.boxId) {
                    throw new Error('配置参数boxId不能为空!');
                }
                self._boxElm = evm.$(self.boxId);
                var boxStyle = 'position:absolute;'
                    + 'left:' + this.boxLeft + 'px;'
                    + 'top:' + this.boxTop + 'px;'
                    + 'display:' + (self.isVisible ? 'block' : 'none') + ';';
                if (!self._boxElm) {
                    var parentNode = evm.$('container') || document.body;
                    self._boxElm = evm.createElem('div', {id: self.boxId, style: boxStyle}, parentNode);
                    //清空容器
                    evm.html(self._boxElm, '');
                } else {
                    evm.css(self._boxElm, boxStyle);
                }
                self._buildView();
                if (this.data && this.data.length) {
                    this.enabled = true;
                }
                if (self.isActive) {
                    self.onWidgetFocus();
                } else if (self.isMenu) {
                    self.onMenuFocus();
                }
                self._trigger('onInitComplete');
            });
        },
        /**
         * 获取焦点位置对应的数据对象
         * @returns {*}
         */
        getActiveData: function () {
            if (!this.data) {
                return;
            }
            var ret = this.data[this.fixFocusPosInData];
            return ret;
        },
        /**
         * 获取焦点对应的DOM对象
         * @returns {*}
         */
        getActiveElm: function () {
            var activeData = this.getActiveData();
            if (activeData && activeData.elemId) {
                return evm.$(activeData.elemId);
            }
            return null;
        },
        /**
         * 对象恢复为初始状态
         */
        destroy: function () {
            //初始化一些属性
            this._oriData = null;
            this.data = null;
            this.enabled = false;
            this.fixFocusPosInData = this._config.fixFocusPosInData;
            this.fixFocusPosInVisual = this._config.fixFocusPosInVisual;
            this._leftSize = 0;
            this._rightSize = 0;
            evm.html(this._boxElm, '');
            this._trigger('onDestroy');
        },
        /**
         * 设置异步获取数据的请求地址
         * @param url
         */
        setFetchDataUrl: function (url) {
            this.fetchDataUrl = url;
        },
        /**
         * 删除列表项的菜单状态
         */
        onMenuBlur: function () {
            var menuElm = this.getActiveElm();
            if (menuElm) {
                evm.removeClass(menuElm, 'menu');
                this._trigger('onMenuBlur', {
                    menuPos: this.fixFocusPosInVisual,
                    menuElm: menuElm
                });
            }
        },
        /**
         * 设置列表菜单状态
         */
        onMenuFocus: function () {
            var menuElm = this.getActiveElm();
            if (menuElm) {
                evm.addClass(menuElm, 'menu');
                this._trigger('onMenuFocus', {
                    menuPos: this.fixFocusPosInVisual,
                    menuElm: menuElm
                });
            }
        },
        /**
         * 按确定键操作
         */
        onSelect: function () {
            if (this.selectForChange) {
                this.onChange();
                this.onFocus();
            }
            this._trigger('onSelect');
        },
        /**
         * 设置接口返回的原始数据对象
         * @param data
         */
        setOriData: function (data) {
            this._oriData = data;
        },
        /**
         * 获取接口返回的原始数据对象
         * @returns {null|*}
         */
        getOriData: function () {
            return this._oriData;
        },
        /**
         * 组件获取焦点时候执行
         */
        onWidgetFocus: function () {
            this._trigger('onWidgetFocus');
            //列表项获取焦点
            this.onFocus();
            this.isActive = true;
        },
        /**
         * 组件失去焦点时候执行
         * @param dir
         */
        onWidgetBlur: function (dir) {
            if (this._trigger('onWidgetBlur', {
                alias: dir
            })) {
                this.onBlur();
                if (this.isMenu) {
                    this.onMenuFocus();
                }
                this.isActive = false;
            }
        },
        /**
         * 获取容器元素
         */
        getBoxElm: function () {
            return this._boxElm;
        },
        /**
         * 显示容器元素
         */
        show: function () {
            this.isVisible = true;
            evm.util.common.show(this._boxElm);
            this._trigger('show');
        },
        /**
         * 隐藏容器元素
         */
        hide: function () {
            this.isVisible = false;
            evm.util.common.hide(this._boxElm);
            this._trigger('hide');
        },
        /**
         * 设置焦点在可视区间的位置
         * @param {number} val - [0,visualLength]
         */
        setFixFocusPosInVisual: function (val) {
            this.fixFocusPosInVisual = val;
        },
        /**
         * 设置焦点对应的数据中的位置
         * @param {number} val
         */
        setFixFocusPosInData: function (val) {
            this.fixFocusPosInData = val;
        }
    };
    VListFixFocus.prototype.constructor = VListFixFocus;
    /**
     * 默认配置对象
     */
    VListFixFocus.defaults = {
        /*基类配置参数*/
        prefix: 'Ape',
        compName: 'VListFixFocus',
        parentComp: null,
        /*类本身配置参数*/
        //只能设置为'once'
        fetchDataType: 'once',
        //数据接口地址
        fetchDataUrl: '',
        //垂直列表数据对象数组
        data: null,
        //焦点在数据中的位置
        fixFocusPosInData: 0,
        //获取焦点项的文字
        getActiveItemText: function () {
            var activeData = this.getActiveData();
            return activeData[this.marqueeTextProp];
        },
        //容器元素的id属性
        boxId: '',
        //容器元素的文档坐标x值
        boxLeft: 0,
        //容器元素的文档坐标y值
        boxTop: 0,
        //列表项宽度
        itemWidth: 0,
        //列表项高度
        itemHeight: 0,
        //列表项位置视图配置
        //@examle [{left:0,top:0},{}]
        view: [],
        //最大文字字数限制
        textLimit: 100,
        //超出最大文字字数限制部分的替代字符
        //默认为空
        moreReplacement: '',
        //配置滚动对象后自动开启文字滚动
        marqueeInst: null,
        //滚动文字对应于数据对象中的属性名称
        //如果属性名称不是默认'text'，必须手工配置
        marqueeTextProp: 'text',
        //可视区域可容纳的列表项长度
        visualLength: 8,
        //固定焦点在可视列表中的位置，起始位置为0
        //默认值为4
        fixFocusPosInVisual: 4,
        //todo:暂时没有用到fixMedian
        //焦点两侧列表项不对称时候，上或者下长度优先
        //默认下部长度优先，即默认值为'bottom'
        //bottom or top
        fixMedian: 'bottom',
        //列表项作为菜单标识
        isMenu: false,
        //需要按确认键才更新菜单项对应的内容
        selectForChange: false,
        //列表数据处理
        //@example
        /**
         [{
			text: 'text',
			callback: function (val) {
				return '<div class="inner">' + val + '</div>';
			}
		}]
         */
        model: [],
        //没有数据给出默认提示
        //todo:该功能没有实现
        emptyMsg: '',
        emptyImage: '',
        /**
         * userEvent类似自定义的事件触发时候执行
         * 可以配置执行自己的操作
         * @param {string} method 自定义事件名称 onInitComplete,onWidgetBlur,onWidgetFocus,onFocus,onBlur,onSelect
         *        ,onPreFetchData,fetchDataSuccess,onMenuBlur,onMenuFocus
         * @param {object} args 返回一个对象
         */
        userEvent: function (method, args) {
        },
        //数据结构可能不同，需要处理获取到的数据
        //默认给出一种栏目数据结构
        //注意：需要计算总页码数
        dataHandler: function (data) {
            var node = data && data.node;
            var ret = null;
            if (node) {
                ret = data[node];
            }
            if (!ret) {
                return null;
            }
            //处理ret，去掉空的对象
            for (var i = 0; i < ret.length; i++) {
                if (evm.util.common.isEmptyObject(ret[i])) {
                    ret.splice(i, 1);
                    i = i - 1;
                }
            }
            if (ret && ret instanceof Array) {
                //初始化分页
                if (this.fetchDataType === 'once') {
                    this.pageTotalNum = Math.ceil(ret.length / this.pageSize);
                }
            }
            return ret;
        },
        //用于表示组件是否可被选中状态
        enabled: false,
        //用户可定义的扩展属性
        ext: {},
        //组件处于焦点状态，默认非焦点状态
        isActive: false,
        //组件的容器DOM是否可见，默认为可见
        isVisible: true,
        //开启CSS3动画
        //可根据盒子性能或者不兼容选择关闭CSS3动画
        openCSS3Anim: true,
        //动画运动间隔时间
        duration: '300ms',
        //可视区间前一个位置
        prePos: {
            left: 0,
            top: -70
        },
        //可视区间后一个位置
        afterPos: {
            left: 0,
            top: 558
        },
        // 按键防御间隔
        keyDefenceDuration: 120
    };
    evm.ext.comp.VListFixFocus = VListFixFocus;
})(this);
