/**
 * 常用工具函数集合
 * 依赖/swatapps/ape/evm.js
 * 创建时间：2016-04-13
 * 版本：v0.0.1
 */
(function (window, undefined) {
	evm.namespace('evm.util.common');
	var c = evm.util.common;

	var config = {
		logSwitch: false /*默认关闭调试日志*/
		/**
		 * 默认日志模式complex，simple or complex
		 * 默认为复杂模式，日志对多有用户隐藏
		 * 复杂模式可能部分盒子出现问题，可以设置为简单模式
		 */
		, logMode: 'simple'
		, logLevel: 'debug' /*日志输出级别 debug,info,warn,error*/
	};
	//日志
	var logLevelMap = {
		debug: 0,
		info: 1,
		warn: 2,
		error: 3
	};
	var log = function (currentLogLevel, str) {
		var config = c.logger.config,
			logSwitch = config.logSwitch,
			logMode = config.logMode,
			logLevel = logLevelMap[config.logLevel],
			currentLevel = logLevelMap[currentLogLevel];

		if (!logSwitch) {//关闭日志
			return;
		}
		if (currentLevel < logLevel) {
			return;
		}
		if (logMode === 'simple') {
			var logElm = evm.$('log');
			if (!logElm) {
				logElm = evm.createElem('div', {
					id: 'log',
					style: 'position:absolute;left:50px;top:50px;font-size:22px;color:#000;'
				});
			}

			var newNode = document.createElement('p');
			if (logElm.firstChild) {
				logElm.insertBefore(newNode, logElm.firstChild);
			}
			else {
				logElm.appendChild(newNode);
			}
			newNode.innerText = str;
		}
		else if (logMode === 'complex') {
			evm.log(str);
		}
	};
	c.logger = {
		config: {
			logSwitch: config.logSwitch,
			logMode: config.logMode,
			logLevel: config.logLevel
		},
		//设置配置参数
		setConfig: function (logSwitch, logMode, logLevel) {
			if (logSwitch != null) {
				this.config.logSwitch = logSwitch;
			}
			if (logMode != null) {
				this.config.logMode = logMode;
			}
			if (logLevel != null) {
				this.config.logLevel = logLevel;
			}
		},
		debug: function (str) {
			str = '[debug]' + str;
			log('debug', str);
		},
		info: function (str) {
			str = '[info]' + str;
			log('info', str);
		},
		warn: function (str) {
			str = '[warn]' + str;
			log('warn', str);
		},
		error: function (str) {
			str = '[error]' + str;
			log('error', str);
		}
	};
	/**
	 * 调试日志
	 * 保留支持老版本
	 * @deprecated
	 * @param str
	 */
	c.log = function (str) {
		this.logger.info(str);
	};

	evm.util.common.logger.debug('common.js 日志启动正常');
	evm.util.common.logger.debug('common.js 加载执行开始');
	/**
	 * 日期格式化
	 * @param {Date} date
	 * @param {string} F 格式化字符串
	 * @returns {*|string}
	 */
	c.dateFormat = function (date, F) {
		try {
			F = F || 'YYYY-mm-dd HH:ii:ss';
			var o =
			{
				'm+': date.getMonth() + 1	//月
				, 'd+': date.getDate()		//日
				, 'H+': date.getHours()		//时
				, 'i+': date.getMinutes()	//分
				, 's+': date.getSeconds()	//秒
				, 'q+': Math.floor((date.getMonth() + 3) / 3)	//季
				, 'S+': date.getMilliseconds()	//毫秒
				, 'w+': ['0', '1', '2', '3', '4', '5', '6'][date.getDay()]
				, 'W+': ['日', '一', '二', '三', '四', '五', '六'][date.getDay()]
			};
			if (/(Y+)/.test(F)) F = F.replace(RegExp.$1, ( date.getFullYear() + '' ).substr(4 - RegExp.$1.length));
			for (var K in o) {
				if (new RegExp('(' + K + ')').test(F))
					F = F.replace(RegExp.$1
						, ( RegExp.$1.length == 1 ) ? ( o[K] ) : ( (( K == 'S+' ? '000' : '00' ) + o[K]).substr(('' + o[K]).length)));
			}
		}
		catch (e) {
			evm.log('日期格式化失败:' + e.message);
		}
		return F;
	};
	/**
	 * json字符串转换为json对象
	 * @param str
	 * @returns {*}
	 */
	c.str2json = function (str) {
		if (!evm.isString(str)) {
			return str;
		}
		var json = false;
		try {
			json = eval("(" + str + ")");
		}
		catch (e) {
			json = false;
		}
		return json;
	};
	/**
	 * 将对象转换为 Json 串
	 * @param O
	 * @returns {string}
	 */
	c.json2str = function (O) {
		// Key 不使用引号, 所以不能包含空格
		//if( JSON && JSON.stringify ) return JSON.stringify( O );
		var S = '{', sEnd = '}';
		if (c.isArray(O)) {
			S = '[';
			sEnd = ']';
		}
		for (var i in O) {
			S += i + ':';
			if (evm.isUndefined(O[i])) S += '""';
			else if (evm.isObject(O[i])) S += c.json2str(O[i]);
			else if (c.isArray(O[i])) S += c.json2str(O[i]);
			else if (evm.isString(O[i])) S += '"' + O[i] + '"';
			else S += O[i];
			S += ',';
		}
		S = S.substr(0, S.length - 1);
		S += sEnd;
		return S;
	};
	/**
	 * 获取查询字符串的值
	 * @param key
	 * @returns {string}
	 */
	c.getQueryString = function (key) {
		var searchStr = window.location.search.substr(1);
		var reg = new RegExp(key + '=([^=&]*)');
		var value = '';
		if (reg.test(searchStr)) {
			value = window.decodeURIComponent(searchStr.match(reg)[1]);
		}
		return value;
	};
	/**
	 * 判断是否为数组
	 * @param obj
	 * @returns {boolean}
	 */
	c.isArray = function (obj) {
		return Object.prototype.toString.call(obj) === '[object Array]';
	};
	/**
	 * 判断对象是否为空
	 * @param obj
	 */
	c.isEmptyObject = function (obj) {
		for (var name in obj) {
			return false;
		}
		return true;
	};
	/**
	 * 判断值是否为空
	 * @param value
	 * @returns {*}
	 */
	c.isEmptyValue = function (value) {
		var type;
		if (value == null) {
			return true;
		}
		type = Object.prototype.toString.call(value).slice(8, -1);
		//evm.log('evm.util.common.isEmptyValue->type:'+type);
		switch (type) {
			case 'String':
				return !value;
			case 'Array':
				return !value.length;
			case 'Object':
				return c.isEmptyObject(value);
			default://其他对象均视作非空
				return false;
		}
	};
	/**
	 * 删除字符串首尾空白符
	 * @param value
	 * @returns {*}
	 */
	c.trim = function (value) {
		if (evm.isString(value)) {
			var reg = /^\s+|\s+$/g;
			return value.replace(reg, '');
		}
		return value;
	};
	/**
	 * 修改jQuery.extend
	 * 功能类似evm.extend
	 * 调用：evm.util.common.extend(true,target,source);
	 */
	c.extend = function () {
		var src, copyIsArray, copy, name, options, clone,
			target = arguments[0] || {},
			i = 1,
			length = arguments.length,
			deep = false;

		// Handle a deep copy situation
		if (typeof target === "boolean") {
			deep = target;
			target = arguments[1] || {};
			// skip the boolean and the target
			i = 2;
		}

		// Handle case when target is a string or something (possible in deep copy)
		if (typeof target !== "object" && !evm.isFunction(target)) {
			target = {};
		}

		// if only one argument is passed
		if (length === i) {
			target = {};
			--i;
		}

		for (; i < length; i++) {
			// Only deal with non-null/undefined values
			if ((options = arguments[i]) != null) {
				// Extend the base object
				for (name in options) {
					src = target[name];
					copy = options[name];

					// Prevent never-ending loop
					if (target === copy) {
						continue;
					}

					// Recurse if we're merging plain objects or arrays
					if (deep && copy && ( (copyIsArray = c.isArray(copy)) || evm.isObject(copy))) {
						if (copyIsArray) {
							copyIsArray = false;
							clone = src && c.isArray(src) ? src : [];

						} else {
							clone = src && evm.isObject(src) ? src : {};
						}

						// Never move original objects, clone them
						target[name] = c.extend(deep, clone, copy);

						// Don't bring in undefined values
					} else if (copy !== undefined) {
						target[name] = copy;
					}
				}
			}
		}

		// Return the modified object
		return target;
	};
	/*DOM操作*/
	c.show = function (el) {
		if (el) {
			el.style.display = 'block';
		}
	};
	c.hide = function (el) {
		if (el) {
			el.style.display = 'none';
		}
	};
	/**
	 * 调整函数执行的上下文
	 * @example
	 * evm.util.common.proxy(obj,function(){//this指向obj});
	 */
	c.proxy = function (context, fn) {
		context = context || window;

		return function () {
			return fn.apply(context, [].slice(arguments));
		};
	};

	/*持久化数据*/
	c.persist = {
		load: function (cookieName) {
			var cookieObj = null,
				cookieStr = evm.cookie(cookieName);

			if (cookieName) {
				evm.util.common.logger.info('cookieStr:' + cookieStr);
				cookieObj = c.str2json(cookieStr);
				evm.cutCookie(cookieName);
			}

			return cookieObj;
		},
		/**
		 * @param {string} cookieName cookie的名称
		 * @param {object|string} obj
		 */
		save: function (cookieName, obj) {
			if (cookieName && evm.isString(obj)) {
				evm.cookie(cookieName, obj);
				return true;
			}
			else if (cookieName && evm.isObject(obj)) {
				//组件数据持久化
				//需要注意对象的结构
				//json对象转换成字符串格式如：{mainView:{module1:{activeIndex:0,pageIndex:1},module2:{isActive:true,activeIndex:2,pageIndex:2}}}

				var jsonStr;
				try {
					jsonStr = c.json2str(obj);
					evm.util.common.logger.info('<evm.util.common.persist.save>cookie字符串: '+jsonStr);
				}
				catch (e) {
					evm.util.common.logger.error('<evm.util.common.persist.save>数据持久化失败，转换为json字符串报错');
				}

				if (jsonStr) {
					evm.cookie(cookieName, jsonStr);
					return true;
				}
				else {
					evm.util.common.logger.error('<evm.util.common.persist.save>数据持久化失败，obj->jsonStr为空');
				}
			}
			else {
				evm.util.common.logger.error('<evm.util.common.persist.save>数据持久化失败，第一个参数为cookieName,第二个参数为对象或者字符串类型');
			}
			return false;
		},
		//恢复组件状态
		recoverModuleState: function (cookieObj) {
			if ($WebApp && $WebApp.UI) {
				if (cookieObj && !c.isEmptyObject(cookieObj)) {
					for (var viewName in cookieObj) {
						var view = $WebApp.UI[viewName];
						var viewObj = cookieObj[viewName];

						if (view && !c.isEmptyObject(viewObj)) {
							for (var i = 0; i < view.module_array.length; i++) {
								var module = view.module_array[i];
								if (module) {
									var moduleInst = module.inst;
									var moduleName = module.moduleName;
									if (moduleInst && moduleName) {
										for (var moduleNameProp in viewObj) {
											var moduleObj = viewObj[moduleNameProp];
											if (moduleName === moduleNameProp && !c.isEmptyObject(moduleObj)) {
												evm.util.common.extend(true, moduleInst, moduleObj);
												break;
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	};
	evm.util.common.logger.debug('common.js 加载执行结束');
})(this);
