/**
 * 文字滚动插件
 * 返回一个Marquee类的实例
 * 依赖/swatapps/ape/evm.js
 * 创建时间：2016-03-12
 * 版本：v1.0
 *
 * 2016-04-12
 * 1.配置文件新增wrapClassName属性
 */
(function(window){
	function Marquee(options){
		//默认配置
		var defaults = {
			//控制滚动速度
			scrollAmount:2,
			//滚动文字字体大小
			fontSize:22,
			//滚动文字框元素的宽度
			width:0,
			//滚动文字框元素的高度
			height:30,
			prefix:'evm-marquee',
			//文字包含元素的className,默认为inner
			wrapClassName:'inner'
		};
		evm.extend(this,defaults,options);

		//私有属性
		this._timerId = null;
		this._intervalId = null;
		this._boxElm = null;
		this._displayElm = null;
		this._marqueeElm = null;

		this._text = '';
		this._oriText = '';
	}

	/**
	 * 初始化滚动DOM
	 * @param {object} parentNode 需要滚动文字的父元素
	 * @param text 现在展示的文字
	 * @param oriText 原始的文字
	 * @param {number} fontSize 文字大小
	 * @param {number} height 滚动框的高度
	 * @returns {*}
	 */
	Marquee.prototype._createDOM = function(parentNode,text,oriText){
		if(arguments.length !== 3){
			throw new Error('Marquee.prototype.createDOM需要3个参数');
		}
		if(!parentNode){
			return false;
		}
		else if(encodeURIComponent(text) === encodeURIComponent(oriText)){
			evm.html(parentNode,text);
			return false;
		}
		//设置滚动初始化标识
		var flag = parentNode.getAttribute('data-marquee');
		if(flag){
			this._boxElm = evm.getElementsByClassName(this.prefix+'-box',parentNode,'div')[0];
			this._displayElm = evm.getElementsByClassName(this.prefix+'-display',parentNode,'div')[0];
			this._marqueeElm = evm.getElementsByClassName(this.prefix+'-marquee',parentNode,'div')[0];
			return true;
		}
		else{
			//滚动相关节点未建立
			parentNode.setAttribute('data-marquee',1);
			//清空原来文字，建立自己的DOM结构
			evm.html(parentNode,'');

			//创建滚动包含框
			this._boxElm = evm.createElem('div',{
				className:this.prefix+'-box'
			},parentNode);
			evm.css(this._boxElm,{
				position:'relative',
				width:this.width+'px',
				height:this.height+'px',
				lineHeight:this.height+'px',
				overflow:'hidden',
				fontSize:this.fontSize
			});

			//展示文字框
			this._displayElm = evm.createElem('div',{
				className:this.prefix+'-display'
			},this._boxElm);
			evm.text(this._displayElm,text);

			//滚动元素框
			this._marqueeElm = evm.createElem('div',{
				className:this.prefix+'-marquee'
			},this._boxElm);
			evm.css(this._marqueeElm,{
				position:'absolute',
				left:0,
				top:0,
				width:(oriText.length + 8) * this.fontSize*2 + 'px',
				textAlgin:'left'
			});
			var className = this.prefix + '-text';
			evm.html(this._marqueeElm,'<div class="'+ className +'" style="display:inline-block;float:left;">' + oriText + '<b>&nbsp;&nbsp;&nbsp;&nbsp;</b></div><div class="'+ className +'" style="display:inline-block;float:left;">' + oriText + '<b>&nbsp;&nbsp;&nbsp;&nbsp;</b></div>')
			return true;
		}
	};
	Marquee.prototype.startScroll = function(parentNode,text,oriText){
		if(this._boxElm && this._marqueeElm && this._displayElm) {
			this.stopScroll();
		}

		var self = this;
		var success = this._createDOM(parentNode,text,oriText);
		if(success){
			evm.css(this._marqueeElm,{
				display:'block'
			});
			evm.css(this._displayElm,{
				display:'none'
			});

			var lineWidth;
			var marqueeTextElm = evm.getElementsByClassName(this.prefix + '-text', this._marqueeElm, 'div')[0];
			this._timerId = setInterval(function () {
				//todo clientWidth可能有兼容性问题
				var lineWidth = parseFloat(evm.getStyle(marqueeTextElm,'width')) || marqueeTextElm.clientWidth;
				if (lineWidth) {
					clearInterval(self._timerId);
					var left = parseFloat(evm.getStyle(self._marqueeElm,'left'));
					var step = -self.scrollAmount;
					self._intervalId = setInterval(function () {
						left += step;
						if (left <= -lineWidth) {
							var lineFirst = evm.getElementsByClassName(self.prefix + '-text', self._marqueeElm, 'div')[0];
							self._marqueeElm.appendChild(lineFirst);
							self._marqueeElm.style.left = 0;
							left = 0;
						}
						self._marqueeElm.style.left = left + 'px';
					}, 40);
				}
			}, 10);
		}
	};
	Marquee.prototype.stopScroll = function(){
		var self = this;
		if(this._boxElm && this._marqueeElm && this._displayElm){
			this._intervalId && clearInterval(this._intervalId);
			this._timerId && clearInterval(this._timerId);
			this._intervalId = null;
			this._timerId = null;
			this._marqueeElm && (
				evm.css(this._marqueeElm,{
					left:0
				})
			);

			evm.css(this._marqueeElm,{
				display:'none'
			});
			evm.css(this._displayElm,{
				display:'block'
			});

			this._boxElm = null;
			this._marqueeElm = null;
			this._displayElm = null;
		}
	};

	evm.namespace('evm.util.marquee');
	evm.util.marquee = function (options) {
		return new Marquee(options);
	}
})(this);
