/**
 * 播放器组件通用UI类
 * 依赖/swatapps/ape/evm.js
 * @author wuqq@evmtv.com
 * @version v1.0
 *
 * 注意:
 * 背景UI用repeat-x设置背景图片，发现显示或隐藏背景、音量和进度条拖动都很慢
 */
(function (window) {
	function MediaPlayUI(options) {
		//默认配置
		var defaults = {
			parent:document.body,
			//UI自动隐藏时间，默认为5s
			delayTime:5,
			//播放器UI容器设置
			bg: {
				left: 0,
				top: 620,
				width: 1280,
				height: 100,
				//背景图片，一般平铺
				//值为空表示不设置背景图片
				img: '/swatapps/ape/util/mediaPlayer/gcable/skin/default/bg.png'
			},
			//按键提示设置
			keyTip: {
				left: 0,
				top: 0,
				//默认为空，不设置按键提示
				img: ''
			},
			//进度条
			//首尾包含当前播放时间和总播放时间
			seeking: {
				switch: true,
				left: 68,
				top: 32,
				height:35,
				//进度条
				barWidth:895,
				barHeight:35,
				boxImg:'/swatapps/ape/util/mediaPlayer/gcable/skin/default/volumeBox.png',
				barImg: '/swatapps/ape/util/mediaPlayer/gcable/skin/default/volumeBar.png',
				//进度条指示点
				pointWidth:35,
				pointHeight:35,
				pointImg:'/swatapps/ape/util/mediaPlayer/gcable/skin/default/volumePoint.png'
			},
			//音量条
			volume: {
				switch: true,
				left: 191,
				top: 32,
				width: 895,
				height: 35,
				//音量调方向，V或者H，默认H-水平放置
				direction: 'H',
				boxImg: '/swatapps/ape/util/mediaPlayer/gcable/skin/default/volumeBox.png',
				barImg: '/swatapps/ape/util/mediaPlayer/gcable/skin/default/volumeBar.png',
				pointImg:'/swatapps/ape/util/mediaPlayer/gcable/skin/default/volumePoint.png',
				pointWidth:35,
				pointHeight:35,
				//音量最大显示数字，默认是100
				max:100
			},
			//静音图标
			mute: {
				switch: true,
				left: 138,
				top: 34,
				//开启静音图标
				onImg: '/swatapps/ape/util/mediaPlayer/gcable/skin/default/mute-on.png',
				//关闭静音图标
				offImg: '/swatapps/ape/util/mediaPlayer/gcable/skin/default/mute-off.png'
			},
			pace: {
				switch: true,
				//快进
				ff: {
					left: 0,
					top: 0,
					//每一个pace值有一个图片
					img: []
				},
				//快退
				fr: {
					left: 0,
					top: 0,
					img: []
				}
			},
			//延时隐藏UI时间，默认为5s
			delayHideSecond:5
		};
		evm.extend(this, defaults, options||{});
		//内部属性
		//播放器UI组件的顶层包含元素
		this._boxElm = null;
		//静音
		this._muteElm = null;
		//音量条
		this._volumeElm = null;
		//进度条
		this._seekingBoxElm = null;

		//定时器timerId
		this._timerId = -1;
		this.init();

		//声音调节延时timerId
		this._volumeTimerId = -1;

		//进度条延时timerId
		this._seekTimerId = -1;
	}

	evm.extend(MediaPlayUI.prototype, {
		_format:function(sec){
			var hh = '00';
			var mm = '00';
			var ss = '00';
			sec = parseFloat(sec);
			if(!isNaN(sec) && sec >= 0){
				hh = Math.floor(sec/3600);
				mm = Math.floor((sec - hh*3600)/60);
				ss = sec - hh*3600 - mm*60;

				hh = hh < 10 ? '0'+hh : hh;
				mm = mm < 10 ? '0'+mm : mm;
				ss = ss < 10 ? '0'+ss : ss;
			}
			return hh + ':' + mm + ':' + ss;
		},
		init: function () {
			//背景UI设置
			var bgStyle = 'position:absolute;'
				+ 'display:none;'
				+ 'z-index:1000;'
				+ 'left:' + this.bg.left + 'px;'
				+ 'top:' + this.bg.top + 'px;'
				+ 'width:' + this.bg.width + 'px;'
				+ 'height:' + this.bg.height + 'px;';
			if (this.bg.img) {
				bgStyle += 'background:url(' + this.bg.img + ') no-repeat left top;';
			}
			this._boxElm = evm.createElem('div', {className: 'mediaPlayer-ui', style: bgStyle},this.parent);
			//静音按钮
			if (this.mute.switch) {
				var muteStyle = 'position:absolute;'
					+ 'display:none;'
					+ 'left:' + this.mute.left + 'px;'
					+ 'top:' + this.mute.top + 'px;'
				this._muteElm = evm.createElem('img', {
					className: 'mediaPlayer-ui-mute',
					style: muteStyle
				}, this._boxElm);
				this._muteElm.src = this.mute.offImg;
			}
			//音量条
			if (this.volume.switch) {
				var volumeStyle = 'position:absolute;'
					+ 'display:none;'
					+ 'left:' + this.volume.left + 'px;'
					+ 'top:' + this.volume.top + 'px;'
					+ 'width:' + this.volume.width + 'px;'
					+ 'height:' + this.volume.height + 'px;'
					+ 'background:url(' + this.volume.boxImg + ') no-repeat center center;';
				this._volumeElm = evm.createElem('div', {
					className: 'mediaPlayer-ui-volume',
					style: volumeStyle
				}, this._boxElm);
				var volumeBarStyle = 'position:absolute;'
					+ 'width:' + this.volume.pointWidth + 'px;'
					+ 'height:' + this.volume.height + 'px;'
					+ 'background:url(' + this.volume.barImg + ') no-repeat left center;';
				this._volumeBarElm = evm.createElem('div', {
					className: 'mediaPlayer-ui-volumeBar',
					style: volumeBarStyle
				}, this._volumeElm);
				var volumePointStyle = 'position:absolute;'
					+ 'width:' + this.volume.pointWidth + 'px;'
					+ 'height:' + this.volume.pointHeight + 'px;'
					+ 'top:0px;'
					+ 'right:0px;'
					+ 'background:url(' + this.volume.pointImg + ') no-repeat right center;';
				this._volumePointElm = evm.createElem('div', {
					className: 'mediaPlayer-ui-volumePoint',
					style: volumePointStyle
				}, this._volumeBarElm);
				var volumeTxtStyle = 'position:absolute;'
					+ 'left:' + (this.volume.width + 10) + 'px;'
					+ 'top:0px;'
					+ 'width:100px;'
					+ 'height:' + this.volume.height + 'px;'
					+ 'line-height:' + this.volume.height + 'px;'
					+ 'font-size:21px;'
					+ 'color:#9fa4a8;'
					+ 'font-weight:bold;';
				this._volumeTxtElm = evm.createElem('div', {
					className: 'mediaPlayer-ui-volumeTxt',
					style: volumeTxtStyle
				}, this._volumeElm);
				evm.text(this._volumeTxtElm,'0');
			}
			//进度条
			if(this.seeking.switch){
				var seekingBoxStyle = 'position:absolute;'
					+ 'display:none;'
					+ 'left:' + this.seeking.left + 'px;'
					+ 'top:' + this.seeking.top + 'px;'
					+ 'width:' + (1280 - this.seeking.left) + 'px;'
					+ 'height:' + this.seeking.height + 'px;'
					+ 'line-height:' + this.seeking.height + 'px;'
					+ 'font-size:21px;'
					+ 'color:#9fa4a8;'
					+ 'font-weight:bold;';
				//创建父级元素
				this._seekingBoxElm = evm.createElem('div',{
					className:'mediaPlayer-ui-seeking',
					style:seekingBoxStyle
				},this._boxElm);
				//显示当前播放时间，默认为'00:00:00'
				var seekingElapsedTimeStyle = 'float:left;'
					+ 'padding-right:10px;';
				this._seekingElapsedTimeElm = evm.createElem('span',{
					className:'mediaPlayer-ui-elapsedTime',
					style:seekingElapsedTimeStyle
				},this._seekingBoxElm);
				evm.text(this._seekingElapsedTimeElm,'00:00:00');

				var seekingBarBoxStyle = 'float:left;'
					+ 'width:' + this.seeking.barWidth + 'px;'
					+ 'height:' + this.seeking.barHeight + 'px;'
					+ 'background:url( '+ this.seeking.boxImg +' ) no-repeat left center';
				this._seekingBarBoxElm = evm.createElem('span',{
					className:'mediaPlayer-ui-seekingBox',
					style:seekingBarBoxStyle
				},this._seekingBoxElm);
				var seekingBarStyle = 'position:relative;'
					+ 'float:left;'
					+ 'width:' + this.seeking.pointWidth + 'px;'
					+ 'height:' + this.seeking.barHeight + 'px;'
					+ 'background:url( '+ this.seeking.barImg +' ) no-repeat left center';
				this._seekingBarElm = evm.createElem('span',{
					className:'mediaPlayer-ui-seekingBar',
					style:seekingBarStyle
				},this._seekingBarBoxElm);
				var seekingPointStyle = 'position:absolute;'
					+ 'top:0;'
					+ 'right:0;'
					+ 'width:' + this.seeking.pointWidth + 'px;'
					+ 'height:' + this.seeking.pointHeight + 'px;'
					+ 'background:url( '+ this.seeking.pointImg +' ) no-repeat left center';
				this._seekingPointElm = evm.createElem('span',{
					className:'mediaPlayer-ui-seekingPoint',
					style:seekingPointStyle
				},this._seekingBarElm);
				//总播放时间
				var seekingTotalTimeStyle = 'float:left;'
					+ 'padding-left:10px;';
				this._seekingTotalTimeElm = evm.createElem('span',{
					className:'mediaPlayer-ui-totalTime',
					style:seekingTotalTimeStyle
				},this._seekingBoxElm);
				evm.text(this._seekingTotalTimeElm,'00:00:00');
			}
		},
		adjustVolume: function (volume,maxVolume) {
			var self = this;
			clearTimeout(this._volumeTimerId);
			//直接隐藏进度条
			this.hideSeek();
			this.showBox();
			this.showVolume();
			this.showMute();
			var min = 0,
				max = maxVolume || 32,
				percent = 0,
				w = 0;

			if(volume < 0){
				volume = 0;
			}
			else if(volume > max){
				volume = max;
			}
			if (volume >= 0 && volume <= max) {
				percent = parseInt(volume * this.volume.max/max);
				evm.text(this._volumeTxtElm,percent+'');
				w = (this.volume.width - this.volume.pointWidth) * (volume/max) + this.volume.pointWidth;
				this._volumeTimerId = setTimeout(function(){
					evm.css(self._volumeBarElm, {
						width: w + 'px'
					});
				},100);
			}
			this.delayHidePanel();
		},
		toggleMute: function (isMute) {
			//直接隐藏进度条
			this.hideSeek();
			this.showBox();
			this.showVolume();
			this.showMute();
			if (isMute) {
				this._muteElm.src = this.mute.onImg;
			}
			else {
				this._muteElm.src = this.mute.offImg;
			}
			this.delayHidePanel();
		},
		/**
		 *
		 * @param {number} time，当前时间
		 * @param {number} totalTime，总时间
		 * @param isSeeking 可选，表示拖动进度条操作
		 */
		seek:function(time,totalTime){
			var self = this;

			clearTimeout(this._seekTimerId);
			var w = 0;
			//直接隐藏声音相关元素
			this.hideVolume();
			this.hideMute();
			this.showSeek();
			this.showBox();

			if(!totalTime || !time){
				//未获取到总时长或当前时长直接返回
				return;
			}
			if(time < 0){
				time = 0;
			}
			else if(time > totalTime){
				time = totalTime;
			}
			if(time >= 0 && time <= totalTime){
				w = (this.seeking.barWidth - this.seeking.pointWidth) * (time/totalTime) + this.seeking.pointWidth;
				//log('w:'+w);
				this._seekTimerId = setTimeout(function(){
					evm.css(self._seekingBarElm, {
						width: w + 'px'
					});
				},100);

				this.updateElapsedTime(time);
			}
			this.delayHidePanel();
		},
		//更新进度条
		updateSeek:function(time,totalTime){
			var w = 0;
			if(!totalTime || !time){
				//未获取到总时长或当前时长直接返回
				return;
			}
			if(time >= 0 && time <= totalTime){
				w = (this.seeking.barWidth - this.seeking.pointWidth) * (time/totalTime) + this.seeking.pointWidth;
				evm.css(this._seekingBarElm, {
					width: w + 'px'
				});
				this.updateElapsedTime(time);
			}
		},
		//更新总时长
		updateTotalTime:function(totalTime){
			if(!totalTime){
				return;
			}
			var timeStr = this._format(totalTime);

			evm.text(this._seekingTotalTimeElm,timeStr);
		},
		//更新当前播放时长
		updateElapsedTime:function(elapsedTime){
			if(!elapsedTime){
				return;
			}
			var timeStr = this._format(elapsedTime);

			evm.text(this._seekingElapsedTimeElm,timeStr);
		},
		//延时隐藏播放器UI面板
		//arguments传入需要隐藏的元素
		delayHidePanel: function () {
			var self = this;
			this._timerId = setTimeout(function () {
				self.hideBox();
			}, this.delayTime*1000);
		},
		isBoxVisible:function(){
			return this._boxElm.style.display !== 'none';
		},
		hideBox:function(){
			this._boxElm.style.display = 'none';
		},
		showBox:function(){
			clearTimeout(this._timerId);
			evm.css(this._boxElm, {
				display:'block'
			});
		},
		hideMute:function(){
			evm.css(this._muteElm,{
				display:'none'
			});
		},
		showMute:function(){
			evm.css(this._muteElm,{
				display:'block'
			});
		},
		hideVolume:function(){
			if(!this._volumeElm){
				return;
			}
			evm.css(this._volumeElm,{
				display:'none'
			});
		},
		showVolume:function(){
			if(!this._volumeElm){
				return;
			}
			evm.css(this._volumeElm,{
				display:'block'
			});
		},
		hideSeek:function(){
			if(!this._seekingBoxElm){
				return;
			}
			evm.css(this._seekingBoxElm,{
				display:'none'
			});
		},
		showSeek:function(){
			if(!this._seekingBoxElm){
				return;
			}
			evm.css(this._seekingBoxElm,{
				display:'block'
			});
		}
	});
	evm.namespace('evm.util.mediaPlayer');
	evm.util.mediaPlayer.Ui = MediaPlayUI;
})(this);