/**
 * @file 下拉框
 * @import /swatapps/ape/evm.js /swatapps/ape/widget-v1/base.js
 * @param options
 * @constructor
 * @createTime:2015-11-18
 * @des
 * options
 *
 * @example
 *
 */
function Select(options){
    //组件DOM元素的容器id属性
    this._boxId = options.boxId;
    this._boxElement = evm.$(this._boxId);
    if(!this._boxElement){
        throw new Error('Select组件实例化必须传入容器id属性');
    }
    //构建class属性
    this._prefix = 'select';

    //下拉列表数据
    //{status:1,pageTotalNum:1,pageIndex:1,list:[{id:0,picBlur:'',text:''},{id:0,picBlur:'',text:''}]}
    this._data = options.data || null;

    //显示窗口DOM
    this._displayElement = null;
    this._defaultText = options.defaultText || '';
    this._value = this._defaultText;

    //下拉列表容器元素DOM
    this._ddlWrapper = null;

    //下拉框绝对定位的初始位置
    this._baseLeft = options.baseLeft || 0;
    this._baseTop = options.baseTop || 0;

    //创建选中的高亮背景层
    this._activeLayerElm;
    this._activeLayerDx = options.activeLayerDx;
    this._activeLayerDy = options.activeLayerDy;

    //下拉列表焦点位置
    //-1表示下拉列表关闭状态
    this._ddlActiveIndex = -1;
    //下拉列表
    //分页属性
    this._pageIndex = 1;
    this._pageSize = options.pageSize || 1;
    this._pageTotalNum = 1;
    //每页数据
    this._items = null;

    //配置接口方法
    this.onWidgetFocus = options.onWidgetFocus || function(){};
    this.onWidgetBlur = options.onWidgetBlur || function(){};
    this.doSelect = options.doSelect || function(){};
    this.setPager = options.setPager || function(){};//设置页码和翻页提示
    this.fetchItemsAsyn = options.fetchItemsAsyn;//异步获取分页数据，可选
}
extend(Select,KeyListenBase);
evm.extend(Select.prototype,{
    action:function(alias,event){
        switch(alias){
            case 'UP':
                if(this._ddlActiveIndex === -1){
                    this._blur();
                    this.onWidgetBlur(alias);
                }
                else{
                    this._move(-1);
                }

                break;
            case 'RIGHT':
                this._blur();
                this.onWidgetBlur(alias);
                break;
            case 'DOWN':
                if(this._ddlActiveIndex === -1){
                    this._blur();
                    this.onWidgetBlur(alias);
                }
                else{
                    this._move(1);
                }
                break;
            case 'LEFT':
                this._blur();
                this.onWidgetBlur(alias);
                break;
            case 'SELECT':
                this._doSelect();
                break;
            case undefined:
                this._focus();
                break;
        }
    },
    getValue:function(){
        return this._value;
    },
    init:function(data){
        //初始化dom
        this._boxElement.style.position = 'absolute';
        this._boxElement.style.left = this._baseLeft + 'px';
        this._boxElement.style.top = this._baseTop + 'px';

        this._displayElement = document.createElement('DIV');
        this._displayElement.style.position = 'absolute';
        this._displayElement.style.left = '0';
        this._displayElement.style.top = '0';
        evm.addClass(this._displayElement,this._prefix+'-display');
        this._boxElement.appendChild(this._displayElement);

        this._ddlWrapper = document.createElement('DIV');
        this._ddlWrapper.style.display = 'none';
        this._ddlWrapper.style.position = 'absolute';
        evm.addClass(this._ddlWrapper,this._prefix+'-ddlWrapper');
        this._boxElement.appendChild(this._ddlWrapper);
        var displayElementW = parseFloat(evm.getStyle(this._displayElement,'width'));
        var displayElementH = parseFloat(evm.getStyle(this._displayElement,'height'));
        this._ddlWrapper.style.top = displayElementH + 'px';
        this._ddlWrapper.style.width = displayElementW + 'px';

        //配置数据
        if(data) this._data = JSON.parse(data);
        if(!this._data){
            throw new Error('Select.init:列表数据不存在');
        }
        if(!this._data.list || this._data.length === 0){
            throw new Error('Select.init:列表数据为空');
        }

        this._data = this._data.list;
        this._pageTotalNum = this._data.pageTotalNum || Math.ceil(this._data.length/this._pageSize);

        this._fixPageIndex();
        var me = this;
        this._updateItems(function(){
            me._fixActiveIndex();
            me._create();

            if(typeof me.setPager === 'function'){
                me.setPager(me._pageIndex,me._pageTotalNum);
            }

            me.onWidgetFocus(me._activeIndex,me._items[me._activeIndex]);
        });
    },
    prev:function(){
        var pi = this._pageIndex;
        pi += -1;
        if(pi >= 1){
            this._blur();
            this._pageIndex = pi;

            var me = this;
            this._updateItems(function(){
                me._ddlActiveIndex = me._items.length - 1;
                me._create();
                me._focus();

                if(typeof me.setPager === 'function'){
                    me.setPager(me._pageIndex,me._pageTotalNum);
                }

                me.onWidgetFocus(me._activeIndex,me._items[me._activeIndex]);
            });
        }
        else{
            this._blur();
            this._ddlActiveIndex = -1;
            this._ddlWrapper.style.display = 'none';
            this._focus();
        }
    },
    next:function(){
        var pi = this._pageIndex;
        pi += 1;
        if(pi <= this._pageTotalNum){
            this._blur();
            this._pageIndex = pi;

            var me = this;
            this._updateItems(function(){
                me._ddlActiveIndex = 0;
                me._create();
                me._focus();

                if(typeof me.setPager === 'function'){
                    me.setPager(me._pageIndex,me._pageTotalNum);
                }

                me.onWidgetFocus(me._activeIndex,me._items[me._activeIndex]);
            });
        }
    },
    _blur:function(){
        if(this._ddlActiveIndex === -1){
            evm.removeClass(this._displayElement,'focus');
        }
        else{
            var ddl = evm.getElementsByClassName('select-ddl-item','div',this._ddlWrapper);
            var selected = ddl[this._ddlActiveIndex];
            if(selected){
                evm.removeClass(selected,'focus');
            }
        }

        this._activeLayerElm.style.display = 'none';
    },
    _create:function(){
        this._displayElement.innerHTML = '<span>'+ this._defaultText +'</span>';

        evm.removeChildren(this._ddlWrapper);

        for(var i = 0;i < this._items.length;i++){
            var tempElm = document.createElement('DIV');
            evm.text(tempElm,this._items[i].text);
            evm.addClass(tempElm,this._prefix+'-ddl-item');
            this._ddlWrapper.appendChild(tempElm);
        }

        //生成高亮背景层
        if(!this._activeLayerElm){
            this._activeLayerElm = document.createElement('div');
            this._activeLayerElm.style.display = 'none';
            this._activeLayerElm.style.position = 'absolute';
            this._activeLayerElm.className = this._boxId + '-' + this._prefix + '-activeLayer activeLayer';
            document.getElementsByTagName('body')[0].appendChild(this._activeLayerElm);
        }
    },
    _doSelect:function(){
        if(this._ddlActiveIndex === -1){
            this._ddlWrapper.style.display = 'block';
            this._blur();
            this._ddlActiveIndex = 0;
            var me = this;
            this._updateItems(function(){
                me._fixActiveIndex();
                me._create();
                me._focus();

                if(typeof me.setPager === 'function'){
                    me.setPager(me._pageIndex,me._pageTotalNum);
                }

                me.onWidgetFocus(me._activeIndex,me._items[me._activeIndex]);
            });
        }
        else{
            var item = this._items[this._ddlActiveIndex];
            this._displayElement.innerHTML = '<span>'+ item.text +'</span>';
            this._blur();
            this._ddlActiveIndex = -1;
            this._pageIndex = 1;
            this._ddlWrapper.style.display = 'none';
            this._value = item.text;
            this._focus();
        }
    },
    _fixActiveIndex:function(){
        this._activeIndex = Math.max(0,Math.min(this._items.length-1,this._activeIndex));
    },
    _fixPageIndex:function(){
        this._pageIndex = Math.max(1,Math.min(this._pageTotalNum,this._pageIndex));
    },
    _focus:function(){
        var pos,
            dim;
        if(this._ddlActiveIndex === -1){
            evm.addClass(this._displayElement,'focus');
            dim = this._getPos(this._displayElement);

            pos = evm.getElementPosition(this._displayElement);
        }
        else{
            var ddl = evm.getElementsByClassName('select-ddl-item','div',this._ddlWrapper);
            var selected = ddl[this._ddlActiveIndex];
            if(selected){
                evm.addClass(selected,'focus');
            }
            dim = this._getPos(selected);

            pos = evm.getElementPosition(selected);
        }

        this._activeLayerElm.style.width = dim.width + 'px';
        this._activeLayerElm.style.height = dim.height + 'px';
        this._activeLayerElm.style.left = pos.x + this._activeLayerDx + 'px';
        this._activeLayerElm.style.top = pos.y + this._activeLayerDy + 'px';
        this._activeLayerElm.style.display = 'block';

    },
    _getPos:function(elm){
        var o = {};
        o.left = parseFloat(evm.getStyle(elm,'left'));
        o.top = parseFloat(evm.getStyle(elm,'top'));
        o.width = parseFloat(evm.getStyle(elm,'width'));
        o.height = parseFloat(evm.getStyle(elm,'height'));
        return o;
    },
    _move:function(step){
        var ai = this._ddlActiveIndex;
        ai += step;

        if(ai >= 0 && ai <this._items.length){
            this._blur();
            this._ddlActiveIndex = ai;
            this._focus();
        }
        else if(ai < 0){
            this.prev();
        }
        else if(ai >= this._items.length){
            this.next();
        }
    },
    _updateItems:function(callback){
        var me = this;
        if(typeof this.fetchItemsAsyn === 'function' && typeof callback === 'function'){
            //todo:分页时异步获取数据，还没有测试
            this.fetchItemsAsyn(this._pageIndex,function(data){
                data = JSON.parse(data);
                me._data = data.list;
                me._pageTotalNum = data.pageTotalNum;
                me._items = me._data;
                callback();
            });
        }
        else{
            this._items = this._data.slice((this._pageIndex-1)*this._pageSize,this._pageIndex*this._pageSize);
            callback();
        }
    }
});