/**
 * @file 等大小网格列表，可以翻页
 * @import /swatapps/ape/evm.js /swatapps/ape/widget-v1/base.js
 * @param options
 * @constructor
 * @createTime:2015-11-24
 * @des
 * options
 *
 * @example
 *	new EqualGridList({
		boxId:'nav',
		data:navData,
		gridWidth:130,
		gridHeight:130,
		baseLeft:35,
		baseTop:20,
		gapH:0,
		gapV:0,
	})
 */
function EqualGridList(options) {
    //组件DOM元素容器的id属性
    this._boxId = options.boxId;
    this._boxElm = evm.$(this._boxId);
    if(!this._boxElm){
        throw new Error('EqualGridList实例化失败，未配置组件的容器元素');
    }
    //构建class属性
    this._prefix = 'equalGridList';
    //网格数据
    //{status:1,pageTotalNum:1,pageIndex:1,list:[{id:0,picBlur:'',text:''},{id:0,picBlur:'',text:''}]}
    this._data = options.data || null;
    this._menuFocusPageIndex = 1;
    this._menuActiveIndex = 0;
    //网格焦点索引位置
    this._activeIndex = 0;
    //配置自动分页或手动点击上下页按钮分页
    this._autoPaging = options.autoPaging === false ? false : true;
    //配置按键翻页 上下自动翻页V或左右自动翻页H，默认水平翻页
    this._pagingKeyDir = options.pagingKeyDir === 'V' ? 'V' : 'H';
    //分页属性
    this._pageIndex = 1;
    this._pageSize = 1;
    this._pageTotalNum = 1;
    //每页数据
    this._items = [];
    //网格大小
    this._gridWidth = options.gridWidth;
    this._gridHeight = options.gridHeight;
    //第一个网格左上角的文档坐标位置，用于绝对定位每个网格位置
    this._baseLeft = options.baseLeft || 0;
    this._baseTop = options.baseTop || 0;

    //网格水平和垂直间距
    this._gapH = options.gapH || 0;
    this._gapV = options.gapV || 0;

    //网格列数和行数
    this._cols = options.cols || 1;
    this._rows = options.rows || 1;

    //如果没有返回数据默认提示图片或文字
    this.defaultImage = options.defaultImage;
    this.defaultText = options.defaultText;

    //设置显示区域最大显示文字个数
    //超过显示文字个数，焦点选中时候滚动显示完整文本内容
    this._maxTextNum = options.maxTextNum || 1000;

    //配置接口方法
    //网格组件
    this.onWidgetInit = options.onWidgetInit;
    this.onWidgetFocus = options.onWidgetFocus;
    this.onWidgetBlur = options.onWidgetBlur;
    //每个网格项目
    this.onChangeStart = options.onChangeStart;
    this.onChangeEnd = options.onChangeEnd;
    this.onFocus = options.onFocus;
    this.onBlur = options.onBlur;
    this.doSelect = options.doSelect;
    //其他
    this.onFlipover = options.onFlipover;//设置页码和翻页提示
    this.fetchItemsAsyn = options.fetchItemsAsyn;//异步获取分页数据，可选
    this.onDestroy = options.onDestroy;
}
extend(EqualGridList, KeyListenBase);
evm.extend(EqualGridList.prototype, {
    action: function (alias,event) {
        var ai = this._activeIndex;
        switch (alias) {
            case 'UP':
                if (this._autoPaging && this._pagingKeyDir === 'V') {
                    if (ai - this._cols >= 0) {
                        this._onChangeStart();
                        this._activeIndex -= this._cols;
                        this._onChangeEnd();
                    }
                    else {
                        if(this._pageIndex - 1 >= 1){
                            this.prev(alias);
                        }
                        else{
                            this._onChangeStart();
                            this._onWidgetBlur(alias,event);
                        }
                    }
                }
                else {
                    if (ai - this._cols >= 0) {
                        this._onChangeStart();
                        this._activeIndex -= this._cols;
                        this._onChangeEnd();
                    }
                    else {
                        this._onChangeStart();
                        this._onWidgetBlur(alias,event);
                    }
                }
                break;
            case 'RIGHT':
                if (this._autoPaging && this._pagingKeyDir === 'H') {
                    if (ai % this._cols + 1 < this._cols && (ai + 1) < this._items.length) {
                        this._onChangeStart();
                        this._activeIndex++;
                        this._onChangeEnd();
                    }
                    else {
                        if (this._pageIndex === this._pageTotalNum && (ai + 1) < this._items.length) {
                            this._onChangeStart();
                            this._activeIndex++;
                            this._onChangeEnd();
                        }
                        else {
                            if(this._pageIndex + 1 <= this._pageTotalNum){
                                this.next(alias);
                            }
                            else{
                                this._onChangeStart();
                                this._onWidgetBlur(alias,event);
                            }

                        }
                    }
                }
                else {
                    if ((ai + 1) % this._cols !== 0 && (ai + 1) < this._items.length) {
                        this._onChangeStart();
                        this._activeIndex++;
                        this._onChangeEnd();
                    }
                    else {
                        this._onChangeStart();
                        this._onWidgetBlur(alias,event);
                    }
                }
                break;
            case 'DOWN':
                if (this._autoPaging && this._pagingKeyDir === 'V') {
                    if (ai + this._cols < this._items.length) {
                        this._onChangeStart();
                        this._activeIndex += this._cols;
                        this._onChangeEnd();
                    }
                    else {
                        if(this._pageIndex + 1 <= this._pageTotalNum){
                            this.next(alias);
                        }
                        else{
                            this._onChangeStart();
                            this._onWidgetBlur(alias,event);
                        }
                    }
                }
                else {
                    if (ai + this._cols < this._items.length) {
                        this._onChangeStart();
                        this._activeIndex += this._cols;
                        this._onChangeEnd();
                    }
                    else {
                        this._onChangeStart();
                        this._onWidgetBlur(alias,event);
                    }
                }
                break;
            case 'LEFT':
                if (this._autoPaging && this._pagingKeyDir === 'H') {
                    if (ai % this._cols - 1 >= 0) {
                        this._onChangeStart();
                        this._activeIndex--;
                        this._onChangeEnd();
                    }
                    else {
                        if (this._pageIndex === 1 && (ai - 1) >= 0) {
                            this._onChangeStart();
                            this._activeIndex--;
                            this._onChangeEnd();
                        }
                        else {
                            if(this._pageIndex - 1 >= 1){
                                this.prev(alias);
                            }
                            else{
                                this._onChangeStart();
                                this._onWidgetBlur(alias,event);
                            }
                        }
                    }
                }
                else {
                    if ((ai - 1) >= 0 && ai % this._cols !== 0) {
                            this._onChangeStart();
                            this._activeIndex--;
                            this._onChangeEnd();
                    } else {
                        this._onChangeStart();
                        this._onWidgetBlur(alias,event);
                    }
                }
                break;
            case 'SELECT':
                this._doSelect();
                break;
            default:
                this._onWidgetFocus();
                break;
        }
    },
    destroy: function () {
        this.removeDOM();

        //销毁数据
        this._data = null;
        this._items = [];

        this._trigger('onDestroy',[]);
    },
    getActiveElement:function(){
        var listItem = evm.getElementsByClassName('equalGridList-item','div',this._boxElm)[this._activeIndex];
        return listItem;
    },
    getActiveIndex: function () {
        return this._activeIndex;
    },
    getActiveItem:function(){
        return this._items[this._activeIndex];
    },
    getActiveElementPosition:function(){
        var boxElm = this._boxElm,
            divElm = boxElm.childNodes[this._activeIndex];

        var pos = evm.getElementPosition(divElm);
        return pos;
    },
    getBoxId:function(){
        return this._boxId;
    },
    getGridWidth:function(){
        return this._gridWidth;
    },
    getGridHeight:function(){
        return this._gridHeight;
    },
    getItems: function () {
        return this._items;
    },
    getMenuActiveIndex:function(){
        return this._menuActiveIndex;
    },
    getMenuFocusPageIndex:function(){
        return this._menuFocusPageIndex;
    },
    getPageIndex: function () {
        return this._pageIndex;
    },
    getPageSize: function () {
        return this._cols * this._rows;
    },
	getPageTotalNum:function(){
		return this._pageTotalNum;
	},
    init: function (data) {
        if (data) {
            this._data = evm.json.parse(data);
			this._initData();
        }
		else if(this._data){
			this._data = evm.json.parse(this._data);
			this._initData();
		}

        this._fixPageIndex();
        var me = this;
        this._updateItems(function () {
            me._fixActiveIndex();
			if (typeof me.onFlipover === 'function') {
				me.onFlipover(me._pageIndex, me._pageTotalNum);
			}
            me._create();

            me._trigger('onWidgetInit',[me._activeIndex,me._items[me._activeIndex]]);
        });
    },
    removeDOM:function(){
        //销毁dom对象
        var box = this._boxElm;
        evm.removeChildren(box);
    },
    setActiveIndex: function (nIndex) {
        this._activeIndex = nIndex;
    },
    setBaseTop:function(nVal){
        this._baseTop = nVal;
    },
    setMenuActiveIndex:function(nVal){
        this._menuActiveIndex = nVal;
    },
    //设置选中状态的页面
    setMenuFocusPageIndex:function(nVal){
        this._menuFocusPageIndex = nVal;
    },
    setPageIndex: function (nIndex) {
        this._pageIndex = nIndex;
    },
    prev: function () {
        var me = this;
        var pi = this._pageIndex;
        if (pi - 1 >= 1) {
            this._onChangeStart();
            this._pageIndex--;
            this._updateItems(function () {
				if (typeof me.onFlipover === 'function') {
					me.onFlipover(me._pageIndex, me._pageTotalNum,'prev');
				}
                me._create();
                //me._activeIndex = me._items.length - 1;
                //me._onChangeEnd();
            });
        }
    },
    next: function () {
        var me = this;
        var pi = this._pageIndex;
        if (pi + 1 <= this._pageTotalNum) {
            this._onChangeStart();
            this._pageIndex++;
            this._updateItems(function () {
				if (typeof me.onFlipover === 'function') {
					me.onFlipover(me._pageIndex, me._pageTotalNum,'next');
				}
                me._create();
                //me._activeIndex = 0;

                //me._onChangeEnd();
            });
        }
    },
    //视图展示可以继承该类重写
    _create: function () {
        var boxElm = this._boxElm,
            oPos,
            divElm,
            tempElm;

        this.removeDOM();

        for (var i = 0; i < this._items.length; i++) {
            divElm = document.createElement('DIV');
            divElm.className = this._prefix + '-' + 'item';
            divElm.style.position = 'absolute';
            divElm.style.width = this._gridWidth + 'px';
            divElm.style.height = this._gridHeight + 'px';

            //增加图片
            if (this._items[i].picBlur) {
                var imgElm = document.createElement('img');
                imgElm.src = this._items[i].picBlur;
                divElm.appendChild(imgElm);
            }

            if (this._items[i].text) {
                //增加文字背景层
                var overlayElm = document.createElement('div');
                overlayElm.className = 'overlay';
                divElm.appendChild(overlayElm);

                //增加文字结点
                var textElm = document.createElement('div');
                textElm.className = this._prefix + '-' + 'text';

                var text = this._items[i].text;
                if(text.length > this._maxTextNum){
                    text = text.substr(0,this._maxTextNum)+'...';
                    evm.addClass(divElm,'marquee');

                    var marqueeElm = document.createElement('MARQUEE');
                    evm.text(marqueeElm,this._items[i].text);
                    marqueeElm.style.display = 'none';
                    marqueeElm.style.width = (this._items[i].text.length+5)*22 + 'px';
                    textElm.appendChild(marqueeElm);
                }
                var spanElm = document.createElement('span');
                evm.text(spanElm,text);
                textElm.appendChild(spanElm);
                divElm.appendChild(textElm);
            }

            oPos = this._getPos(i);
            divElm.style.left = oPos.left + 'px';
            divElm.style.top = oPos.top + 'px';

            boxElm.appendChild(divElm);
        }
    },
    _fixActiveIndex: function () {
        this._activeIndex = Math.max(0, Math.min(this._items.length - 1, this._activeIndex));
    },
    _fixPageIndex: function () {
        this._pageIndex = Math.max(1, Math.min(this._pageTotalNum, this._pageIndex));
    },
    //初始化数据
    _initData: function () {
        if (!this._data) {
            throw new Error('EqualGridList:列表数据不存在');
        }
        if (!this._data.list || this._data.length === 0) {
            throw new Error('EqualGridList:列表数据为空或者未配置list属性');
        }
        if(this._data.pageTotalNum && !(parseInt(this._data.pageTotalNum) >= 1)){
            throw new Error('EqualGridList:列表数据未正确配置pageTotalNum属性');
        }
        if(this._data.pageIndex && !(parseInt(this._data.pageIndex) >= 1)){
            throw new Error('EqualGridList:列表数据未正确配置pageIndex属性');
        }
        this._pageSize = this._cols * this._rows;
        this._pageTotalNum = this._data.pageTotalNum ? this._data.pageTotalNum : Math.ceil(this._data.list.length / this._pageSize);
        if(this._data.pageIndex){
            this._pageIndex = this._data.pageIndex;
        }
        this._data = this._data.list;
    },
    _onWidgetBlur: function (alias) {
        this._trigger('onWidgetBlur',[alias]);

        if (this._isMenu) {
            var boxElm = document.getElementById(this._boxId),
                divElm = boxElm.childNodes[this._activeIndex];

            evm.addClass(divElm, 'focus');
        }
    },
    _onWidgetFocus:function(){
        var args = [this._activeIndex,this._items[this._activeIndex]];
        this._trigger('onWidgetFocus',args);
        this._onFocus();
    },
    _updateItems: function (callback) {
        var me = this;
        if (typeof this.fetchItemsAsyn === 'function' && typeof callback === 'function') {
            //todo:分页时异步获取数据，还没有测试
            this.fetchItemsAsyn(this._pageIndex, function (data) {
                data = evm.json.parse(data);
                if(data){
                    me._data = data;
                    me._initData();
                }
                me._items = me._data;
                callback();
            });
        }
        else {
            this._items = this._data.slice((this._pageIndex - 1) * this._pageSize, this._pageIndex * this._pageSize);
			callback();
        }
    },
    _getPos: function (nIndex) {
        var oPos = {};

        oPos.left = this._baseLeft + (nIndex % this._cols) * (this._gridWidth + this._gapH);
        oPos.top = this._baseTop + parseInt(nIndex / this._cols) * (this._gridHeight + this._gapV);
        return oPos;
    },
    _onChangeStart:function(){
        this._onBlur();
        this._trigger('onChangeStart',[this._activeIndex,this._items[this._activeIndex]]);
    },
    _onChangeEnd:function(){
        this._onFocus();
        this._trigger('onChangeEnd',[this._activeIndex,this._items[this._activeIndex]]);
    },
    _onFocus: function () {
        //修正焦点位置
        this._fixActiveIndex();

        var boxElm = this._boxElm,
            divElm = boxElm.childNodes[this._activeIndex];

        divElm && evm.addClass(divElm, 'focus');

        this._trigger('onFocus',[this._activeIndex,this._items[this._activeIndex]]);
    },
    _onBlur: function () {
        var boxElm = document.getElementById(this._boxId),
            divElm = boxElm.childNodes[this._activeIndex];

        divElm && evm.removeClass(divElm, 'focus');

        this._trigger('onBlur',[this._activeIndex,this._items[this._activeIndex]]);
    },
    _doSelect: function () {
        var item = this._items[this._activeIndex],
            id = item.id;

        this._trigger('doSelect',[this._activeIndex,item]);
    },
    //中文标点符号转为半角符号
    _chMask2Half: function (str) {
        //部分盒子上面替换后字符串出现乱码
        //暂时直接返回
        return str;

        return str.replace(/[，！。]/g, function (match) {
            switch (match) {
                case '，':
                    return ',';
                case '！':
                    return '!';
                case '。':
                    return '.';
            }
        });
    },
    //按照中文字符统计字符串长度
    _characterCountByCh: function (str) {
        var length = str.length;
        var realLength = 0
        var temLength = 0;
        for (var i = 0; i < length; i++) {
            var charCode = str.charCodeAt(i);
            if (charCode >= 0 && charCode <= 128) {
                temLength += 1;
            } else {
                realLength += 1;
            }
            if (temLength == 2) {
                realLength += 1;
                temLength = 0;
            }
        }
        return realLength;
    },
    _trigger:function(eventName,args){
        if(typeof this[eventName] === 'function'){
            this[eventName].apply(this,args);
        }
    }
});

