/**
 * @file 网格列表，相邻基础网格可以合并，可以翻页
 * @import /swatapps/ape/evm.js /swatapps/ape/widget-v1/base.js
 * @param options
 * @constructor
 * @createTime:2015-12-29
 * @des
 * options
 *
 * @example
 */
function MergGridList(options) {
	/*
	 * 私有属性
	 */
	//构建class属性
	this._prefix = 'mergGridList';

	/*
	 * 公有属性
	 */
    //组件Element容器的id属性
    this.boxId = options.boxId;
    this.boxElm = evm.$(this.boxId);
    if(!this.boxElm){
        this.boxElm = document.createElement('DIV');
		this.boxElm.id = this.boxId;
		var viewBox = evm.$('container') || document.getElementsByTagName('body')[0];
		viewBox.appendChild(this.boxElm);
    }

    //保存全部数据
    //{status:1,pageTotalNum:1,pageIndex:1,list:[{id:0,picBlur:'',text:''},{id:0,picBlur:'',text:''}]}
    this.data = options.data || null;
	//如果配置异步获取分页数据方法
	//需要配置该属性为false
	this.isNativeData = options.isNativeData || true;

	//标识组件是否为菜单
	this.isMenu = options.isMenu || false;
	//菜单选中项的所在页数和索引值
    this._menuPageIndex = 1;
    this._menuActiveIndex = 0;

    //组件的页面焦点位置索引
    this.activeIndex = options.activeIndex || 0;
    //配置自动分页或手动点击上下页按钮分页
    this.autoPaging = options.autoPaging === false ? false : true;
    //配置按键翻页 上下自动翻页V或左右自动翻页H，默认水平翻页
    this.pagingKeyDir = options.pagingKeyDir === 'V' ? 'V' : 'H';
    //分页属性
    this.pageIndex = options.pageIndex || 1;
    this.pageSize = options.pageSize || 1;
    this.pageTotalNum = 1;

    //保存当前页数据
    this._items = [];
    //网格单元的宽度和高度值
    this.gridWidth = options.gridWidth;
    this.gridHeight = options.gridHeight;
    //第一个网格左上角的文档坐标位置，用于绝对定位每个网格位置
    this.baseLeft = options.baseLeft || 0;
    this.baseTop = options.baseTop || 0;
	//网格单元对应的二维数组
	this.gridTwoDim = [];

    //网格水平和垂直间距
    this.gapH = options.gapH || 0;
    this.gapV = options.gapV || 0;

    //网格列数和行数
    this._cols = options.cols || 1;
    this._rows = options.rows || 1;

    //如果没有返回数据默认提示图片或文字
    this.defaultImage = options.defaultImage;
    this.defaultText = options.defaultText;

	//开启文字滚动 默认滚动关闭
	this.switchTextScroll = options.switchTextScroll || false;
    //设置显示区域最大显示文字个数
    //超过显示文字个数，焦点选中时候滚动显示完整文本内容
    this._maxTextNum = options.maxTextNum || 1000;
	//设置文本文字大小，默认22px
	this._fontSize = options.fontSize || 22;

    //配置接口方法
    //网格组件
    this.onWidgetInit = options.onWidgetInit;
    this.onWidgetFocus = options.onWidgetFocus;
    this.onWidgetBlur = options.onWidgetBlur;
    //每个网格项目
    this.onChangeStart = options.onChangeStart;
    this.onChangeEnd = options.onChangeEnd;
    this.onFocus = options.onFocus;
    this.onBlur = options.onBlur;
    this.doSelect = options.doSelect;
    //其他
    this.onFlipover = options.onFlipover;//设置页码和翻页提示
    this.fetchItemsAsyn = options.fetchItemsAsyn;//异步获取分页数据，可选
    this.onDestroy = options.onDestroy;
}
extend(MergGridList, KeyListenBase);
evm.extend(MergGridList.prototype, {
    action: function (alias,event) {
        var ai = this.activeIndex;
        switch (alias) {
            case 'UP':
				this._moveU(alias,event);
                break;
            case 'RIGHT':
                //边界条件判断
				this._moveR(alias,event);
                break;
            case 'DOWN':
				this._moveD(alias,event);
                break;
            case 'LEFT':
				this._moveL(alias,event);
                break;
            case 'SELECT':
                this._doSelect();
                break;
			case 'BACK':
				this._onWidgetBlur(alias,event);
				break;
			case undefined:
                this._onWidgetFocus();
                break;
        }
    },
	calGridSize:function(nIndex){
		var res = {};
		var gridItem = this._items[nIndex];
		if(gridItem){
			res.width = gridItem.sizeH*this.gridWidth + (gridItem.sizeH-1)*this.gapH;
			res.height = gridItem.sizeV*this.gridHeight + (gridItem.sizeV-1)*this.gapV;
		}
		else{
			res.width = 0;
			res.height = 0;
		}
		return res;
	},
	calPageTotalNum:function(){
		if(!this.data) return;
		var totalSize = 0;
		for(var i = 0;i < this.data.length;i++){
			var item = this.data[i];
			totalSize += item.sizeH*item.sizeV;
		}

		return Math.ceil(totalSize/(this._cols*this._rows));
	},
	//视图展示可以继承该类重写该方法
	create: function () {
		var boxElm = this.boxElm,
			oPos,
			divElm,
			tempElm;

		this.removeDOM();

		for (var i = 0; i < this._items.length; i++) {
			var gridWidth = this.calGridSize(i).width;
			var gridHeight = this.calGridSize(i).height;

			divElm = document.createElement('DIV');
			divElm.className = this._prefix + '-' + 'item';
			divElm.style.position = 'absolute';
			divElm.style.width = gridWidth + 'px';
			divElm.style.height = gridHeight + 'px';

			//增加图片
			if (this._items[i].picBlur) {
				var imgElm = document.createElement('img');
				imgElm.src = this._items[i].picBlur;
				imgElm.style.width = gridWidth + 'px';
				imgElm.style.height = gridHeight + 'px';
				divElm.appendChild(imgElm);
			}

			if (this._items[i].text) {
				//增加文字背景层
				var overlayElm = document.createElement('div');
				overlayElm.className = 'overlay';
				overlayElm.style.width = gridWidth + 'px';
				overlayElm.style.height = gridHeight + 'px';
				divElm.appendChild(overlayElm);

				//增加文字结点
				var textElm = document.createElement('div');
				textElm.className = this._prefix + '-' + 'text';
				textElm.style.width = gridWidth + 'px';
				textElm.style.height = gridHeight + 'px';

				var text = this._items[i].text;
				if(text.length > this._maxTextNum){
					text = text.substr(0,this._maxTextNum)+'...';
					evm.addClass(divElm,'marquee');
				}
				var spanElm = document.createElement('span');
				evm.text(spanElm,text);
				textElm.appendChild(spanElm);
				divElm.appendChild(textElm);
			}

			oPos = this._getPos(i);
			divElm.style.left = oPos.left + 'px';
			divElm.style.top = oPos.top + 'px';

			boxElm.appendChild(divElm);
		}
	},
    destroy: function () {
        this.removeDOM();

        //销毁数据
        this._data = null;
        this._items = [];

        this._trigger('onDestroy',[]);
    },
	//(r,c)下一个可以插入数组位置
	fillGridTwoDim:function(r,c,dr,dc,obj){
		var me = this;
		var canFill = true;

		function checkFill(){
			for(var i = c;i < c+dc;i++){
				for(var j = r;j < r+dr;j++){
					if(me.gridTwoDim[i][j].length){
						return false;
					}
				}
			}
			return true;
		}
		while(!(canFill = checkFill())){
			//水平查找
			c++;
			if(c >= this._cols){
				c = 0;
				r++;
			}

			if( r >= this._rows){
				break;
			}
		}
		if(canFill){
			for(var i = c;i < c+dc;i++){
				for(var j = r;j < r+dr;j++){
					this.gridTwoDim[i][j].push(obj);
				}
			}
			return {
				col:c,
				row:r,
				dr:dr,
				dc:dc
			}
		}
		else{
			return null;
		}
	},
    getActiveElement:function(){
        var listItem = evm.getElementsByClassName('mergGridList-item','div',this.boxElm)[this.activeIndex];
        return listItem;
    },
    getActiveItem:function(){
        return this._items[this.activeIndex];
    },
    getActiveElementPosition:function(){
        var divElm = this.getActiveElement();

        var pos = evm.getElementPosition(divElm);
        return pos;
    },
    getItems: function () {
        return this._items;
    },
    getMenuActiveIndex:function(){
        return this._menuActiveIndex;
    },
    getMenuFocusPageIndex:function(){
        return this._menuFocusPageIndex;
    },
	getPageTotalNum:function(){
		return this.pageTotalNum;
	},
	hide:function(left,top){
		if(left != null && top != null){
			this.boxElm.style.top = top + 'px';
			this.boxElm.style.left = left + 'px';
		}
		else{
			evm.hide(this.boxElm);
		}
	},
    init: function (data) {
		for(var i = 0;i < this._cols;i++){
			var arr = [];
			for(var j = 0;j < this._rows;j++){
				arr.push([]);
			}
			this.gridTwoDim.push(arr);
		}

        if (data) {
            this.data = evm.json.parse(data);
        }
		if(this.isNativeData){
			this._initData();
		}

		this._fixPageIndex();
        var me = this;
        this._updateItems(function () {
            me._fixActiveIndex();
			me._trigger('onFlipover',[me.pageIndex,me.pageTotalNum]);
            me.create();

			me._onChangeStart({type:'init',alias:undefined});
            me._trigger('onWidgetInit',[me.activeIndex,me._items[me.activeIndex]]);
			me._onChangeEnd({type:'init',alias:undefined});
        });
    },
	next: function (alias) {
		var me = this;
		var pi = this._pageIndex;
		if (pi + 1 <= this.pageTotalNum) {
			this._onChangeStart({type:'next',alias:alias});
			this._pageIndex++;
			this._updateItems(function () {
				me.create();
				if (typeof me.onFlipover === 'function') {
					me.onFlipover(me._pageIndex, me.pageTotalNum,'next');
				}

				me._onChangeEnd({type:'next',alias:alias});
			});
		}
	},
    removeDOM:function(){
        //销毁dom对象
        var box = this.boxElm;
        evm.removeChildren(box);
    },
	show:function(){
		evm.show(this.boxElm);
	},
    prev: function (alias) {
        var me = this;
        var pi = this.pageIndex;
        if (pi - 1 >= 1) {
            this._onChangeStart({type:'prev',alias:alias});
            this.pageIndex--;
            this._updateItems(function () {
                me.create();
				if (typeof me.onFlipover === 'function') {
					me.onFlipover(me.pageIndex, me.pageTotalNum,'prev');
				}

                me._onChangeEnd({type:'prev',alias:alias});
            });
        }
    },
    _fixActiveIndex: function () {
		if(this._items && this._items.length){
			this.activeIndex = Math.max(0, Math.min(this._items.length - 1, this.activeIndex));
		}
    },
    _fixPageIndex: function () {
		if(this.pageTotalNum){
			this.pageIndex = Math.max(1, Math.min(this.pageTotalNum, this.pageIndex));
		}
    },
    //初始化数据
    _initData: function () {
		var recData = this.data;
        if (!this.data) {
            throw new Error('EqualGridList:列表数据不存在');
        }
        if (!this.data.list || this.data.length === 0) {
            throw new Error('EqualGridList:列表数据为空或者未配置list属性');
        }
        if(this.data.pageTotalNum && !(parseInt(this.data.pageTotalNum) >= 1)){
            throw new Error('EqualGridList:列表数据未正确配置pageTotalNum属性');
        }
        if(this.data.pageIndex && !(parseInt(this.data.pageIndex) >= 1)){
            throw new Error('EqualGridList:列表数据未正确配置pageIndex属性');
        }
        this.pageSize = this.pageSize || (this.cols * this.rows);
        if(this.data.pageIndex){
            this.pageIndex = this.data.pageIndex;
        }
        this.data = this.data.list;
		//网格项大小数据字段
		for(var i = 0;i < this.data.length;i++){
			var item = this.data[i];
			var size = item.size;
			if(size){
				size = size.split('*');
				if(size.length !== 2){
					throw new Error('MergGridList._initData:网格项大小格式配置错误，格式为1*2,配置项实际配置为->'+item.size);
				}
				item.sizeH = size[0];
				item.sizeV = size[1];
				delete item.size;
			}
			else{
				item.sizeH = 1;
				item.sizeV = 1;
			}
		}
		this.pageTotalNum = recData.pageTotalNum ? parseInt(recData.pageTotalNum) : this.calPageTotalNum();
    },
	_moveD:function(alias,event){
		var ai = this.activeIndex;
		var pi = this.pageIndex;
		var activeItem = this.getActiveItem();
		var availableNext = this.gridTwoDim[activeItem.gridPosC][activeItem.gridPosR+1];
		if(availableNext && availableNext.length){
			availableNext = availableNext[0];
		}

		if(!availableNext){
			if(pi === this.pageTotalNum || !this.autoPaging){
				//超出边界
				this._onChangeStart({type:'widgetBlur',alias:alias});
				this._onWidgetBlur(alias,event);
			}
			else{
				if(this.pagingKeyDir === 'V'){
					this.next(alias);
				}

			}
		}
		else{
			this._onChangeStart({type:'inner',alias:alias});
			for(var i = 0;i < this._items.length;i++){
				if(this._items[i] === availableNext){
					this.activeIndex = i;
				}
			}
			this._onChangeEnd({type:'inner',alias:alias});
		}
	},
	_moveL:function(alias,event){
		var ai = this.activeIndex;
		var pi = this.pageIndex;
		var activeItem = this.getActiveItem();
		var availableNext = this._items[ai-1];

		if(!availableNext){
			if(pi === 1 || !this.autoPaging){
				//超出边界
				this._onChangeStart({type:'widgetBlur',alias:alias});
				this._onWidgetBlur(alias,event);
			}
			else{
				if(this.pagingKeyDir === 'H') {
					this.prev(alias);
				}
			}
		}
		else{
			if(pi === 1 && !this.isMenu || availableNext.gridPosR === activeItem.gridPosR){
				this._onChangeStart({type:'inner',alias:alias});
				this.activeIndex--;
				this._onChangeEnd({type:'inner',alias:alias});
			}
			else{
				if(this.pagingKeyDir === 'H') {
					this.prev(alias);
				}
			}
		}
	},
	_moveR:function(alias,event){
		var ai = this.activeIndex;
		var pi = this.pageIndex;
		var activeItem = this.getActiveItem();
		var availableNext = this._items[ai+1];

		if(!availableNext){
			if(pi === this.pageTotalNum || !this.autoPaging){
				//超出边界
				this._onChangeStart({type:'widgetBlur',alias:alias});
				this._onWidgetBlur(alias,event);
			}
			else{
				if(this.pagingKeyDir === 'H'){
					this.next(alias);
				}
			}
		}
		else{
			if(pi === this.pageTotalNum && !this.isMenu || availableNext.gridPosR === activeItem.gridPosR){
				this._onChangeStart({type:'inner',alias:alias});
				this.activeIndex++;
				this._onChangeEnd({type:'inner',alias:alias});
			}
			else{
				if(this.pagingKeyDir === 'H') {
					this.next(alias);
				}
			}
		}
	},
	_moveU:function(alias,event){
		var ai = this.activeIndex;
		var pi = this.pageIndex;
		var activeItem = this.getActiveItem();
		var availableNext = this.gridTwoDim[activeItem.gridPosC][activeItem.gridPosR-1];
		if(availableNext && availableNext.length){
			availableNext = availableNext[0];
		}

		if(!availableNext){
			if(pi === 1 || !this.autoPaging){
				//超出边界
				this._onChangeStart({type:'widgetBlur',alias:alias});
				this._onWidgetBlur(alias,event);
			}
			else{
				if(this.pagingKeyDir === 'V'){
					this.prev(alias);
				}

			}
		}
		else{
			this._onChangeStart({type:'inner',alias:alias});
			for(var i = 0;i < this._items.length;i++){
				if(this._items[i] === availableNext){
					this.activeIndex = i;
				}
			}
			this._onChangeEnd({type:'inner',alias:alias});
		}
	},
    _onWidgetBlur: function (alias,event) {
        this._trigger('onWidgetBlur',[alias,event]);

		//如果组件没有失去焦点，自动获取焦点
		if(this.isListened()){
			this._onFocus();
		}
    },
    _onWidgetFocus:function(){
        var args = [this.activeIndex,this._items[this.activeIndex]];
        this._trigger('onWidgetFocus',args);
        this._onFocus();
    },
    _updateItems: function (callback) {
		var me = this;
		function dealItems(items){
			var r = 0,c = 0;
			for(var i = 0;i < items.length;i++){
				var gridPos = me.fillGridTwoDim(r,c,+items[i].sizeV,+items[i].sizeH,items[i]);
				if(gridPos){
					items[i].gridPosR = r;
					items[i].gridPosC = c;


					c += gridPos.dc;
					if(c >= me._cols){
						c = c%me._cols;
						r++;
					}
				}
			}
		}
        var me = this;
        if (typeof this.fetchItemsAsyn === 'function' && typeof callback === 'function' && !this.isNativeData) {
			//该分支暂时不可用
            this.fetchItemsAsyn(this.pageIndex, function (data) {
                data = evm.json.parse(data);
				me.data = data;
				me._initData();

                me._items = me.data;
				dealItems(me._items);
                callback();
            });
        }
        else {
            this._items = this.data.slice((this.pageIndex - 1) * this.pageSize, this.pageIndex * this.pageSize);
			dealItems(this._items);
			callback();
        }
    },
    _getPos: function (nIndex) {
		var oPos = {};
		var len = this._items.length;
		if(nIndex < 0 || nIndex >= len){
			oPos.left = -10000;
			oPos.top = -10000;
			return oPos;
		}
		var item= this._items[nIndex];

		oPos.left = this.baseLeft + (item.gridPosC) * (this.gridWidth + this.gapH);
		oPos.top = this.baseTop + (item.gridPosR) * (this.gridHeight + this.gapV);
        return oPos;
    },
	//status {object}触发方法的状态
	//{type:'inner',alias:'UP'}
    _onChangeStart:function(status){
		if(status.type === 'inner' || status.type === 'widgetBlur'){
			this._onBlur();
		}
        this._trigger('onChangeStart',[status,this.activeIndex,this._items[this.activeIndex]]);
    },
    _onChangeEnd:function(status){
		if(status.type === 'inner' ){
			this._onFocus();
		}
        this._trigger('onChangeEnd',[status,this.activeIndex,this._items[this.activeIndex]]);
    },
    _onFocus: function () {
        //修正焦点位置
        this._fixActiveIndex();

        var divElm = this.getActiveElement();
        divElm && evm.addClass(divElm, 'focus');

        this._trigger('onFocus',[this.activeIndex,this._items[this.activeIndex]]);
    },
    _onBlur: function () {
		var divElm = this.getActiveElement();
        divElm && evm.removeClass(divElm, 'focus');

        this._trigger('onBlur',[this.activeIndex,this._items[this.activeIndex]]);
    },
    _doSelect: function () {
        var item = this._items[this.activeIndex],
            id = item.id;

		//设置实际焦点位置
		this._menuActiveIndex = this.activeIndex;
		this._menuFocusPageIndex = this.pageIndex;

        this._trigger('doSelect',[this.activeIndex,item]);
    },
    _trigger:function(eventName,args){
        if(typeof this[eventName] === 'function'){
            this[eventName].apply(this,args);
        }
    }
});

