
/**
 * @file 等大小网格列表，可以翻页
 * 依赖js文件：/swatapps/ape/evm.js /swatapps/ape/ext.evm.js
 * 创建日期：2016-01-11
 * @param {object} options
 * @constructor
 * @author wuqq@evmtv.com
 */

function EqualGridList(options) {
	/*
	 * 组件DOM元素容器的id属性
	 * @public
	 */
	this.boxId = options.boxId;
	this.boxElm = evm.$(this.boxId);
	if(!this.boxElm){
		this.boxElm = document.createElement('DIV');
		this.boxElm.id = this.boxId;
		var viewBox = evm.$('container') || document.getElementsByTagName('body')[0];
		viewBox.appendChild(this.boxElm);
	}
	/**
	 * 构建class属性的命名空间
	 * @type {string}
	 * @private
	 */
	this._prefix = 'equalGridList';
	/**
	 * 数据
	 * {pageTotalNum:1,pageIndex:1,list:[{id:0,picBlur:'',text:''},{id:0,picBlur:'',text:''}]}
	 * @type {*|null}
	 */
	this.data = options.data || null;
	/**
	 * 非异步获取分页数据标识为true
	 * 异步获取分页数据配置为false
	 * @type {boolean}
	 */
	this.isNativeData = options.isNativeData === false ? false : true;
	/**
	 * 组件是否为菜单
	 * 一般菜单为一行或者一列
	 * @type {boolean}
	 * @private
	 */
	this._isMenu = options.isMenu === true ? true : false;
	/**
	 * 内容对应的菜单栏目位置索引值
	 * @type {number|*}
	 */
	this.menuIndex = options.menuIndex || 0;
	/**
	 * 内容对应的菜单栏目位置所在的页码
	 * @type {number|*}
	 */
	this.menuPageIndex = options.menuPageIndex || 1;
	/**
	 * 焦点索引位置
	 * @type {number|*|MergGridList.activeIndex|ElementsWrapperForKey.activeIndex}
	 */
	this.activeIndex = options.activeIndex || 0;
	/**
	 * 配置自动分页或手动点击上下页按钮分页
	 * @type {boolean}
	 * @private
	 */
	this._autoPaging = options.autoPaging === false ? false : true;
	/**
	 * 配置按键翻页 上下自动翻页V或左右自动翻页H
	 * @type {string}
	 * @private
	 */
	this._pagingKeyDir = options.pagingKeyDir === 'V' ? 'V' : 'H';
	/**
	 * 当前页码
	 * 取值范围 >= 1
	 * @type {nameInput.data.pageIndex|*|number|MergGridList.pageIndex}
	 */
	this.pageIndex = options.pageIndex || 1;
	/**
	 * 每页显示项目数
	 * @type {number}
	 */
	this.pageSize = 1;
	/**
	 * 总页数
	 * @type {number}
	 */
	this.pageTotalNum = 1;
	/**
	 * 当前页数据
	 * @type {Array}
	 * @private
	 */
	this._items = [];
	/**
	 * 网格宽度
	 * @type {list.gridWidth|*|MergGridList.gridWidth}
	 * @private
	 */
	this.gridWidth = options.gridWidth;
	/**
	 * 网格高度
	 * @type {list.gridHeight|*|MergGridList.gridHeight}
	 */
	this.gridHeight = options.gridHeight;
	/**
	 * 第一个网格左上角的文档坐标left值
	 * @type {nameInput.baseLeft|*|list.baseLeft|addressInput.baseLeft|MergGridList.baseLeft|number}
	 * @private
	 */
	this.baseLeft = options.baseLeft || 0;
	/**
	 * 第一个网格左上角的文档坐标top值
	 * @type {nameInput.baseTop|*|list.baseTop|addressInput.baseTop|MergGridList.baseTop|number}
	 */
	this.baseTop = options.baseTop || 0;
	/**
	 * 网格水平间距
	 * @type {list.gapH|*|MergGridList.gapH|number}
	 */
	this.gapH = options.gapH || 0;
	/**
	 * 网格垂直间距
	 * @type {list.gapV|*|MergGridList.gapV|number}
	 */
	this.gapV = options.gapV || 0;
	/**
	 * 网格列数
	 * @type {*|number}
	 */
	this.cols = options.cols || 1;
	/**
	 * 网格行数
	 * @type {*|number}
	 */
	this.rows = options.rows || 1;
	/**
	 * 没有数据图片提示
	 * @type {*|MergGridList.defaultImage|EqualGridList.defaultImage}
	 */
	this.defaultImage = options.defaultImage;
	/**
	 * 没有数据文字提示
	 * @type {nameInput.defaultText|*|MergGridList.defaultText|EqualGridList.defaultText}
	 */
	this.defaultText = options.defaultText;
	/**
	 * 设置显示区域最大显示文字个数
	 * @type {evm.ext.comp.VListNoPaging.maxTextNum|*|number}
	 * @private
	 */
	this._maxTextNum = options.maxTextNum || 1000;
	/**
	 * 更多文字显示符号
	 * @type {*|EqualGridList.moreSymbol}
	 */
	this.moreSymbol = options.moreSymbol != null ? options.moreSymbol : '...';
	/**
	 * 设置文本文字大小
	 * @type {*|number}
	 */
	this.fontSize = options.fontSize || 22;
	/**
	 * 设置组件是否活动状态
	 * @type {boolean|*}
	 */
	this.isActiveModule = options.isActiveModule || false;
	/**
	 * 用于表示组件是否可被选中状态
	 * @type {boolean}
	 */
	this.enabled = options.enabled === false ? false : true;

	//配置接口方法
	//网格组件
	this.onWidgetInit = options.onWidgetInit;
	this.onWidgetFocus = options.onWidgetFocus;
	this.onWidgetBlur = options.onWidgetBlur;
	//每个网格项目
	this.onFocus = options.onFocus;
	this.onBlur = options.onBlur;
	this.doSelect = options.doSelect;
	//其他
	//设置页码和翻页提示
	this.onFlipover = options.onFlipover;
	//异步获取分页数据，可选
	this.fetchItemsAsyn = options.fetchItemsAsyn;
	this.onDestroy = options.onDestroy;
}
extend(EqualGridList, evm.KeyListenBase);
evm.extend(EqualGridList.prototype, {
	action: function (alias,event) {
		var ai = this.activeIndex;
		switch (alias) {
			case 'UP':
				this._moveUp(alias,event);
				break;
			case 'RIGHT':
				this._moveRight(alias,event);
				break;
			case 'DOWN':
				this._moveDown(alias,event);
				break;
			case 'LEFT':
				this._moveLeft(alias,event);
				break;
			case 'SELECT':
				this._doSelect();
				break;
			case 'BACK':
				this._onWidgetBlur(alias,event);
				break;
			case undefined:
				this._onWidgetFocus();
				break;
		}
	},
	blur: function () {
		var boxElm = document.getElementById(this.boxId),
			divElm = boxElm.childNodes[this.activeIndex];

		divElm && evm.removeClass(divElm, 'focus');

		this._trigger('onBlur',[this.activeIndex,this._items[this.activeIndex]]);
	},
	clearMenu:function(){
		var menuElms = evm.getElementsByClassName('menu','div',this.boxElm);
		for(var i = 0;i < menuElms.length;i++){
			evm.removeClass(menuElms[i],'menu');
		}
	},
	destroy: function () {
		this.removeDOM();

		//销毁数据或者初始化数据
		this.data = null;
		this._items = [];
		this.activeIndex = 0;
		this.pageIndex = 1;
		this.pageTotalNum = 1;

		this._trigger('onDestroy',[]);
	},
	focus: function () {
		//修正焦点位置
		this._fixActiveIndex();
		var divElm = this.getActiveElement();
		divElm && evm.addClass(divElm, 'focus');

		this._trigger('onFocus',[this.activeIndex,this._items[this.activeIndex]]);
	},
	getActiveElement:function(){
		var listItem = evm.getElementsByClassName('equalGridList-item','div',this.boxElm)[this.activeIndex];
		return listItem;
	},
	getActiveItem:function(){
		return this._items[this.activeIndex];
	},
	getActiveElementPosition:function(){
		var divElm = this.getActiveElement();

		var pos = evm.getElementPosition(divElm);
		return pos;
	},
	getItems: function () {
		return this._items;
	},
	getIsMenu:function(){
		return this._isMenu;
	},
	getMaxTextNum:function(){
		return this._maxTextNum;
	},
	getPageSize: function () {
		return this.cols*this.rows;
	},
	getPrefix:function(){
		return this._prefix;
	},
	init: function (data) {
		if (data) {
			this.data = evm.json.parse(data);
		}
		if(this.data){
			this._initData();
			this._fixPageIndex();
		}

		var me = this;
		this._updateItems('init',function () {
			me._fixActiveIndex();
			me.create();

			if(me.isActiveModule){
				me.addListener();
				this.isActiveModule = false;
			}
			me._trigger('onFlipover',[me.pageIndex, me.pageTotalNum]);
			me._trigger('onWidgetInit',[me.activeIndex,me._items[me.activeIndex]]);
		});
	},
	next: function (alias,event,callback) {
		var me = this;
		var pi = this.pageIndex;
		if (pi + 1 <= this.pageTotalNum) {
			this.blur();
			this.pageIndex++;
			this._updateItems('next',function () {
				me.create();
				me._trigger('onFlipover',[me.pageIndex, me.pageTotalNum,'next']);

				if(me._autoPaging){
					var activeIndex = me.activeIndex;
					var items = me.getItems();
					var firstInRow;
					if(me._pagingKeyDir === 'H'){
						firstInRow = Math.floor(activeIndex/me.cols) * me.cols;
					}
					else{
						firstInRow -= me.cols;
					}
					if(items[firstInRow]){
						me.activeIndex = firstInRow;
					}
					else{
						me.activeIndex = 0;
					}
					me.focus();
				}
				callback();
			});
		}
	},
	prev: function (alias,event,callback) {
		var me = this;
		var pi = this.pageIndex;
		if (pi - 1 >= 1) {
			this.blur();
			this.pageIndex--;
			this._updateItems('prev',function () {
				me.create();
				me._trigger('onFlipover',[me.pageIndex, me.pageTotalNum,'prev']);
				if(me._autoPaging){
					var items = me.getItems();
					var activeIndex = me.activeIndex;

					var lastInRow;
					if(me._pagingKeyDir === 'H'){
						lastInRow = (Math.floor(activeIndex/me.cols) + 1)*me.cols - 1;
					}
					else{
						activeIndex = activeIndex + me.cols;
					}
					if(items[lastInRow]){
						me.activeIndex = lastInRow;
					}
					me.focus();
				}
				callback();
			});
		}
	},
	removeDOM:function(){
		//销毁dom对象
		var box = this.boxElm;
		var children = evm.getElementsByClassName('equalGridList-item','*',box);
		for(var i = 0;i < children.length;i++){
			box.removeChild(children[i]);
		}
	},
	setAutoPaging:function(bVal){
		this._autoPaging = bVal;
	},
	/**
	 * 视图展示可以继承该类重写
	 */
	create: function () {
		var boxElm = this.boxElm,
			oPos,
			divElm,
			tempElm;

		this.removeDOM();

		var items = this.getItems();
		var prefix = this.getPrefix();
		var maxTextNum = this.getMaxTextNum();
		for (var i = 0; i < items.length; i++) {
			divElm = document.createElement('DIV');
			divElm.className = prefix + '-' + 'item';
			divElm.style.position = 'absolute';
			divElm.style.width = this.gridWidth + 'px';
			divElm.style.height = this.gridHeight + 'px';

			//增加图片
			if (items[i].picBlur) {
				var imgElm = document.createElement('img');
				imgElm.src = items[i].picBlur;
				divElm.appendChild(imgElm);
			}

			if (items[i].text) {
				//增加文字背景层
				var overlayElm = document.createElement('div');
				overlayElm.className = 'overlay';
				divElm.appendChild(overlayElm);

				//增加文字结点
				var textElm = document.createElement('div');
				textElm.className = prefix + '-' + 'text';

				var text = items[i].text + '';
				if(text.length > maxTextNum){
					text = text.substr(0,maxTextNum)+this.moreSymbol;
					evm.addClass(divElm,'marquee');
				}
				var spanElm = document.createElement('span');
				evm.text(spanElm,text);
				textElm.appendChild(spanElm);
				divElm.appendChild(textElm);
			}

			oPos = this.getPos(i);
			divElm.style.left = oPos.left + 'px';
			divElm.style.top = oPos.top + 'px';

			boxElm.appendChild(divElm);
		}
	},
	_fixActiveIndex: function () {
		if(this.activeIndex !== 0 && !this.activeIndex){
			this.activeIndex = 0;
		}
		this.activeIndex = Math.max(0, Math.min(this._items.length - 1, this.activeIndex));
	},
	_fixPageIndex: function () {
		this.pageIndex = Math.max(1, Math.min(this.pageTotalNum, this.pageIndex));
	},
	//初始化数据
	_initData: function () {
		if (!this.data) {
			evm.log.write(this.boxId+'._initData:列表数据不存在');
			throw new Error('EqualGridList:列表数据不存在');
		}
		if (!evm.isArray(this.data.list)) {
			evm.log.write(this.boxId+'._initData:data.list应该是数组');
			throw new Error('EqualGridList:data.list应该是数组');
		}
		if(this.data.pageTotalNum && !(parseInt(this.data.pageTotalNum) >= 1)){
			evm.log.write(this.boxId+'._initData:列表数据未正确配置pageTotalNum属性')
			throw new Error('EqualGridList:列表数据未正确配置pageTotalNum属性');
		}
		if(this.data.pageIndex && !(parseInt(this.data.pageIndex) >= 1)){
			evm.log.write(this.boxId+'._initData:列表数据未正确配置pageIndex属性');
			throw new Error('EqualGridList:列表数据未正确配置pageIndex属性');
		}
		this.pageSize = this.cols * this.rows;
		this.pageTotalNum = this.data.pageTotalNum ? parseInt(this.data.pageTotalNum) : Math.ceil(this.data.list.length / this.pageSize);
		if(this.data.pageIndex){
			this.pageIndex = parseInt(this.data.pageIndex);
		}
		if(isNaN(this.pageTotalNum)){
			this.pageTotalNum = 0;
		}
		if(isNaN(this.pageIndex)){
			this.pageIndex = 1;
		}
		if(isNaN(this.activeIndex)){
			this.activeIndex = 0;
		}

		this.data = this.data.list;
	},
	_moveDown:function(alias,event){
		var ai = this.activeIndex;
		if (ai + this.cols < this._items.length) {
			this.blur();
			this.activeIndex += this.cols;
			this.focus();
		}
		else {
			if(this.pageIndex + 1 <= this.pageTotalNum && this._autoPaging && this._pagingKeyDir === 'V'){
				this.next(alias);
			}
			else{
				this.blur();
				this._onWidgetBlur(alias,event);
			}
		}
	},
	_moveLeft:function(alias,event){
		var me = this;
		var ai = this.activeIndex;
		var evt = {};
		evt.status = 'LEFT';
		if (this._autoPaging && this._pagingKeyDir === 'H') {
			if (ai % this.cols - 1 >= 0) {
				this.blur();
				this.activeIndex--;
				this.focus();
			}
			else {
				if (this.pageIndex === 1 && (ai - 1) >= 0) {
					this.blur();
					this.activeIndex--;
					this.focus();
				}
				else {
					if(this.pageIndex - 1 >= 1){
						this.prev(alias,event);
					}
					else{
						this.blur();
						this._onWidgetBlur(alias,event);
					}
				}
			}
		}
		else {
			if ((ai - 1) >= 0 && ai % this.cols !== 0) {
				this.blur();
				this.activeIndex--;
				this.focus();
			} else {
				evt.callback = function(){
					if(me.activeIndex - 1 >= 0){
						me.blur();
						me.activeIndex--;
						me.focus();
					}
					evt = null;
				};
				this.blur();
				this._onWidgetBlur(alias,event,evt);
			}
		}
	},
	_moveRight:function(alias,event){
		var me = this;
		var ai = this.activeIndex;
		var evt = {};
		evt.status = 'RIGHT';
		if (this._autoPaging && this._pagingKeyDir === 'H') {
			if (ai % this.cols + 1 < this.cols && (ai + 1) < this._items.length) {
				this.blur();
				this.activeIndex++;
				this.focus();
			}
			else {
				if (this.pageIndex === this.pageTotalNum && (ai + 1) < this._items.length) {
					this.blur();
					this.activeIndex++;
					this.focus();
				}
				else {
					if(this.pageIndex + 1 <= this.pageTotalNum){
						this.next(alias);
					}
					else{
						this.blur();
						this._onWidgetBlur(alias,event);
					}

				}
			}
		}
		else {
			if ((ai + 1) % this.cols !== 0 && (ai + 1) < this._items.length) {
				this.blur();
				this.activeIndex++;
				this.focus();
			}
			else {
				evt.callback = function(){
					if ((me.activeIndex + 1) < me._items.length) {
						me.blur();
						me.activeIndex++;
						me.focus();
					}
					evt = null;
				};
				this.blur();
				this._onWidgetBlur(alias,event,evt);
			}
		}
	},
	_moveUp:function(alias,event){
		var ai = this.activeIndex;
		if (ai - this.cols >= 0) {
			this.blur();
			this.activeIndex -= this.cols;
			this.focus();
		}
		else {
			if(this.pageIndex - 1 >= 1 && this._autoPaging && this._pagingKeyDir === 'V'){
				this.prev(alias,event);
			}
			else{
				this.blur();
				this._onWidgetBlur(alias,event);
			}
		}
	},
	_onWidgetBlur: function (alias,event,evt) {
		this._trigger('onWidgetBlur',[alias,event,evt]);

		//如果组件没有失去焦点，自动获取焦点
		if(this.isListened()){
			if(evt){
				evt.callback();
			}
			this.focus();
		}
	},
	_onWidgetFocus:function(){
		var args = [this.activeIndex,this._items[this.activeIndex]];
		this._trigger('onWidgetFocus',args);

		this.focus();
	},
	_updateItems: function (type,callback) {
		var me = this;

		if(type === 'init' && this.data){
			this._items = this.data.slice((this.pageIndex - 1) * this.pageSize, this.pageIndex * this.pageSize);
			callback();
		}
		else{
			if (typeof this.fetchItemsAsyn === 'function' && typeof callback === 'function' && !this.isNativeData) {
				//todo:分页时异步获取数据，还没有测试
				this.fetchItemsAsyn(this.pageIndex, function (data) {
					data = evm.json.parse(data);
					if(data){
						me.data = data;
						me._initData();
					}
					me._items = me.data;
					callback();
				});
			}
			else{
				this._items = this.data.slice((this.pageIndex - 1) * this.pageSize, this.pageIndex * this.pageSize);
				callback();
			}
		}
	},
	getPos: function (nIndex) {
		var oPos = {};

		oPos.left = this.baseLeft + (nIndex % this.cols) * (this.gridWidth + this.gapH);
		oPos.top = this.baseTop + parseInt(nIndex / this.cols) * (this.gridHeight + this.gapV);
		return oPos;
	},
	_doSelect: function () {
		var item = this._items[this.activeIndex],
			id = item.id;

		this._trigger('doSelect',[this.activeIndex,item]);
	},
	_trigger:function(eventName,args){
		if(typeof this[eventName] === 'function'){
			this[eventName].apply(this,args);
		}
	}
});

