
/**
 *  @file 图文格子列表水平自动分页
 *  @import /swatapps/ape/evm.js
 *  @des 组件接口参考杨涛云组件定义规范
 *  todo:目前接口不支持分页数据，所以列表分页是一次读取所有数据:(
 */
(function(){
    evm.ext.comp.HGridPaging = function(options){
        //强制使用new
        if(this === evm.ext.comp || this === window){
            return new arguments.callee(options);
        }
        //必须传参
        if(evm.isUndefined(options) && !(evm.isObject(options))){
            throw new Error('evm.ext.comp.HGridPaging：没有传入参数');
        }
        function handler(options){
            // 触发事件的默认方法
            try{
                //console.log('VListBox 事件触发,默认处理方法');
            }catch(E){}
        }

        evm.ext.comp.BaseComponent.call(this,options);

        //====配置属性定义;=========================
        this.config = options;
        //自定义属性
        //容器元素的id属性
        this.box_id = this.config.boxID || this.comp_name;
        this._allRemoteData = this.config.allRemoteData;
        //列表元素容器的id属性
        this._listWrapId = this.box_id+'-list';
        //列表数据 {Array}
        /**
         *  {
         *      picBlur:'',
         *      picFocus:'',
         *      id:'',
         *      text:'',
         *      redirectUrl:''
         *  }
         */
        this.data = this.config.data;
        //列表项定位
        this._itemWidth = this.config.itemWidth;
        this._itemHeight = this.config.itemHeight;
        this._gapX = this.config.gapX || 10;
        this._gapY = this.config.gapY || 10;
        //分页属性
        this._pageSize = this.config.pageSize || 4;
        this._pageIndex = this.config.pageIndex || 1;
        this._pageTotalNum = 1;
        //单页列表数据属性
        this._items = null;
        //单页列表焦点索引
        this._activeIndex = this.config.activeIndex || 0;
        //列表项最大可显示字的数量
        this._maxTextNum = this.config.maxTextNum || 17;

        this._rowsNum = this.config.rowsNum || 3;
        this._colsNum = this.config.colsNum || 5;


        //====自定义事件的回调方法,在创建对象时传入对应的回调方法;=========================
        this.onBlur = this.config.onBlur || handler;
        this.onChange = this.config.onChange || handler;
        this.onChangeOut = this.config.onChangeOut || handler;
        this.onFocus = this.config.onFocus || handler;
        this.onFilter = this.config.onFilter || handler;
        this.onSelect = this.config.onSelect || handler;
        this.onEvent  = this.config.onEvent || handler;
    };
    evm.extend(evm.ext.comp.HGridPaging.prototype,{
        action:function(dir,keyEvent){
            var item = this._items[this._activeIndex],
                activeName = this._activeName;

            switch(dir){
                case 'UP':
                    this._onBlur();
                    if(this._activeIndex - this._colsNum >= 0){
                        this._activeIndex -= this._colsNum;
                        this._onFocus();
                        this.onChange(dir, this._activeIndex, this._allRemoteData);
                    }
                    else{
                        this.onBlur(dir);
                    }
                    break;
                case 'DOWN':
                    this._onBlur();
                    if(this._activeIndex + this._colsNum < this._items.length){
                        this._activeIndex += this._colsNum;
                        this._onFocus();
                        this.onChange(dir, this._activeIndex, this._allRemoteData);
                    }
                    else{
                        this.onBlur(dir);
                    }
                    break;
                case 'RIGHT':
                    this._onBlur();
                    if ((this._activeIndex % this._colsNum + 1 < this._colsNum) && (this._activeIndex + 1 < this._items.length)) {//单页切换栏目
                        this._activeIndex++;
                        this._onFocus();
                        this.onChange(dir, this._activeIndex, this._allRemoteData);
                    }
                    else {
                        if (this._pageIndex + 1 <= this._pageTotalNum) {//向下翻页
                            this._pageNext();
                            this._onFocus();
                            this.onChange(dir, this._activeIndex, this._allRemoteData);
                        }
                        else {//最后一页，跳出
                            this.onBlur(dir);
                        }
                    }
                    break;
                case 'LEFT':
                    this._onBlur();
                    if(this._activeIndex % this._colsNum - 1 >= 0){
                        this._activeIndex--;
                        this._onFocus();
                        this.onChange(dir,this._activeIndex,this._allRemoteData);
                    }
                    else{
                        if(this._pageIndex - 1 > 0){//向上翻页
                            this._pagePrev();
                            this._onFocus();
                            this.onChange(dir,this._activeIndex,this._allRemoteData);
                        }
                        else{//第一页，跳出
                            this.onBlur(dir);
                        }
                    }
                    break;
                case 'SELECT':
                    this._onSelect();
                    break;
                default:
                    this._onFocus();
                    this.onFocus(dir,this._activeIndex,this._allRemoteData);
                    break;
            }
            return this;
        },
        init:function(remoteData){
            remoteData && (this.data = remoteData);
            if(this.data && this.data.length > 0){
                this.setPageIndex(1);
                this.setActiveIndex(0);
                this._create();
            }
            return this;
        },
        destory:function(){
            var listElm = document.getElementById(this._listWrapId),
                pagingElm = document.getElementById(this._pagingWrapId);

            if(listElm){
                listElm.parentNode.removeChild(listElm);
            }
        },
        setPageIndex:function(pageIndex){
            var realPageIndex = Math.min(this._pageTotalNum || 0,Math.max(0,pageIndex));
            this._pageIndex = realPageIndex;
        },
        setActiveIndex:function(index){
            var realIndex = Math.min((this._items && this._items.length) || 0,Math.max(0,index));
            this._activeIndex = realIndex;
        },
        setAllRemoteData:function(data){
            this._allRemoteData = data;
        },
        _create:function(){
            var boxElm = document.getElementById(this.box_id),
                listElm = document.getElementById(this._listWrapId),
                liElm,
                liWidth,
                liHeight,
                liPosX,
                liPosY,
                item,
                spanElm,
                temp,
                imgElm;

            //配置分页数据
            this._pageTotalNum = Math.ceil(this.data.length/this._pageSize);
            this._items = this.data.slice((this._pageIndex-1)*this._pageSize,this._pageIndex*this._pageSize);

            if(listElm){
                listElm.parentNode.removeChild(listElm);
            }

            if(this._items && this._items.length){
                listElm = document.createElement('ul');
                listElm.id = this._listWrapId;
                boxElm.appendChild(listElm);
                //加载列表
                for(var i = 0;i < this._items.length;i++){
                    liElm = document.createElement('li');
                    //设置列表样式
                    liPosX = (i%this._colsNum)*(this._itemWidth+this._gapX);
                    liPosY = parseInt(i/this._colsNum)*(this._itemHeight+this._gapY);
                    evm.css(liElm,'left:'+liPosX+'px;top:'+liPosY+'px;');

                    item = this._items[i];
                    if(item.picFocus && item.picBlur){
                        imgElm = document.createElement('img');
                        imgElm.src = item.picBlur;
                        liElm.appendChild(imgElm);
                    }
                    if(item.text){
                        spanElm = document.createElement('span');
                        if(item.text.length > this._maxTextNum){
                            liElm.setAttribute('data-autoscroll','true');
                            evm.css(spanElm,'width:'+ item.text.length+'em');

                            evm.html(spanElm,item.text);
                        }
                        else{
                            evm.html(spanElm,item.text);
                        }
                        liElm.appendChild(spanElm);
                    }
                    listElm.appendChild(liElm);
                }
            }
        },
        _onFocus:function(){
            var listElm = document.getElementById(this._listWrapId),
                item = this._items[this._activeIndex];
            evm.addClass(listElm.childNodes[this._activeIndex],'active');

            //开启自动滚动文字
            this._stopAutoScroll(listElm.childNodes[this._activeIndex]);
            this._startAutoScroll(listElm.childNodes[this._activeIndex]);
        },
        _onBlur:function(){
            var listElm = document.getElementById(this._listWrapId),
                item = this._items[this._activeIndex];

            evm.removeClass(listElm.childNodes[this._activeIndex],'active');

            //取消自动滚动文字
            this._stopAutoScroll(listElm.childNodes[this._activeIndex]);
        },
        _pagePrev:function(){
            this._activeIndex = this._pageSize - 1;
            this._pageIndex--;
            this._create();
        },
        _pageNext:function(){
            this._activeIndex = 0;
            this._pageIndex++;
            this._create();
        },
        _onSelect:function(){
            this.onSelect(this._activeIndex,this._items,this._allRemoteData);
        },
        _startAutoScroll:function(itemWrapElm){
            this._autoScroll(itemWrapElm,0);
        },
        _stopAutoScroll:function(itemWrapElm){
            this._autoScroll(itemWrapElm,1);
        },
        _autoScroll:function(itemWrapElm,type){
            var me = this,
                itemElm = itemWrapElm,
                child = itemElm.firstChild,
                itemElmWidth,
                childWidth,
                interval = 30;

            if(!itemElm.getAttribute('data-autoscroll')){
                return;
            }

            itemElmWidth = itemElm.offsetWidth;
            childWidth = child.offsetWidth;

            function childScroll(){
                var left = child.offsetLeft;

                left = left - 1;

                if(left <= itemElmWidth - childWidth){
                    child.style.left = 0 + 'px';
                }
                else{
                    child.style.left = left + 'px';
                }
                if((left - 1) <= (itemElmWidth - childWidth)){
                    interval = 1000;
                }
                else{
                    interval = 20;
                }
                me._timeId = setTimeout(childScroll,interval);
            }

            if(type === 1){
                clearTimeout(me._timeId);
                child.style.left = 0 + 'px';
                return;
            }
            me._timeId = setTimeout(childScroll,interval);
        }
    });
})();