/**
 * @file 等大小网格列表，可以自动分页
 * @import /swatapps/ape/evm.js
 * @param options
 * @constructor
 */
function EqualGridListAutoPaging(options){
    //父元素id属性
    this._boxId = options.boxId;
    //网格数据
    this._data = null;
    //网格焦点索引位置
    this._activeIndex = 0;
    //分页属性
    this._pageIndex = 1;
    this._pageSize = 1;
    this._pageTotalNum = 1;
    //每页数据
    this._items = null;
    this._itemWidth = options.itemWidth;
    this._itemHeight = options.itemHeight;
    this._baseLeft = options.baseLeft || 0;
    this._baseTop = options.baseTop || 0;
    this._gapH = options.gapH || 0;
    this._gapV = options.gapV || 0;

    this._cols = options.cols || 1;
    this._rows = options.rows || 1;

    this._prevId = options.prevId || 'prev';
    this._nextId = options.nextId || 'next';

    this.onWidgetBlur = options.onWidgetBlur || function(){};
    this.doSelect = options.doSelect || function(){};
}
EqualGridListAutoPaging.prototype = {
    action:function(dir){
        var ai = this._activeIndex;
        switch(dir){
            case 'UP':
                if(ai - this._cols >= 0){
                    this._onBlur();
                    this._activeIndex -= this._cols;
                    this._onFocus();
                }
                break;
            case 'RIGHT':
                if(ai % this._cols + 1 < this._cols && (ai + 1) < this._items.length){
                    this._onBlur();
                    this._activeIndex++;
                    this._onFocus();
                }
                else{
                    if(this._pageIndex === this._pageTotalNum && (ai + 1) < this._items.length){
                        this._onBlur();
                        this._activeIndex++;
                        this._onFocus();
                    }
                    else{
                        this._next(dir);
                    }
                }
                break;
            case 'DOWN':
                if(ai + this._cols < this._items.length){
                    this._onBlur();
                    this._activeIndex += this._cols;
                    this._onFocus();
                }
                break;
            case 'LEFT':
                if(ai % this._cols - 1 >= 0){
                    this._onBlur();
                    this._activeIndex--;
                    this._onFocus();
                }
                else{
                    if(this._pageIndex === 1 && (ai - 1) >= 0){
                        this._onBlur();
                        this._activeIndex--;
                        this._onFocus();
                    }
                    else{
                        this._prev(dir);
                    }
                }
                break;
            case 'SELECT':
                this._doSelect();
                break;
            default:
                this._onFocus();
                break;
        }
    },
    init:function(data){
        data && (this._data = data);
        if(!data || data.length < 0){
            throw new Error('未获取列表数据!');
        }

        this._pageSize = this._cols*this._rows;
        this._pageTotalNum = Math.ceil(this._data.length/this._pageSize);

        this._fixPageIndex();
        this._updateItems();
        this._fixActiveIndex();
        this._create();
    },
    getActiveIndex:function(){
        return this._activeIndex;
    },
    setActiveIndex:function(nIndex){
        this._activeIndex = nIndex;
    },
    _fixActiveIndex:function(){
        this._activeIndex = Math.max(0,Math.min(this._items.length-1,this._activeIndex));
    },
    getPageIndex:function(){
        return this._pageIndex;
    },
    setPageIndex:function(nIndex){
        this._pageIndex = nIndex;
    },
    _fixPageIndex:function(){
        this._pageIndex = Math.max(1,Math.min(this._pageTotalNum,this._pageIndex));
    },
    _create:function(){
        var boxElm = document.getElementById(this._boxId),
            liElm,
            htmlStr,
            oPos;

        while(boxElm.firstChild){
            boxElm.removeChild(boxElm.firstChild);
        }
        for(var i = 0;i < this._items.length;i++){
            liElm = document.createElement('li');
            htmlStr = '<img src="' + this._items[i].picBlur + '" />';

            oPos = this._getPos(i);

            liElm.style.left = oPos.left + 'px';
            liElm.style.top = oPos.top + 'px';

            liElm.innerHTML = htmlStr;

            boxElm.appendChild(liElm);
        }
        this._updateBtnsStatus();
    },
    _updateItems:function(){
        this._items = this._data.slice((this._pageIndex-1)*this._pageSize,this._pageIndex*this._pageSize);
    },
    _getPos:function(nIndex){
        var oPos = {};

        oPos.left = this._baseLeft + (nIndex % this._cols)*(this._itemWidth+this._gapH);
        oPos.top = this._baseTop + parseInt(nIndex / this._cols)*(this._itemHeight+this._gapV);
        return oPos;
    },
    _onFocus:function(){
        var boxElm = document.getElementById(this._boxId),
            liElm = boxElm.childNodes[this._activeIndex],
            oPos = this._getPos(this._activeIndex);

        evm.addClass(liElm,'focus');
        //设置阴影效果
        if(!evm.$('list-shadow')){
            return;
        }
        evm.css(evm.$('list-shadow'), 'display:block');
        evm.css(evm.$('list-shadow'), 'left:' + (oPos.left-4) + 'px');
        evm.css(evm.$('list-shadow'), 'top:' + (oPos.top-4) + 'px');
    },
    _onBlur:function(){
        var boxElm = document.getElementById(this._boxId),
            liElm = boxElm.childNodes[this._activeIndex],
            oPos = this._getPos(this._activeIndex);

        evm.removeClass(liElm,'focus');
        if(!evm.$('list-shadow')){
            return;
        }
        evm.css(evm.$('list-shadow'), 'display:none');
    },
    _prev:function(dir){
        var pi = this._pageIndex;
        if(pi - 1 >= 1){
            this._onBlur();
            this._pageIndex--;
            this._updateItems();
            this._create();
            this._activeIndex = this._items.length - 1;
            this._onFocus();
        }
        else{
            this._onBlur();
            this.onWidgetBlur(dir);
        }
    },
    _next:function(dir){
        var pi = this._pageIndex;
        if(pi + 1 <= this._pageTotalNum){
            this._onBlur();
            this._pageIndex++;
            this._updateItems();
            this._create();
            this._activeIndex = 0;
            this._onFocus();
        }
        else{
            this._onBlur();
            this.onWidgetBlur(dir);
        }
    },
    _updateBtnsStatus:function(){
        var ptl = this._pageTotalNum,
            pi = this._pageIndex,
            prevElm = document.getElementById(this._prevId),
            nextElm = document.getElementById(this._nextId);

        prevElm.style.display = 'none';
        nextElm.style.display = 'none';

        if(pi - 1 > 0){
            prevElm.style.display = 'block';
        }
        if(pi + 1 <= ptl){
            nextElm.style.display = 'block';
        }
    },
    _doSelect:function(){
        var item = this._items[this._activeIndex],
            id = item.id;

        this.doSelect(id,item);
    }
};
