/**
 *  @file 竖向列表不含分页
 *  @import /swatapps/ape/evm.js
 *  @des 组件接口参考杨涛云组件定义规范
 */
(function(){
    evm.ext.comp.VListNoPaging = function(options){
        //强制使用new
        if(this === evm.ext.comp || this === window){
            return new arguments.callee(options);
        }
        //必须传参
        if(evm.isUndefined(options) && !(evm.isObject(options))){
            throw new Error('evm.ext.comp.VListNoPaging实例化失败：没有传入参数');
        }
        function handler(options){
            // 触发事件的默认方法
            try{
                //console.log('VListBox 事件触发,默认处理方法');
            }catch(E){}
        }

        evm.ext.comp.BaseComponent.call(this,options);

        //====配置属性定义;=========================
        this.config = options;
        //自定义属性
        //容器元素的id属性
        this.box_id = this.config.boxID || this.comp_name;
        this._allRemoteData = this.config.allRemoteData;
        //列表数据 {Array}
        /**
         *  {
         *      picBlur:'',
         *      picFocus:'',
         *      id:'',
         *      text:'',
         *      redirectUrl:''
         *  }
         */
        this.data = this.config.data;
        //容器中列表项的最大个数
        this.max_list_num = this.config.maxLists || 6;
        //列表项最大可显示字的数量
        this.maxTextNum = this.config.maxTextNum || 17;
        //焦点元素索引
        this._activeIndex = 0;

        //====自定义事件的回调方法,在创建对象时传入对应的回调方法;=========================
        this.onBlur = this.config.onBlur || handler;
        this.onChange = this.config.onChange || handler;
        this.onChangeOut = this.config.onChangeOut || handler;
        this.onFocus = this.config.onFocus || handler;
        this.onFilter = this.config.onFilter || handler;
        this.onSelect = this.config.onSelect || handler;
        this.onEvent  = this.config.onEvent || handler;
    }

    evm.extend(evm.ext.comp.VListNoPaging.prototype,{
        action:function(dir,keyEvent){
            switch(dir){
                case 'UP':
                    this._onBlur();
                    if(this._activeIndex - 1 >= 0){
                        this._activeIndex--;
                        this._onFocus();
                    }
                    else{
                        this.onBlur(dir);
                    }
                    break;
                case 'DOWN':
                    this._onBlur();
                    if(this._activeIndex + 1 < this.data.length){
                        this._activeIndex++;
                        this._onFocus();
                    }
                    else{
                        this.onBlur(dir);
                    }
                    break;
                case 'RIGHT':
                    this._onBlur();
                    this.onBlur(dir);
                    break;
                case 'LEFT':
                    this._onBlur();
                    this.onBlur(dir);
                    break;
                case 'SELECT':
                    this.onSelect(dir,this._activeIndex,this._allRemoteData);
                    break;
                default:
                    this._onFocus();
                    break;
            }
            return this;
        },
        init:function(remoteData){
            remoteData && (this.data = remoteData);
            if(this.data && this.data.length > 0){
                this.data = this.data.slice(0,this.max_list_num);
                this._create();
            }
            return this;
        },
        setActiveIndex:function(index){
            var realIndex = Math.min(this.data.length,Math.max(0,index));
            this._activeIndex = realIndex;
        },
        setActiveIndexLast:function(){
            this._activeIndex = this.data.length - 1;
        },
        getActiveIndex:function(){
            return this._activeIndex;
        },
        setAllRemoteData:function(data){
            this._allRemoteData = data;
        },
        getAllRemoteData:function(){
            return this._allRemoteData;
        },
        _create:function(){
            var me = this,
                boxElm = document.getElementById(this.box_id),
                prefix = 'vListNoPaging-item',
                listData = me.data,
                itemElm,
                item;
            for(var i = 0;i < listData.length;i++){
                item = listData[i];
                itemElm = document.createElement('div');
                evm.addClass(itemElm,prefix);
                itemElm.id = this.box_id + '-' + prefix + '-' + i;
                //列表背景有提供图片
                if(item.picBlur && item.picFocus){
                    evm.css(itemElm,'background-image:url('+ item.picBlur +')');
                    this.data[i].withPic = true;
                }
                else{
                    if(item.text.length > this.maxTextNum){
                        itemElm.setAttribute('data-autoscroll','true');
                        evm.html(itemElm,'<span style="width:'+ item.text.length +'em">'+item.text+'</span>');
                    }
                    else{
                        evm.html(itemElm,'<span>'+item.text+'</span>');
                    }
                }
                boxElm.appendChild(itemElm);
            }
        },
        _onFocus:function(){
            var item = this.data[this._activeIndex],
                prefix = this.box_id + '-' +'vListNoPaging-item-',
                itemElem = document.getElementById(prefix+this._activeIndex);

            if(item.withPic){
                evm.css(itemElem,'background-image:url('+ item.picFocus +')');
            }
            else{
                evm.addClass(itemElem,'active');
            }

            //开启自动滚动文字
            this._stopAutoScroll(itemElem);
            this._startAutoScroll(itemElem);
        },
        _onBlur:function(){
            var item = this.data[this._activeIndex],
                prefix = this.box_id + '-' +'vListNoPaging-item-',
                itemElem = document.getElementById(prefix+this._activeIndex);

            if(item.withPic){
                evm.css(itemElem,'background-image:url('+ item.picBlur +')');
            }
            else{
                itemElem.className = 'vListNoPaging-item';
                //evm.removeClass(itemElem,'active');
            }

            //取消自动滚动文字
            this._stopAutoScroll(itemElem);
        },
        _startAutoScroll:function(itemWrapElm){
            this._autoScroll(itemWrapElm,0);
        },
        _stopAutoScroll:function(itemWrapElm){
            this._autoScroll(itemWrapElm,1);
        },
        _autoScroll:function(itemWrapElm,type){
            var me = this,
                itemElm = itemWrapElm,
                child = itemElm.firstChild,
                itemElmWidth,
                childWidth,
                interval = 30;

            if(!itemElm.getAttribute('data-autoscroll')){
                return;
            }

            itemElmWidth = itemElm.offsetWidth;
            childWidth = child.offsetWidth;

            function childScroll(){
                var left = child.offsetLeft;

                left = left - 1;

                if(left <= itemElmWidth - childWidth){
                    child.style.left = 0 + 'px';
                }
                else{
                    child.style.left = left + 'px';
                }
                if((left - 1) <= (itemElmWidth - childWidth)){
                    interval = 1000;
                }
                else{
                    interval = 20;
                }
                me._timeId = setTimeout(childScroll,interval);
            }

            if(type === 1){
                clearTimeout(me._timeId);
                child.style.left = 0 + 'px';
                return;
            }
            me._timeId = setTimeout(childScroll,interval);
        }
    });
})();
